#include "EnhWaveView.hh"

using namespace std;
using namespace GSML;

/***************
 * ViewOverlay *
 ***************/

ViewOverlay::ViewOverlay (ViewOverlayManager* i_manager, Gdk_Pixmap* i_pixmap)
{
  m_manager = i_manager;
  m_pixmap = i_pixmap;
}

/***********************
 * WaveFormViewOverlay *
 **********************/

WaveFormViewOverlay::WaveFormViewOverlay (WavePlotTracer<MinMaxAvgWavePlot>* i_waveplot,
					  ViewOverlayManager* i_manager, Gdk_Pixmap* i_pixmap)
  : ViewOverlay (i_manager, i_pixmap)
{
  m_waveplot = i_waveplot;
  
  pcm_default type;
  
  waveform_dBfs = type.dBfs ();
}

WaveFormViewOverlay::~WaveFormViewOverlay ()
{
}

void WaveFormViewOverlay::Regenerate (const Area& n_size, sample_index real_start, sample_index real_end)
{
  area = n_size;
  
  waveform_middle = m_height / 2;
  waveform_vscale = waveform_dBfs / (waveform_middle - 10);
}

void WaveFormViewOverlay::Draw (const Area& area)
{
}

void WaveFormViewOverlay::EnterNotivication (GdkEventMotion* event)
{
}

void WaveFormViewOverlay::LeaveNotification ()
{
}

void WaveFormViewOverlay::MotionNotification (GdkEventMotion* event, GdkModifierType state,
					      sample_index real_pos)
{
}

void WaveFormViewOverlay::ButtonPress (GdkEventButton* event, sample_index real_pos)
{
}

void WaveFormViewOverlay::ButtonRelease (GdkEventButton* event, sample_index real_pos)
{
}

/*********************
 * MarkerViewOverlay *
 ********************/

MarkerViewOverlay::MarkerViewOverlay (ViewOverlayManager* i_manager, Gdk_Pixmap* i_pixmap)
{
}

MarkerViewOverlay::~MarkerViewOverlay ()
{
}

void MarkerViewOverlay::Regenerate (sample_index width, sample_index real_start, sample_index real_end)
{
}

void MarkerViewOverlay::Draw (const Area& area)
{
}

void MarkerViewOverlay::EnterNotivication (GdkEventMotion* event)
{
}

void MarkerViewOverlay::LeaveNotification ()
{
}

void MarkerViewOverlay::MotionNotification (GdkEventMotion* event, GdkModifierType state, sample_index)
{
}

void MarkerViewOverlay::ButtonPress (GdkEventButton* event, sample_index real_pos)
{
}

void MarkerViewOverlay::ButtonRelease (GdkEventButton* event, sample_index real_pos)
{
}


/****************
 * EnhTrackView *
 ****************/

EnhTrackView::EnhTrackView (WavePlotTracer<MinMaxAvgWavePlot>* i_plot, const WavePlotConfig* i_config,
			    EnhNormalView* i_parent, bool i_master = false)
{
  m_waveplot = i_plot;
  m_config = i_config;
  m_parent = i_parent;
  m_master = i_master;
  
  m_current_plot = m_waveplot->Plot ();
  
  m_width = 600;
  m_height = 50;
  
  m_start_delta = 0;
  m_internal_delta = 0;
  
  m_cache_length = m_width * SMOTH_SCROLL_FACTOR;
  
  set_events (GDK_EXPOSURE_MASK
	      | GDK_LEAVE_NOTIFY_MASK
	      | GDK_BUTTON_PRESS_MASK
	      | GDK_BUTTON_RELEASE_MASK
	      | GDK_POINTER_MOTION_MASK
	      | GDK_POINTER_MOTION_HINT_MASK);
  
  size (m_width, m_height);
  
  refresh_needed = false;
  refresh_all = false;
  
  m_waveplot->SignalWavePlotChanged.connect (slot (this, &EnhTrackView::SlotWavePlotChanged) );
  m_waveplot->SignalProgressUpdated.connect (slot (this, &EnhTrackView::SlotWavePlotUpdated) );
  
  m_current_sel = 0;
  m_current_markers = 0;
  
  connect_selection ();
  connect_markers ();
  
  m_view_overlays.push_back (new WaveFormViewOverlay (this, m_pixmap) );
  m_view_overlays.push_back (new MarkerViewOverlay (this, m_pixmap) );
}

EnhTrackView::~EnhTrackView ()
{
}

void EnhTrackView::SetStartDelta (int n_delta)
{
  m_start_delta = n_delta;
  int t_offset = m_start_delta - m_internal_delta;
  
  //cout << "t_offset: " << t_offset << endl;
  
  if (t_offset >= 0 && t_offset + m_width <= m_cache_length)
    {
      //cout << "smoth-scrolling: drawable doesn't need to be refreshed ..." << endl;
      
      win.draw_pixmap (back_gc,
		       rendered_pixmap,
		       t_offset, 0, // src
		       0, 0, // dest
		       m_width, m_height);
    }
  else {
    sample_index t_max_start = m_config->PlotLength () - m_cache_length;
    m_internal_delta = limit_max (limit_min (m_start_delta - (m_cache_length - m_width) / 2, 0),
				  t_max_start);
    
    add_refresh_all ();
  }
}

void EnhTrackView::SetTempSelection (sample_index n_start, sample_index n_end)
{
  if (!m_use_temp_sel) // first call
    {
      m_use_temp_sel = true;
      m_temp_sel_start = n_start;
      m_temp_sel_end = n_end;
    }
  
  sample_index t_start = m_temp_sel_start;
  sample_index t_end = m_temp_sel_end;

  m_temp_sel_start = n_start;
  m_temp_sel_end = n_end;
  
  // optimal refresh
  if (m_temp_sel_start != t_start) // changes of start
    {
      sample_index t_r_start = min (m_temp_sel_start, t_start);
      sample_index t_r_end = max (m_temp_sel_start, t_start);
      
      add_refresh_from_real (t_r_start, t_r_end);
    }
  if (m_temp_sel_end != t_end) // changes of end
    {
      sample_index t_r_start = min (m_temp_sel_end, t_end);
      sample_index t_r_end = max (m_temp_sel_end, t_end);
      cout << "refreshing: " << t_r_start << " - " << t_r_end << endl;
      add_refresh_from_real (t_r_start, t_r_end);
    }
}

void EnhTrackView::RemoveTempSelection ()
{
  if (m_use_temp_sel)
    {
      m_use_temp_sel = false;
      
      add_refresh_from_real (m_temp_sel_start, m_temp_sel_end);
    }
  else
    cout << "DEBUG: temp_selction was not used?!?" << endl;
}

void EnhTrackView::SetRefresh ()
{
  add_refresh_all ();
}

int EnhTrackView::Width ()
{
  return m_width;
}

int EnhTrackView::Height ()
{
  return m_height;
}

void EnhTrackView::Debug () const
{
  cout << "EnhWaveView::Debug:" << endl;
  cout << "  m_start_delta:    " <<  m_start_delta << endl
       << "  m_internal_delta: " << m_internal_delta << endl;
}

// view-overlay-manager

void EnhTrackView::RegisterSensitiveArea (const SensitiveArea& area)
{
}

void EnhTrackView::UnRegisterSensitiveArea (const SensitiveArea& area)
{
}

void EnhTrackView::DrawBackground (const Area& area, Priority priority)
{
}

void EnhTrackView::FinishedDrawing (const Area& area, Priority priority)
{
}

void EnhTrackView::add_refresh (int n_start, int n_end)
{
  SigC::Threads::MLock hack (refresh_lock);
  
  if (refresh_needed)
    {
      refresh_start = limit_max (n_start, refresh_start);
      refresh_end = limit_min (n_end, refresh_end);
    }
  else
    {
      refresh_start = n_start;
      refresh_end = n_end;
      refresh_needed = true;
    }
}

void EnhTrackView::add_refresh_from_real (sample_index n_start, sample_index n_end)
{
  sample_index t_start = m_config->Start ();
  sample_index t_sps = m_config->SamplesPerSample ();
  
  //cout << "add_refresh_from_real: " << n_start << " - " << n_end << endl;
  
  t_start += m_internal_delta * t_sps; // start of cached region
  
  if (n_end > t_start && n_start < t_start + m_cache_length * t_sps)
  {
    sample_index t_r_start = limit_min (n_start - t_start, 0) / t_sps;
    sample_index t_r_end = limit_max ( (n_end- t_start) / t_sps, m_cache_length);
    
    t_r_end = limit_min (t_r_end, t_r_start + 1);
    
    //cout << "  r4r: will refresh: " << t_r_start << " - " << t_r_end << endl;
    add_refresh (t_r_start, t_r_end);
  }
  else
    {
      //cout << "add_refresh_from_real: not in range ..." << endl;
    }
}   

void EnhTrackView::add_refresh_all ()
{
  SigC::Threads::MLock hack (refresh_lock);
  refresh_needed = true;
  refresh_all = true;
}

gint EnhTrackView::configure_event_impl (GdkEventConfigure* event)
{
  //cout << "EnhTrackView::configure_event_impl:" << endl;
  
  win = get_window ();
  visual = win.get_visual ();
  
  if (pixmap)
    pixmap.release();
  
  if (overlay_pixmap)
    overlay_pixmap.release();
  
  if (rendered_pixmap)
    rendered_pixmap.release();
  
  m_width = width ();
  m_height = height ();
  m_cache_length = m_width * SMOTH_SCROLL_FACTOR;
  
  pixmap.create (get_window (),  m_cache_length, m_height);
  overlay_pixmap.create (get_window (),  m_cache_length, m_height);
  rendered_pixmap.create (get_window (),  m_cache_length, m_height);
  
  if (!over_gc)
    {
      over_gc = Gdk_GC (pixmap);
      Gdk_Color color ("white");
      get_colormap ().alloc (color);
      over_gc.set_foreground (color);
      over_gc.set_function (GDK_INVERT);
    }
  
  m_parent->SlotChildConfigured (m_width);
  
  add_refresh_all ();
  
  return true;
}

int EnhTrackView::expose_event_impl (GdkEventExpose* event)
{
  // cout << "EnhTrackView::expose_event_impl:" << endl;
  
  int t_offset = m_start_delta - m_internal_delta;
  
  //cout << "EnhTrackView::expose_event_impl: src.x: " << event->area.x + t_offset
  //     << ", dest.x: " << event->area.x << endl;
  
  win.draw_pixmap (back_gc,
  		   rendered_pixmap,
  		   event->area.x + t_offset, event->area.y, // src
  		   event->area.x, event->area.y, // dest
  		   event->area.width, event->area.height);
  
  return true;
}

// this handles the mouse button events to set selections ...
gint EnhTrackView::button_press_event_impl (GdkEventButton* event)
{
  // cout << "EnhTrackView::button_press_event_impl:" << endl;
  
  m_parent->SlotButtenPress (event);
  
  return true;
}

gint EnhTrackView::button_release_event_impl (GdkEventButton* event)
{
  // cout << "EnhTrackView::button_release_event_impl:" << endl;
  
  m_parent->SlotButtenRelease (event);
  
  return true;
}

gint EnhTrackView::motion_notify_event_impl (GdkEventMotion* event)
{
  // cout << "EnhTrackView::motion_notify_event_impl:" << endl;
  
  int x, y;
  GdkModifierType state;
  Gdk_Window window (event->window);
  
  if (event->is_hint)
    {
      window.get_pointer (x, y, state);
    }
  else
    {
      x = (int) event->x;
      y = (int) event->y;
      state = (GdkModifierType) event->state;
    }
  
  m_parent->SlotMotionNotify (event, state);
  
  return true;
}

void EnhTrackView::draw_wave ()
{
  // cout << "trackView::draw_wave:" << endl;
  
  SigC::Threads::MLock hack (refresh_lock);
  
  if (!back_gc)
    {
      back_gc = Gdk_GC (pixmap);
      Gdk_Color color ("white");
      get_colormap ().alloc (color);
      back_gc.set_foreground (color);
    }
  if (!back_sel_gc)
    {
      back_sel_gc = Gdk_GC (pixmap);
      Gdk_Color color ("light grey");
      get_colormap ().alloc (color);
      back_sel_gc.set_foreground (color);
    }
  if (!back_temp_sel_gc)
    {
      back_temp_sel_gc = Gdk_GC (pixmap);
      Gdk_Color color ("light blue");
      get_colormap ().alloc (color);
      back_temp_sel_gc.set_foreground (color);
    }
  if (!back_undef_gc)
    {
      back_undef_gc = Gdk_GC (pixmap);
      Gdk_Color color ("orange");
      get_colormap ().alloc (color);
      back_undef_gc.set_foreground (color);
    }
  if (!wave_gc)
    {
      wave_gc = Gdk_GC (pixmap);
      Gdk_Color color ("blue");
      get_colormap ().alloc (color);
      wave_gc.set_foreground (color);
    }
  if (!wave_sel_gc)
    {
      wave_sel_gc = Gdk_GC (pixmap);
      Gdk_Color color ("dark blue");
      get_colormap ().alloc (color);
      wave_sel_gc.set_foreground (color);
    }
  if (!wave_temp_sel_gc)
    {
      wave_temp_sel_gc = Gdk_GC (pixmap);
      Gdk_Color color ("red");
      get_colormap ().alloc (color);
      wave_temp_sel_gc.set_foreground (color);
    }
  if (!grid_gc)
    {
      grid_gc = Gdk_GC (pixmap);
      Gdk_Color color ("dark grey");
      get_colormap ().alloc (color);
      grid_gc.set_foreground (color);
    }
  if (!marker_gc)
    {
      marker_gc = Gdk_GC (pixmap);
      Gdk_Color color ("dark red");
      get_colormap ().alloc (color);
      marker_gc.set_foreground (color);
    } 
 
  const MinMaxAvgWavePlot::Progress* t_progress = &(m_waveplot->Plot ()->GetProgress () );
  
  int t_width = limit_max (m_cache_length, t_progress->Length () );
  
  sample_index t_start;
  sample_index t_end;
  
  if (refresh_all) {
    t_start = 0;
    t_end = m_cache_length;
  } else {
    t_start = limit_max (refresh_start, m_cache_length);
    t_end = limit_max (refresh_end, m_cache_length);
  }
  
  // cout << "drawing (in cached pixmap): " << t_start << " - " << t_end << endl;
  
  // find first selection segmenz
  sample_index t_sps = m_config->SamplesPerSample ();
  sample_index t_real_sample = + m_config->Start () + (t_start + m_internal_delta) * t_sps;
  
  TrackLayer::Selection::const_iterator t_sel;
  TrackLayer::Selection::const_iterator t_sel_end;
  
  if (m_current_sel)
    {
      t_sel = m_current_sel->begin ();
      t_sel_end = m_current_sel->end ();
      
      while (t_sel != t_sel_end && t_sel->end < t_real_sample) {
	//cout << " this segment ends before current sample: " << t_sel->end << endl;
	++t_sel;
      }
    }
  //else
  //  cout << "DEBUG: no selection available?! - right??" << endl;
  
  DataLayer::Markers::const_iterator t_marker;
  DataLayer::Markers::const_iterator t_marker_end;
  
  if (m_current_markers)
    {
      t_marker = m_current_markers->begin ();
      t_marker_end = m_current_markers->end ();
      
      while (t_marker != t_marker_end && t_marker->position < t_real_sample) {
	//cout << " this marker is before current sample: name: "
	//     << t_marker->name << ", pos: " << t_marker->position << endl;
	++t_marker;
      }
    }
  //else
  //  cout << "DEBUG: no markers available?! - right??" << endl;
  
  prepare_waveform (t_progress);
  
  sample_index t_wave_plot_length = m_config->PlotLength ();
  
  sample_index t_v_grid_pos = t_start + (50 * t_sps - t_real_sample % (50 * t_sps) ) / t_sps;
  //cout << "first grid pos: " << t_v_grid_pos << ", my start: " << t_start << endl;
  
  for (sample_index i = t_start; i < t_end; ++i)
    {
      int plot_pos = i + m_internal_delta;
      sample_index t_real_sample_end = t_real_sample + t_sps;
      
      bool t_temp_sel = false;
      bool t_selected = false;
      bool t_marker_here = false;
      
      // temp selection ??
      if (m_use_temp_sel && m_temp_sel_start < t_real_sample_end && m_temp_sel_end >= t_real_sample)
	{
	  t_selected = true;
	  t_temp_sel = true;
	}
      else if (m_current_sel) // real selection ??
	{
	  if (t_sel != t_sel_end && t_sel->start < t_real_sample_end)
	    t_selected = true;
	}
      
      // marker ??
      if (m_current_markers)
	if (t_marker != t_marker_end && t_marker->position >= t_real_sample
	    && t_marker->position < t_real_sample_end)
	  t_marker_here = true;
      
      // draw background
      if (t_temp_sel)
	pixmap.draw_line (back_temp_sel_gc, i, 0, i, m_height);
      else if (t_selected)
	pixmap.draw_line (back_sel_gc, i, 0, i, m_height);
      else
	pixmap.draw_line (back_gc, i, 0, i, m_height);
      
      bool t_v_grid = i == t_v_grid_pos;
      
      draw_grid (i, grid_gc, t_v_grid);
	
      if (t_v_grid)
	t_v_grid_pos += 50;
      
      if (plot_pos < t_wave_plot_length)
	{
	  if ( t_progress->IsUseable (plot_pos) )
	    {
	      if (t_temp_sel)
		draw_waveform (i, wave_temp_sel_gc, plot_pos);
	      else if (t_selected)
		draw_waveform (i, wave_sel_gc, plot_pos);
	      else
		draw_waveform (i, wave_gc, plot_pos);
	    }
	}
      else
	{
	  cout << "BUG: EnhWaveView::draw_wave: plot_pos out of range!!!" << endl;
	  cout << "     plot_pos: " << plot_pos << ", t_wave_plot_length: " << t_wave_plot_length << endl;
	}
      
      /*
      else
	{
	  pixmap.draw_line (back_undef_gc, i, 0, i, m_height);
	}
      */
      
      if (t_marker_here)
	draw_marker (i, marker_gc);
      
      // next selection segment
      if (m_current_sel && t_sel != t_sel_end)
	if (t_sel->end <= t_real_sample_end)
	  while (t_sel != t_sel_end && t_sel->end <= t_real_sample_end)
	    ++t_sel;
      
      // next marker
      if (m_current_markers && t_marker != t_marker_end)
	if (t_marker->position < t_real_sample_end)
	  while (t_marker != t_marker_end && t_marker->position < t_real_sample_end)
	    ++t_marker;
      
      t_real_sample += t_sps;
    } 
  
  if (m_master)
    {
      /*
	Gdk_Font t_font;
	t_font.load ("-adobe-helvetica-bold-r-normal-*-13-*-*-*-p-*-iso8859-1");
	
	Gdk_GC text_gc;
	{
	text_gc = Gdk_GC (pixmap);
	Gdk_Color color ("black");
	get_colormap ().alloc (color);
	text_gc.set_foreground (color);
	}
	
	string text;
	text = "ZFactor: " + itos ( (int) t_sps) + ", s_delta: " + itos (m_start_delta);
	pixmap.draw_string (t_font, text_gc, 5, 25, text);
      */
    }
  
  
  // render to cache pixmap
  
  rendered_pixmap.draw_pixmap (back_gc,
			       pixmap,
			       t_start, 0, // src
			       t_start, 0, // dest
			       t_end - t_start, m_height);
  
  rendered_pixmap.draw_pixmap (over_gc,
			       overlay_pixmap,
			       t_start, 0, // src
			       t_start, 0, // dest
			       t_end - t_start, m_height);
  
  // something on the screen to redrawn ??
 
  // cout << "  processed region: " << t_start << " - " << t_end << endl;
 
  int t_offset = m_start_delta - m_internal_delta;
  // cout << "t_offset: " << t_offset << endl;
  
  t_start = limit_max (limit_min (t_start - t_offset, 0), m_width);
  t_end = limit_max (limit_min (t_end - t_offset, 0), m_width);
  
  int t_length = t_end - t_start;
  
  if (t_length > 0)
    {
      //cout << "  drawn region currently on-screen -> redraw ..." << endl
      //	   << "  temp values: " << t_start
      //	   << ", t_end: " << t_end
      //	   << ", t_length: " << t_length << endl;
      
      GdkRectangle update_rect;
      update_rect.x = t_start;
      update_rect.y = 0;
      update_rect.width = t_length;
      update_rect.height = m_height;
      
      draw (&update_rect);
    }
  
  refresh_needed = false;
  refresh_all = false;
}

void EnhTrackView::prepare_waveform (const MinMaxAvgWavePlot::Progress* n_progress)
{
  pcm_default type;
  
  waveform_progress = n_progress;
  
  waveform_middle = m_height / 2;
  waveform_dBfs = type.dBfs ();
  waveform_vscale = waveform_dBfs / (waveform_middle - 10);
}

void EnhTrackView::draw_waveform (int view_pos, Gdk_GC& gc, int plot_pos)
{
  // draw wave
  
  pcm_default::value_type min_v = waveform_progress->min_data [plot_pos];
  pcm_default::value_type max_v = waveform_progress->max_data [plot_pos];
  
  int min_value = min_v / waveform_vscale;
  int max_value = max_v / waveform_vscale;
  
  /* // fill wave to mid-line
  min_value = limit_max (min_value, 0);
  max_value = limit_min (max_value, 0);
  */
  pixmap.draw_line (gc,
		    view_pos, waveform_middle + min_value,
		    view_pos, waveform_middle + max_value);
}

void EnhTrackView::draw_grid (int view_pos, Gdk_GC& gc, bool draw_vert)
{
  pixmap.draw_point (gc, view_pos, waveform_middle -  waveform_dBfs / waveform_vscale);
  pixmap.draw_point (gc, view_pos, waveform_middle +  waveform_dBfs / waveform_vscale);
  if (draw_vert)
    pixmap.draw_line (gc,
		      view_pos, 0,
		      view_pos, m_height);
}

void EnhTrackView::draw_marker (int view_pos, Gdk_GC& gc)
{
  //cout << "drawwing marker ..." << endl;
  pixmap.draw_line (gc,
		    view_pos, 0,
		    view_pos, m_height);
}

void EnhTrackView::connect_selection ()
{
  if (m_current_sel)
    {
      connection_selection_selected.disconnect ();
      connection_selection_unselected.disconnect ();
      connection_selection_active_changed.disconnect ();
    }
  
  TrackLayer* t_track = m_current_plot->GetTrackLayer ();
  if (t_track)
    {
      m_current_sel = &(t_track->selection);
      
      connection_selection_selected 
	= m_current_sel->SignalSelected.connect (slot (this, &EnhTrackView::SlotSelectionSelected) );
      connection_selection_unselected
	= m_current_sel->SignalUnselected.connect (slot (this, &EnhTrackView::SlotSelectionUnselected) );
      connection_selection_active_changed
	= m_current_sel->SignalActiveChanged.connect (slot (this, &EnhTrackView::SlotSelectionActiveChanged) );
    }
  else
    {
      cout << "WARNING: Cant get Track ?!? (OK before first WavePlot::Config ...)" << endl;
      m_current_sel = 0;
    }
}

void EnhTrackView::connect_markers ()
{
  cout << "EnhTrackView::connect_markers:" << endl;
  if (m_current_markers)
    {
      cout << "  disconnecing:" << endl;
      connection_marker_added.disconnect ();
      connection_marker_deleted.disconnect ();
      connection_marker_changed.disconnect ();
    }
  
  DataLayer* t_layer = m_current_plot->GetDataLayer ();
  if (t_layer)
    {
      m_current_markers = &(t_layer->markers);
      
      cout << "  connecting:" << endl;
      connection_marker_added
	= m_current_markers->SignalAdded.connect (slot (this, &EnhTrackView::SlotMarkerAdded) );
      connection_marker_deleted
	= m_current_markers->SignalDeleted.connect (slot (this, &EnhTrackView::SlotMarkerDeleted) );
      connection_marker_changed
	= m_current_markers->SignalChanged.connect (slot (this, &EnhTrackView::SlotMarkerChanged) );
    }
  else
     {
      cout << "WARNING: Cant get Layer?!? (OK before first WavePlot::Config ...)" << endl;
      m_current_markers = 0;
     }
}


void EnhTrackView::SlotGUIRefresh ()
{
  // cout << "EnhTrackView::SlotGUIRefresh:" << endl;
  
  if (refresh_needed)
    {
      // cout << "EnhTrackView::SlotGUIRefresh: -> refreshing ..." << endl;
      draw_wave ();
    }
}

void EnhTrackView::SlotSelectionSelected (sample_index start, sample_index length)
{
  //cout << "EnhTrackView::SlotSelectionSelected:" << endl;
  
  add_refresh_from_real (start, start + length);
}

void EnhTrackView::SlotSelectionUnselected (sample_index start, sample_index length)
{
  //cout << "EnhTrackView::SlotSelectionUnselected:" << endl;
  
  add_refresh_from_real (start, start + length);
}

void EnhTrackView::SlotSelectionActiveChanged (bool n_state)
{
  //cout << "EnhTrackView::SlotSelectionActiveChanged:" << endl;
  add_refresh_all ();
}

void EnhTrackView::SlotMarkerAdded (const string& n_name, sample_index n_pos)
{
  //cout << "EnhTrackView::SlotMarkerAdded: n_pos: " << n_pos << endl;
  add_refresh_from_real (n_pos, n_pos + 1);
}
 
void EnhTrackView::SlotMarkerDeleted (const string& n_name, sample_index n_pos)
{
  //cout << "EnhTrackView::SlotMarkerDeleted: n_pos: " << n_pos << endl;
  add_refresh_from_real (n_pos, n_pos + 1);
}
  
void EnhTrackView::SlotMarkerChanged (const string& n_name, sample_index o_pos, sample_index n_pos)
{
  //cout << "EnhTrackView::SlotMarkerChanged: o_pos: " << o_pos << ", n_pos: " << n_pos << endl;
  add_refresh_from_real (o_pos, o_pos + 1);
  add_refresh_from_real (n_pos, n_pos + 1);
}

void EnhTrackView::SlotWavePlotChanged (MinMaxAvgWavePlot* n_plot)
{
  //cout << "EnhTrackView::SlotWavePlotChanged:" << endl;
  
  m_current_plot = n_plot;
  
  connect_selection ();
  connect_markers ();
  
  add_refresh_all ();
}

void EnhTrackView::SlotWavePlotUpdated (sample_index start, sample_index length)
{
  //cout << "EnhTrackView::SlotWavePlotUpdated: start: " << start << ", length: " << length << endl;
  if (length > 0) // we get 0 if more changed (in a too complex way (old_style) ...)
    {
      if (start <= m_internal_delta + m_cache_length && start + length > m_internal_delta)
	{
	  // cout << "update in cached region ..." << endl;
	  sample_index t_start = start - m_internal_delta;;
	  sample_index t_end = limit_max (start + length - m_internal_delta, m_cache_length);
	  
	  add_refresh (t_start, t_end);
	}
      //else
      //  cout << "update in not cached region ..." << endl;
    }
 else
   {
     SigC::Threads::MLock hack (refresh_lock);
     cout << "debug. got length 0 (old style) -> refreshing all ..." << endl;
     
     add_refresh_all ();
   }
}

EnhNormalView::EnhNormalView (WavePlotManager* i_manager, Signal0<void>* i_timeout)
  : m_adjustment (0, 0, 0)
{
  m_manager = i_manager;
  m_timeout = i_timeout;
  
  initialize ();
}

EnhNormalView::EnhNormalView (const EnhNormalView& src)
  : m_adjustment (0, 0, 0)
{
  m_manager = src.m_manager;
  m_timeout = src.m_timeout;
  
  initialize ();
  
  set_start (src.m_start);
  set_zoom (src.m_sps);
}

EnhNormalView::~EnhNormalView ()
{
  // TODO: nicer clean up
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    {
      // del_helper2 (i->plot); should be deleted by the wp-manager
      del_helper2 (i->view);
    }
}

void EnhNormalView::SetStart (sample_index n_start)
{
  m_auto_zoom_all = false;
  
  set_start (n_start);
}

sample_index EnhNormalView::Start ()
{
  return m_start;
}

void EnhNormalView::SetZoom (sample_index n_zoom_factor)
{
  m_auto_zoom_all = false;
  set_zoom (limit_min (n_zoom_factor, 1) );
}

void EnhNormalView::SetZoomAll ()
{
  sample_index t_length = m_manager->CurrentLayerPtr ()->Length ();
  
  m_auto_zoom_all = true;
  
  set_start (0);
  set_zoom (limit_min (m_layer_length / m_view_width, 1) );
}

int EnhNormalView::Zoom ()
{
  return m_sps;
}

sample_index EnhNormalView::Width ()
{
  return m_view_width;
}

sample_index EnhNormalView::Length ()
{
  return m_view_length;
}

void EnhNormalView::SetTrackVisible (track_index track_no, bool status)
{
  // TODO
}

bool EnhNormalView::IsTrackVisible (track_index track_no)
{
  // TODO
}

View* EnhNormalView::Clone ()
{
  EnhNormalView* t_view = new EnhNormalView (*this);
  return t_view;
}

void EnhNormalView::initialize ()
{
  m_start = 0;
  m_layer_length = m_manager->CurrentLayerPtr () -> Length ();
  m_plot_start = 0;
  m_plot_length = 1;
  m_cache_length = 1;
  m_view_width = 1;
  m_view_length = 1;
  m_sps = 1;
  
  m_view_start_delta = 0;
  m_auto_zoom_all = false;
  
  m_autoscroll = false;
  
  m_adjustment.changed.connect ( slot(this, &EnhNormalView::SlotAdjustmentChanged) );
  m_adjustment.value_changed.connect ( slot(this, &EnhNormalView::SlotAdjustmentValueChanged) );
  
  pack_start (m_hruler, false, false);
  pack_start (view_box);
  m_scrollbar.set_adjustment (m_adjustment);
  pack_start (m_scrollbar, false, false);
  
  // Make a popup window
  {
    using namespace Gtk;
    using namespace Gtk::Menu_Helpers;
    
    MenuList& list_popup = menu_popup.items();
    //list_popup.push_back (MenuElem ("Floa_t 1","<control>t", slot(&void_void_cb)));
    list_popup.push_back (MenuElem ("zoom in", slot (this, &EnhNormalView::slot_context_zoom_in) ) );
    list_popup.push_back (MenuElem ("zoom out", slot (this, &EnhNormalView::slot_context_zoom_out) ) );
    list_popup.push_back (SeparatorElem () );
    list_popup.push_back (MenuElem ("zoom 1:1", slot (this, &EnhNormalView::slot_context_zoom_one2one) ) );
    list_popup.push_back (MenuElem ("zoom all", slot (this, &EnhNormalView::slot_context_zoom_all) ) );
    list_popup.push_back (MenuElem ("zoom selection", slot (this, &EnhNormalView::slot_context_zoom_sel) ) );
    
    list_popup.push_back (SeparatorElem () );
    list_popup.push_back (MenuElem ("select all", slot (this, &EnhNormalView::slot_context_select_all) ) );
    list_popup.push_back (MenuElem ("unselect all", slot (this, &EnhNormalView::slot_context_select_clear) ) );
    list_popup.push_back (MenuElem ("invert selection", slot (this, &EnhNormalView::slot_context_select_invert) ) );
    
    list_popup.push_back (SeparatorElem () );
    list_popup.push_back (MenuElem ("(Debug: redraw)", slot (this, &EnhNormalView::slot_context_debug_redraw) ) );
    list_popup.push_back (MenuElem ("(Debug: reload)", slot (this, &EnhNormalView::slot_context_debug_reload) ) );
    list_popup.push_back (MenuElem ("(Debug: dump internal)", slot (this, &EnhNormalView::slot_context_debug_dump_internal) ) );
    
    // menu_popup.accelerate(*w_window); // works only when no widget isunder the button-press ??
  }
  // the popup menu needs to be told where to place its accelerators
  
  track_index t_tracks = m_manager->CurrentLayerPtr ()->Tracks ();
  
  for (track_index tr = 0; tr < t_tracks; ++tr)
    {
      cout << "adding track view: " << tr << endl;
      view_stuff t_stuff;
      t_stuff.plot = new WavePlotTracer<MinMaxAvgWavePlot> (m_manager, tr);
      t_stuff.view = new EnhTrackView (t_stuff.plot, &m_config, this, tr == 0);
      
      view_box.pack_start (*t_stuff.view);
      m_views.push_back (t_stuff);
    }
  
  m_manager->GetData ()->SignalCurrentLayerChanged.connect (slot (this, &EnhNormalView::SlotLayerChanged) );
  m_timeout->connect (slot (this, &EnhNormalView::SlotGUIRefresh) );
}

void EnhNormalView::set_zoom (int n_sps)
{
  if (m_sps != n_sps)
    {
      m_sps = n_sps;
      set_start (m_start, true, true);
      update_adjustment ();
    }
}

void EnhNormalView::set_start (sample_index n_start, bool adjustment_update = true,
			       bool force_plot_reconf = false)
{
  //cout << "EnhNormalView::set_start: " << n_start << endl;
  
  n_start = limit_min (limit_max (n_start, m_layer_length - m_view_length), 0);
  
  sample_index t_delta = n_start - m_plot_start; 
  
  //cout << "  current: plot_start: " << m_plot_start << ", new start: " << n_start
  //     << ", delta: " << t_delta
  //     << endl << "  current view start delta: " << m_view_start_delta << endl;
  
  // in plot range: easy scrolling (by delta):
  if (!force_plot_reconf && t_delta >= 0 && t_delta / m_sps + m_view_width <= m_plot_length)
    {
      m_view_start_delta = t_delta / m_sps;
      set_views_start_delta ();
    }
  else
    {
      // not in range: reconfig plots (slow)
      //cout << "  can't easy scroll ..." << endl;
	
      sample_index t_cache_length = m_view_width * SCROLL_CACHE_FACTOR * m_sps;
      sample_index t_view_start_delta = m_view_width * m_sps;
	
      sample_index t_plot_start;
      // on which side should be place the caches larger part (scroll direction)
      if (t_delta > 0) // scroll right
	t_plot_start = limit_min (n_start - t_view_start_delta, 0);
      else // scroll left
	t_plot_start = limit_min (n_start - t_cache_length + t_view_start_delta, 0);
      
      //cout << "  new plot-start: " << t_plot_start << endl;
	
      m_view_start_delta = (n_start - t_plot_start) / m_sps;
      //cout << "  new m_view_start_delta: " << m_view_start_delta << endl;
      
      set_views_start_delta ();
      reconfig_plots (t_plot_start, m_sps);
    }
  
  m_start = m_plot_start + m_view_start_delta * m_sps; // correct rounding errors
  
  if (adjustment_update)
    {
      m_adjustment.set_value (m_start);
    }
}

void EnhNormalView::reconfig_plots (sample_index n_start, sample_index n_sps)
{
  cout << "EnhNormalView::reconfig_plots:" << endl;
  debug_internals ();
  m_views [0].view->Debug ();
  
  m_plot_start = n_start;
  m_sps = n_sps;
  
  // new optimization hack ;-) (not only mem, but also zero generation time saving ...)
  m_plot_length = limit_min (limit_max (m_view_width * SCROLL_CACHE_FACTOR,
					(m_layer_length - m_plot_start) / m_sps),
			     m_view_width * SMOTH_SCROLL_FACTOR);
  /*
  if (m_plot_length != m_view_width * SCROLL_CACHE_FACTOR)
    cout << "  Newest optimization hack: m_plot_length limited: " << m_plot_length << endl;
  */
  m_cache_length = m_plot_length * n_sps;

  m_config.SetStart (m_plot_start);
  m_config.SetPlotLength (m_plot_length);
  m_config.SetSamplesPerSample (m_sps);
  m_config.SetViewPoint (m_view_start_delta);
  
  // maybe this must be updated ...
  m_view_length = m_view_width * m_sps;
  m_cache_length = m_plot_length * m_sps;

  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    i->plot->Config (m_config);
}

void EnhNormalView::set_views_start_delta ()
{
  // cout << "EnhNormalView::set_views_start_delta: " << n_view_start_delta << endl;
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    {
      i->view->SetStartDelta (m_view_start_delta);
    }
}

void EnhNormalView::debug_internals ()
{
  cout << "EnhNormalView::debug_internals:" << endl
       << "m_start:            " << m_start << endl
       << "m_layer_length:     " << m_layer_length << endl
       << "m_plot_start:       " << m_plot_start << endl
       << "m_plot_length:      " << m_plot_length << endl
       << "m_cache_length:     " << m_cache_length << endl
       << "m_view_width:       " << m_view_width << endl
       << "m_view_length:      " << m_view_length << endl
       << "m_sps:              " << m_sps << endl
       << "m_view_start_delta: " << m_view_start_delta << endl
       << "m_auto_zoom_all:    " << m_auto_zoom_all << endl;
}

void EnhNormalView::update_adjustment ()
{
  m_adjustment.set_page_size (m_view_length);
  m_adjustment.set_page_increment (m_view_length / 2);
  m_adjustment.set_step_increment (m_view_length / 10);
  m_adjustment.set_upper (m_layer_length);
  
  m_hruler.set_range (m_start,
		      m_start + m_view_length,
		      m_start,
		      m_layer_length);
}

void EnhNormalView::start_selection (sample_index n_current)
{
  selection_start = n_current;
  selecting = true;
  selected = false;
}

void EnhNormalView::set_selection (sample_index n_current)
{
  if (!selecting)
    {
      selection_start = n_current;
      selection_current = n_current;
      selecting = true; 
    }
  selection_current = n_current;
  selected = true;
  
  sample_index t_sel_start = min (selection_start, selection_current);
  sample_index t_sel_end = max (selection_start, selection_current);
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    {
      cout << "temp_sel_: " << t_sel_start << " - " << t_sel_end << endl;
      i->view->SetTempSelection (t_sel_start, t_sel_end);
    }
  
}

void EnhNormalView::end_normal_selection ()
{
  sample_index t_sel_start = min (selection_start, selection_current);
  sample_index t_sel_end = max (selection_start, selection_current);
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    i->view->RemoveTempSelection ();
  
  DataLayer* t_layer = m_manager->CurrentLayerPtr ();
  
  cout << "clearing: " << endl;
  for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
    (*t_layer) [tr]->selection.Clear ();
  
  cout << "selecting: " << t_sel_start << " - " << t_sel_end << endl;
  for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
    (*t_layer) [tr]->selection.Select (t_sel_start, t_sel_end - t_sel_start);
  
  //for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
  //  (*t_layer) [tr]->selection.Debug ();
      
  selecting = false;
}

void EnhNormalView::end_state_selection (bool plus_minus)
{
  sample_index t_sel_start = min (selection_start, selection_current);
  sample_index t_sel_end = max (selection_start, selection_current);
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    i->view->RemoveTempSelection ();
  
  DataLayer* t_layer = m_manager->CurrentLayerPtr ();
  
  if (plus_minus)
    {
      cout << "selecting: " << t_sel_start << " - " << t_sel_end << endl;
      for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
	(*t_layer) [tr] ->selection.Select (t_sel_start, t_sel_end - t_sel_start);
    }
  else
    {
      cout << "unselecting: " << t_sel_start << " - " << t_sel_end << endl;
      for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
	(*t_layer) [tr] ->selection.Unselect (t_sel_start, t_sel_end - t_sel_start);
    }
  
  //for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
  //  (*t_layer) [tr] ->selection.Debug ();
  
  selecting = false;
}

void EnhNormalView::end_no_selection_but_marker ()
{
  DataLayer* t_layer = m_manager->CurrentLayerPtr ();
  
  t_layer->markers.SetMarker ("cur_pos", selection_start);
  
  selecting = false;
}

void EnhNormalView::SlotChildConfigured (int n_width)
{
  cout << "EnhNormalView::SlotChildConfigured: " << n_width << endl;
  
  if (n_width != m_view_width && n_width > 1) // we get a event with 1 from gtk at first - argh ..
    {
      m_view_width = n_width;
      
      // check for auto-zoom all
      if (m_auto_zoom_all)
	{
	  SetZoomAll ();
	}
      else // make shure all values get updated
	{      
	  reconfig_plots (m_plot_start, m_sps);
	  update_adjustment ();
	}
    }
  else
    cout << "  m_view_width vaild -> nothing to do ..." << endl;
}

void EnhNormalView::SlotMotionNotify (GdkEventMotion* event, GdkModifierType state)
{
  //cout << "EnhNormalView::SlotMotionNotify:" << endl;
  
  m_hruler.motion_notify_event.emit (event);
  
  if (state & GDK_BUTTON1_MASK)
    {
      //cout << "motion to: " << event->x << endl;
      
      sample_index t_cursor_pos;
      t_cursor_pos = limit_min_max (m_start + ( (sample_index) event->x * m_sps),
				    0,
				    m_layer_length);
      
      //cout << "cursor_pos: " << t_cursor_pos << endl;
      
      if (event->x < 0)
	{
	  m_autoscroll = true;
	  m_autoscroll_delta = (int) (event->x) * AUTO_SCOLL_ACCEL;
	}
      else if (event->x > m_view_width)
	{
	  m_autoscroll = true;
	  m_autoscroll_delta = (int) (event->x - m_view_width) * AUTO_SCOLL_ACCEL;
	}
      else
	m_autoscroll = false;
      
      set_selection (t_cursor_pos);
    }
}

void EnhNormalView::SlotButtenPress (GdkEventButton* event)
{
  cout << "EnhNormalView::SlotButtenPress:" << endl;
  
  sample_index t_cursor_pos;
  t_cursor_pos = limit_min_max (m_start + ( (sample_index) event->x * m_sps),
				0,
				m_layer_length);
  
  switch (event->button)
    {
    case 0x1:
      {
	start_selection (t_cursor_pos);
      } 
      break;
    case 0x2:
      {
 	//temp_sel.mark_a = cursor_pos;
      }
      break;
    case 0x3:
      {
	menu_popup.popup (event->button, event->time);
      }
      break;
    default:
      cout << "EnhNormalView::SlotButtonPress: unknown event ...??" << endl;
    }
}

void EnhNormalView::SlotButtenRelease (GdkEventButton* event)
{
  cout << "EnhNormalView::SlotButtenRelease:" << endl;
  
  switch (event->button)
    {
    case 0x1:
      {
	// modifiers ...
	if (selected)
	  {
	    if ( (event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK ) // removew
	      {
		end_state_selection (false);
	      }
	    else if ( (event->state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK ) // add
	      {
		end_state_selection (true);
	      }
	    else // normal
	      {
		end_normal_selection ();
	      }
	  }
	else
	  {
	    end_no_selection_but_marker ();
	  }
      } 
      break;
    case 0x2:
      {
 	//temp_sel.mark_a = cursor_pos;
      }
      break;
    case 0x3:
      {
	// menu_popup.popup (event->button, event->time);
      }
      break;
    default:
      cout << "EnhNormalView::SlotButtonRelease: unknown event ...??" << endl;
    }
  if (m_autoscroll)
    m_autoscroll = false;
}

// this is normally from a 2nd thread !! - keep that in mind - especially
// when you use a not threadsafe GUI like GTK-- ...
void EnhNormalView::SlotAdjustmentChanged ()
{
  /*
  cout << "EnhNormalView::SlotAdjustmentChanged:" << endl;
  
  cout << "EnhNormalViewChanged:" << endl;
  cout << "value: " << m_adjustment->get_value ()
       << ", lower: " << m_adjustment->get_lower ()
       << ", upper: " << m_adjustment->get_upper ()
       << ", step_inc: " << m_adjustment->get_step_increment ()
       << ", page_inc: " << m_adjustment->get_page_increment ()
       << ", page_size: " << m_adjustment->get_page_size ()
       << endl;
  */
}

void EnhNormalView::SlotAdjustmentValueChanged ()
{
  sample_index t_start = (sample_index) m_adjustment.get_value ();
  if (t_start != m_start)
    set_start (t_start, false);
  
  m_hruler.set_range (m_start,
		      m_start + m_view_length,
		      m_start,
		      m_layer_length);
}

void EnhNormalView::SlotLayerChanged (DataLayer* n_layer)
{
  cout << "EnhNormalView::SlotLayerChanged:" << endl;
  m_layer_length = n_layer->Length ();
  
  update_adjustment ();
  
  // check for auto-zoom all
  if (m_auto_zoom_all)
    {
      SetZoomAll ();
    }
  
  // maybe adapt start (if layer shorter and zoom at end ...)
  set_start (m_start);
  
  cout << "adjusting view count ..." << endl;
  DataLayer* t_layer = n_layer;
  track_index t_track_count = t_layer->Tracks ();
  
  while (m_views.size () > t_track_count)
    {
      // del_helper2 (m_views.back ().plot); // should be deleted by the wp-manager
      del_helper2 (m_views.back ().view);
      
      m_views.pop_back ();
    }
  
  if (m_views.size () < t_track_count)
    for (track_index tr = m_views.size (); tr < t_track_count; ++tr)
      {
	cout << "adding track view: " << tr << endl;
	view_stuff t_stuff;
	cout << "  createing Plot ..." << endl;
	t_stuff.plot = new WavePlotTracer<MinMaxAvgWavePlot> (m_manager, tr);
	cout << "  configuring Plot ..." << endl;
	t_stuff.plot->Config (m_config);
	cout << "  createing View ..." << endl;
	t_stuff.view = new EnhTrackView (t_stuff.plot, &m_config, this, tr == 0);
	
	cout << "  adding View to VBox ..." << endl;
	view_box.pack_start (*t_stuff.view);
	m_views.push_back (t_stuff);
	
	t_stuff.view->SetStartDelta (m_view_start_delta);
	
	cout << "  showing View ..." << endl;
	t_stuff.view->show ();
      }
}

void EnhNormalView::SlotGUIRefresh ()
{
  if (m_autoscroll)
    {
      set_start (limit_min_max (m_start + m_autoscroll_delta * m_sps,
				0,
				limit_min (m_layer_length - m_view_length, 0) ) );
      
      // set temp selection
      if (m_autoscroll_delta < 0)
	set_selection (m_start);
      else
	set_selection (m_start + m_view_length);
    }
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); i++)
    i->view->SlotGUIRefresh ();
}

void EnhNormalView::slot_context_zoom_in ()
{
  cout << "EnhNormalView::slot_context_zoom_in:" << endl;
  SetZoom (limit_min (m_sps / 4, 1) );
}

void EnhNormalView::slot_context_zoom_out ()
{
  cout << "EnhNormalView::slot_context_zoom_out:" << endl;
  SetZoom (m_sps * 4);
}

void EnhNormalView::slot_context_zoom_one2one ()
{
  cout << "EnhNormalView::slot_context_zoom_one2one:" << endl;
  SetZoom (1);
}

void EnhNormalView::slot_context_zoom_all ()
{
  cout << "EnhNormalView::slot_context_zoom_all:" << endl;
  SetZoomAll ();
}

void EnhNormalView::slot_context_zoom_sel ()
{
  cout << "EnhNormalView::slot_context_zoom_sel:" << endl;
  
  bool sel_found = false;
  sample_index t_start;
  sample_index t_end;
  
  DataLayer* t_layer = m_manager->CurrentLayerPtr ();
  for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
    {
      TrackLayer* t_track = (*t_layer) [tr];
      
      TrackLayer::Selection::const_iterator i = t_track->selection.begin ();
      TrackLayer::Selection::const_iterator i_end = t_track->selection.end ();
      
      for (; i != i_end; ++i)
	{
	  if (!sel_found)
	    {
	      t_start = i->start;
	      t_end = i->end;
	      sel_found = true;
	    }
	  else
	    {
	      t_start = limit_max (i->start, t_start);
	      t_end = limit_min (i->end, t_end);
	    }
	}
      
    }
  if (sel_found)
    {
      cout << "new zoom range: " << t_start << " - " << t_end << endl;
      
      sample_index t_zoom = (t_end - t_start) / Width ();
      
      SetZoom (t_zoom);
      SetStart (t_start);
    }
  else
    cout << "ERROR: no selection found to zoom into ... ?!?!?" << endl;
}

void EnhNormalView::slot_context_select_all ()
{
  cout << "EnhNormalView::slot_context_select_all" << endl;
  
  DataLayer* t_layer = m_manager->CurrentLayerPtr ();
  for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
    (*t_layer) [tr] ->selection.All ();
}

void EnhNormalView::slot_context_select_clear ()
{
  cout << "EnhNormalView::slot_context_select_clear" << endl;
  
  DataLayer* t_layer = m_manager->CurrentLayerPtr ();
  for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
    (*t_layer) [tr] ->selection.Clear ();
}

void EnhNormalView::slot_context_select_invert ()
{
  cout << "EnhNormalView:: slot_context_select_invert" << endl;
  
  DataLayer* t_layer = m_manager->CurrentLayerPtr ();
  for (track_index tr = 0; tr < t_layer->Tracks (); ++tr)
    (*t_layer) [tr] ->selection.Invert ();
}

void EnhNormalView::slot_context_debug_redraw ()
{
  cout << "EnhNormalView::slot_context_debug_redraw:" << endl;
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    {
      i->view->SetRefresh ();
    }
}

void EnhNormalView::slot_context_debug_reload ()
{
  cout << "EnhNormalView::slot_context_debug_reload:" << endl;
  
  debug_internals ();
  m_views [0].view->Debug ();
  
  m_config.SetViewPoint (m_view_start_delta);
  
  for (deque<view_stuff>::iterator i = m_views.begin (); i != m_views.end (); ++i)
    {
      i->plot->Config (m_config);
    }
}

void EnhNormalView::slot_context_debug_dump_internal ()
{
  cout << "EnhNormalView::slot_context_debug_dump_internal:" << endl;
  debug_internals ();
}
