#include <stdio.h>
#include <iostream.h>

#include <sys/ioctl.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>

#include "Basics.hh"
#include "Data.hh"
#include "IO_impl.hh"
#include "WavePlot.hh"

using namespace GSML;

#ifdef OLD_CODE_WHICH_DOESNT_COMPILE
void test_mixer_source ()
{
  cout << "MixerSource - Test:" << endl;
      
  DebugMixerSource* mix_source;
  SampleBufferDefault* buffer;
      
  mix_source = new DebugMixerSource (4, 88200, false, true); // tracks, length, looping, inputable
  buffer = new SampleBufferDefault (100000);
      
  mix_source->Debug ();
      
  cout << "starto:" << endl;
  mix_source->StartO ();
  mix_source->Debug ();
      
  cout << "test reading:" << endl;
  for (track_index i = 0; i < mix_source->PlaybackTracks (); i++)
    {
      cout << "reading track: " << i << endl;
      buffer->Clear ();
      mix_source->Read (i, 1234, buffer);
    }
  cout << "reading track: 1 again ..." << endl;
  buffer->Clear ();
  mix_source->Read (1, 1234, buffer);
      
  mix_source->Debug ();
      
  bool test_state;
      
  /* Test */
  test_state = false;
  // must throw exeption - in o mode ...
  cout << "write io exception test: " << endl;
  try {
    mix_source->Write (1, 1234, buffer);
  }
  catch (MixerSource::Except e)
    {
      cout << "got exception: " << e.info << endl;
	  
      test_state = true;
    }
  if (!test_state)
    cout << "BUG: no exception cought ???!?!?" << endl;
  /* end */
      
  /* Test */
  test_state = false;
  // must throw exeption  ...
  cout <<  "track count exception test: " << endl;
  try {
    buffer->Clear ();
    mix_source->Read (mix_source->PlaybackTracks () + 1, 1234, buffer);
  }
  catch (MixerSource::Except e)
    {
      cout << "got exception: " << e.info << endl;
	  
      test_state = true;
    }
  if (!test_state)
    cout << "BUG: no exception cought ???!?!?" << endl;
  /* end */
      
  /* Test */
  test_state = false;
  // must throw exeption  ...
  cout <<  "read length exception test: " << endl;
  try {
    buffer->Clear ();
    mix_source->Read (0, mix_source->Length () + 20, buffer);
  }
  catch (MixerSource::Except e)
    {
      cout << "got exception: " << e.info << endl;
      cout << "length in buffer: " << buffer->UsedLength () << endl;
      test_state = true;
    }      
  if (!test_state)
    cout << "BUG: no exception cought ???!?!?" << endl;
  /* end */
      
  mix_source->StopIO ();
      
  cout << "IO test: " << endl;
  mix_source->StartIO ();
  mix_source->Debug ();
      
  cout << "test reading:" << endl;
  for (track_index i = 0; i < mix_source->PlaybackTracks (); i++)
    {
      cout << "reading track: " << i << endl;
      buffer->Clear ();
      mix_source->Read (i, 1234, buffer);
    }
  cout << "reading track: 1 again ..." << endl;
  buffer->Clear ();
  mix_source->Read (1, 1234, buffer);
      
  cout << "test writing:" << endl;
  buffer->SetUsedLength (1234);
  for (track_index i = 0; i < mix_source->CaptureTracks (); i++)
    {
      cout << "writing track: " << i << endl;
      mix_source->Write (i, 1234, buffer);
    }
  cout << "writing track: 1 again ..." << endl;
  mix_source->Write (1, 1234, buffer);
      
  mix_source->Debug ();
      
  /* Test */
  test_state = false;
  // must throw exeption  ...
  cout <<  "write length exception test: " << endl;
  try {
    buffer->SetUsedLength (200);
    mix_source->Write (0, 400, buffer);
  }
  catch (MixerSource::Except e)
    {
      cout << "got exception: " << e.info << endl;
      test_state = true;
    }      
  if (!test_state)
    cout << "BUG: no exception cought ???!?!?" << endl;
  /* end */
      
  mix_source->Debug ();
      
  mix_source->StopIO ();
      
  delete buffer;
  delete mix_source;
}

void TestPlaybackTracksChanged (track_index tracks)
{
  cout << "TestPlaybackTracksChanged: " << tracks << endl;
}


void TestCaptureTracksChanged (track_index tracks)
{
  cout << "TestCaptureTracksChanged: " << tracks << endl;
}

void TestPlaybackChannelsChanged (track_index channels)
{
  cout << "TestPlaybackChannelsChanged: " << channels << endl;
}

void TestCaptureChannelsChanged (track_index channels)
{
  cout << "TestCaptureChannelsChanged: " << channels << endl;
}
    
void TestPlaybackValueChanged (track_index src_track, track_index dest_channel, pcm_default::value_type value)
{
  cout << "TestPlaybackValueChanged: " << src_track << ", " << dest_channel << ", " << value.data << endl;
}

void TestCaptureValueChanged (track_index dest_track, track_index src_channel, pcm_default::value_type value)
{
  cout << "TestCaptureValueChanged: " << dest_track << ", " << src_channel << ", " << value.data << endl;
}

void test_mixer_params ()
{
  bool test_state = false;
  UniversalMixerParams* m_pl_params;
  UniversalMixerParams* m_ca_params;

  m_pl_params = new UniversalMixerParams ();
  m_ca_params = new UniversalMixerParams ();
  
  m_pl_params->SignalTracksChanged.connect (slot (&TestPlaybackTracksChanged) );
  m_ca_params->SignalTracksChanged.connect (slot (&TestCaptureTracksChanged) );
  
  m_pl_params->SignalChannelsChanged.connect (slot (&TestPlaybackChannelsChanged) );
  m_ca_params->SignalChannelsChanged.connect (slot (&TestCaptureChannelsChanged) );
  
  m_pl_params->SignalValueChanged.connect (slot (&TestPlaybackValueChanged) );
  m_ca_params->SignalValueChanged.connect (slot (&TestCaptureValueChanged) );
  
  m_pl_params->Debug ();
  m_ca_params->Debug ();
  
  m_pl_params->SetTracks (4);
  m_ca_params->SetTracks (4);
  
  m_pl_params->SetChannels (2);
  m_ca_params->SetChannels (2);
  
  m_pl_params->SetValue (0, 0, (pcm_default::value_type) {55} );
  m_pl_params->SetValue (1, 1, (pcm_default::value_type) {66} );
  m_pl_params->SetValue (2, 0, (pcm_default::value_type) {77} );
  m_pl_params->SetValue (3, 1, (pcm_default::value_type) {88} );

  m_ca_params->SetValue (0, 0, (pcm_default::value_type) {55} );
  m_ca_params->SetValue (1, 1, (pcm_default::value_type) {66} );
  m_ca_params->SetValue (2, 0, (pcm_default::value_type) {77} );
  m_ca_params->SetValue (3, 1, (pcm_default::value_type) {88} );
					   
  m_pl_params->Debug ();
  m_ca_params->Debug ();
  
  /* Test */
  test_state = false;
  // must throw exeption  ...
  cout <<  "playback track range exception test: " << endl;
  try {
    m_pl_params->SetValue (m_pl_params->Tracks () + 1, 0, (pcm_default::value_type) {55} );
  }
  catch (MixerParams::Except e)
    {
      cout << "got exception: " << e.info << endl;
      test_state = true;
    }      
  if (!test_state)
    cout << "BUG: no exception cought ???!?!?" << endl;
  /* end */

  cout <<  "playback channel range exception test: " << endl;
  try {
    m_pl_params->SetValue (0, m_pl_params->Channels () + 1, (pcm_default::value_type) {55} );
  }
  catch (MixerParams::Except e)
    {
      cout << "got exception: " << e.info << endl;
      test_state = true;
    }      
  if (!test_state)
    cout << "BUG: no exception cought ???!?!?" << endl;
  /* end */
  
  /* Test */
  test_state = false;
  // must throw exeption  ...
  cout <<  "IO lock test: " << endl;
  m_ca_params->StartIO ();
  try {
    m_ca_params->SetTracks (666);
  }
  catch (MixerParams::Except e)
    {
      cout << "got exception: " << e.info << endl;
      test_state = true;
    }      
  if (!test_state)
    cout << "BUG: no exception cought ???!?!?" << endl;
  /* end */
  
  m_ca_params->StopIO ();
  
  /* test filling of values */
  m_pl_params->SetChannels (10);
  m_pl_params->Debug ();
  
  m_ca_params->SetTracks (7);
  m_ca_params->Debug ();
  
  /* test shrinking of values */
  m_pl_params->SetChannels (2);
  m_ca_params->SetTracks (4);
  m_pl_params->Debug ();
  m_ca_params->Debug ();
  
  delete m_ca_params;
  delete m_pl_params;
}

void test_mixer_node ()
{
  DebugMixerSource* m_source;
  UniversalMixerParams* m_pl_params;
  UniversalMixerParams* m_ca_params;
  DebugMixerNode* m_node;
  Mixer* m_mixer;
      
  cout << "*** Mixer / Node TEST ***" << endl;
      
  cout << "creating objects ..." << endl;
  m_source = new DebugMixerSource (4, 2000, false, true); // looping, inputable
  m_source->SetGenerator (DebugMixerSource::INCREMENT);
  
  m_pl_params = new UniversalMixerParams ();
  m_ca_params = new UniversalMixerParams ();
      
  m_pl_params->SetTracks (4);
  m_ca_params->SetTracks (4);
  m_pl_params->SetChannels (2);
  m_ca_params->SetChannels (2);
  
  m_node = new DebugMixerNode (4, 2, 50, 100000); // pl_ch, cp_ch, frame_size, delay
  
  m_mixer = new Mixer;
      
  cout << "setting params ..." << endl;
  pcm_default exv;
  m_pl_params->SetValue (0, 0, exv.dBfs () );
  m_pl_params->SetValue (1, 0, exv.dBfs () );
  m_pl_params->SetValue (2, 1, exv.dBfs () );
  m_pl_params->SetValue (3, 1, exv.dBfs () );
      
  m_ca_params->SetValue (0, 1, exv.dBfs () );
  m_ca_params->SetValue (1, 1, exv.dBfs () );
  m_ca_params->SetValue (2, 0, exv.dBfs () );
  m_ca_params->SetValue (3, 0, exv.dBfs () );
      
  cout << "testing Debugs () ..." << endl;
  m_source->Debug ();
  m_pl_params->Debug ();
  m_ca_params->Debug ();
  m_node->Debug ();
  m_mixer->Debug ();
      
  cout << "configuring mixer ..." << endl;
  m_mixer->SetPlayback (true);
  m_mixer->SetCapture (false);
      
  cout << "starting io ..." << endl;
      
  m_node->Init ();
      
  m_mixer->SetNode (m_node);
  m_mixer->StartIO (m_source, m_pl_params, m_ca_params);
      
  while (m_mixer->IsIORunning () )
    {
      cout << "in MAIN loop ..." << endl;
	  
      cout << "*** MIXER ***" << endl;
      m_mixer->Debug ();
	  
      cout << "*** NODE ***" << endl;
      m_node->Debug ();
	  
      sleep (1);
    }
   cout << "deleteing objects ..." << endl;
   delete m_mixer;
   delete m_node;
   delete m_ca_params;
   delete m_pl_params;
   delete m_source;
}

void test_node_benchmark ()
{
  cout << "*** BENCHMARKING ***" << endl;
  
  DebugMixerSource* m_source;
  UniversalMixerParams* m_pl_params;
  UniversalMixerParams* m_ca_params;
  DebugMixerNode* m_node;
  Mixer* m_mixer;
      
  cout << "*** Mixer / Node TEST ***" << endl;
      
  cout << "creating objects ..." << endl;
  m_source = new DebugMixerSource (4, 44100 * 10, false, true); // looping, inputable
      
  m_pl_params = new UniversalMixerParams ();
  m_ca_params = new UniversalMixerParams ();
      
  m_pl_params->SetTracks (4);
  m_ca_params->SetTracks (4);
  m_pl_params->SetChannels (2);
  m_ca_params->SetChannels (2);
      
  m_node = new DebugMixerNode (4, 2, 16000, 0); // pl_ch, cp_ch, frame_size, delay
  m_node->SetBenchmarking (true);
  
  sleep (1); // work around for thread bug ...
      
  m_mixer = new Mixer;
      
  cout << "setting params ..." << endl;
  pcm_default exv;
  m_pl_params->SetValue (0, 0, exv.dBfs () );
  m_pl_params->SetValue (1, 0, exv.dBfs () );
  m_pl_params->SetValue (2, 1, exv.dBfs () );
  m_pl_params->SetValue (3, 1, exv.dBfs () );
      
  m_ca_params->SetValue (0, 1, exv.dBfs () );
  m_ca_params->SetValue (1, 1, exv.dBfs () );
  m_ca_params->SetValue (2, 0, exv.dBfs () );
  m_ca_params->SetValue (3, 0, exv.dBfs () );
      
  cout << "configuring mixer ..." << endl;
  m_mixer->SetPlayback (true);
  m_mixer->SetCapture (true);
      
  cout << "starting io ..." << endl;
      
  m_node->Init ();
  
  m_mixer->SetNode (m_node);
  
  m_mixer->StartIO (m_source, m_pl_params, m_ca_params);
  while (m_mixer->IsIORunning () )
    {
      cout << "processing ..." << endl;
      sleep (1);
    }
  
  cout << "deleteing objects ..." << endl;
  delete m_mixer;
  delete m_node;
  delete m_ca_params;
  delete m_pl_params;
  delete m_source;
}


void main()
{
  try
    {
      cout << "A quick GMSL::IO text ..." << endl;

      DataFileIndex* m_index;
      Mixer* m_mixer;
      ALSAMixerNode* m_node;
      
      Data* m_data;
      
      test_mixer_source ();
      
      test_mixer_params ();
      
      test_mixer_node ();
      
      test_node_benchmark ();
      
      cout << "END ..." << endl;
    }
  catch (SampleBuffer::Except e)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << "GSML_Sample_Buffer:" << endl;
      cout << "Length: " << e.length << ", used length: " << e.used_length << endl;
      cout << e.info << endl;
    }
  catch (DataFile::Except e)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << "GSML_Data_File:" << endl;
      cout << e.info << endl;
    }
  catch (TrackLayer::Except e)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << "GSML_Data_Track:" << endl;
      cout << e.info << endl;
    }
  catch (DataLayer::Except e)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << "GSML_Data_Layer:" << endl;
      cout << e.info << endl;
    }
  catch (Data::Except e)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << "GSML_Data:" << endl;
      cout << e.info << endl;
    }
  catch (DataFileIndex::Except e)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << "GSML_Data_File_Index:" << endl;
      cout << e.info << endl;
    }
  
  catch (BasicExcept e)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << e.info << endl;
    }
  catch (...)
    {
      cout << "EE XX EE PP TT II OO NN !! !! ::" << endl;
      cout << " UNKNOWN ... !!!" << endl;
    }
}

#endif // OLD_CODE_WHICH_DOESNT_COMPILE

main ()
{
  cout << "test_io::main:" << endl;
  
  ALSAHelper::ALSAControl t_alsa;
  
  for (int i = 0; i < 8; ++i)
    {
      string card_name = t_alsa.GetName (i);
      cout << "Card #" << i << ": " << card_name << endl;
      if (card_name != "")
	{
	  //t_alsa.Open (card_name, SND_PCM_PLAYBACK);
	}
    }
  
  cout << "END." << endl;
}
