/*  GSML - General Sound Manipulation Library
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   ALSA C++ wrapper
 */

#ifndef ALSA_HELPER_H
#define ALSA_HELPER_H

#include "Basics.hh"
#include "generated/config.hh"

#ifdef GSML_WITH_ALSA

// ALSA stuff (version 0.9.x ...)
#include <alsa/asoundlib.h>

namespace ALSAHelper
{
  class ALSAControl
  {
  public:
    ALSAControl ();
    ~ALSAControl ();
    
    string GetName (int index);
    
    bool Open (string name, int mode);
    bool Close ();
   
    
  protected:
    /* direct ALSA states */
    snd_ctl_t* handle;
    
    /* states */
    bool opened;
    int err;
    
    /* config */
    string cname;
  };
  
  class ALSAPCM
  {
  public:
    class Except : public GSML::BasicExcept {};
    class ExceptOutOfMemory : public Except {};
    
    ALSAPCM ();
    ~ALSAPCM ();
    
    bool Open (const string& name, snd_pcm_stream_t stream, int mode);
    bool Close ();
    
    snd_pcm_type_t Type ();
    
    /* HW Config */
    bool InstallParams ();
    
    bool SetAccess (snd_pcm_access_t access_mode);
    int GetAccess ();
    
    bool SetFormat (snd_pcm_format_t format);
    int GetFormat ();
    
    bool SetRate (unsigned int rate);
    bool SetRateNear (unsigned int rate);
    int GetRateMin ();
    int GetRateMax ();
    int GetRate ();
    
    bool SetChannels (unsigned int channels);
    int GetChannelsMin ();
    int GetChannelsMax ();
    int GetChannels ();
    
    bool SetBufferTimeNear (unsigned int time);
    bool SetBufferSizeNear (snd_pcm_uframes_t size);
    bool SetBufferSize (snd_pcm_uframes_t size);
    int GetBufferTime ();
    int GetBufferSize ();
    
    bool SetPeriodTimeNear (unsigned int time);
    bool SetPeriodSizeNear (snd_pcm_uframes_t size);
    bool SetPeriodSize (snd_pcm_uframes_t size);
    int GetPeriodTime ();
    int GetPeriodSize ();
    
    bool SetPeriods (unsigned int periods);
    bool SetPeriodsNear (unsigned int periods);
    int GetPeriodsMin ();
    int GetPeriodsMax ();
    int GetPeriods ();
    
    /* SW Config */
    bool InstallSwParams ();
    
    bool SetSwStartThreshold (snd_pcm_uframes_t count);
    snd_pcm_uframes_t GetSwStartThreshold ();
    
    bool SetSwStopThreshold (snd_pcm_uframes_t count);
    snd_pcm_uframes_t GetSwStopThreshold ();
    
    /* Control / IO */
    bool Prepare ();
    bool Drain ();
    bool Start ();
    
    // I guess (...) that n means non_interleaved - and i is interleaved
    snd_pcm_sframes_t WriteI (const void* buffer, snd_pcm_uframes_t size);
    snd_pcm_sframes_t ReadI (void* buffer, snd_pcm_uframes_t size);
    
    snd_pcm_sframes_t WriteN (snd_pcm_t* pcm, void** bufs, snd_pcm_uframes_t size);
    snd_pcm_sframes_t ReadN (snd_pcm_t* pcm, void** bufs, snd_pcm_uframes_t size);
    
    /* Status stuff */
    
    bool UpdateStatus ();
    
    snd_pcm_state_t GetStatusState ();
    void GetTriggerTStamp (snd_timestamp_t* ptr);
    void GetTStamp (snd_timestamp_t* ptr);
    snd_pcm_sframes_t GetStatusDelay ();
    snd_pcm_uframes_t GetStatusAvail ();
    snd_pcm_uframes_t GetStatusAvailMax ();
    
  protected:
    /* direct ALSA states */
    snd_ctl_t* ctl_handle;
    snd_pcm_t* handle;
    
    snd_pcm_hw_params_t* hw_params;
    snd_pcm_sw_params_t* sw_params;
    snd_pcm_status_t* status;
    
    /* states */
    bool opened;
    int err; // the ALSA return code - used in each function ...
    
    /* Config */
    string cname;
  };

} // namespace ALSA_Helper

#endif // GSML_WITH_ALSA
#endif // ALSA_HELPER
