/*  GSML - General Sound Manipulation Library
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   some very basic GSML helpers
 */

#ifndef GSML_BASICS
#define GSML_BASICS

#include "include.hh"
#include "typedefs.hh"

#include <sys/time.h> // used by Timer

#include <sys/types.h> // used by StopInDebugger
#include <signal.h>    //          ||
#include <unistd.h>    //          ||

namespace GSML
{
#define STRING_TEMP_LENGTH 128
  
  // Our Exception Base
  class BasicExcept
  {
  public:
    string info;
  };
  
  // to delete container of objects via for_each ...
  template <class T>
  class del_helper
  {
  public:
    void operator () (T& del_obj)
    {
      //cout << "del_helper template: deleting address: " << del_obj << endl;
      delete del_obj;
      del_obj = 0;
    }
  };
  
  template <class T>
  void del_helper2 (T& del_obj)
  {
    //cout << "del_helper2 template: deleting address: " << del_obj << endl;
    delete del_obj;
    del_obj = 0;
  }
  
  template <class U, class V>
  inline const U limit_min (const U& value, const V& min)
  {
    if (value < min)
      return min;
    else
      return value;
  };
  
  template <class U, class V>
  inline const U limit_max (const U& value, const V& max)
  {
    if (value > max)
      return max;
    else
      return value;
  };
  
  // should only be used when min < max ... !!
  template <class U, class V, class W>
  inline const U limit_min_max (const U& value, const V& min, const W& max)
  {
    if (value < min)
      return min;
    else if (value > max)
      return max;
    else
      return value;
  };
  
  class Timer
  {
  public:
    Timer ();
    
    void Reset ();
    long long Delta ();
    
  private:
    timeval m_start;
  };
  
  class AutoTimer
  {
  public:
    AutoTimer (const string& i_text);
    ~AutoTimer ();
    
    long long Delta ();
    
  private:
    Timer m_timer;
    string m_text;
   };
  
  // find a unique filename fname + i; the first i can be specified by; the fname is overwritten;
  // the used index is returned - btw. a file is unique when there is no file with the tested name
  // -> even if the trailing path is not available ... !!?!? ...
  int FindUniqueName (string& fname, const string& base, const string& ext, int first_tried_index = 0);
  
  void StopInDebugger ();
  
  namespace ThreadHelper
  {
    bool EnableRealtimeScheduling ();
    void USleep (int delay);
    
    bool SetPriority (int priority);
    
  } // end namespace ThreadHelper
  
  // int to string
  string itos (const int& i);
  string itos (const unsigned int& i);
  // double to string
  string dtos (const double& d);
  
  // TODO: for synced debug output ... ??
} // end namespace GSML

#endif //GSML_BASICS
