/*  GSML - General Sound Manipulation Library
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Index to keep track of used files, temp-file creation and removal.
 */

#ifndef GSML_DATA_FILE_INDEX
#define GSML_DATA_FILE_INDEX

// for mkdir, file access, stat ...

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "include.hh"
#include "typedefs.hh"
#include "Basics.hh"

#include "SampleBuffer.hh"
#include "DataFile.hh"

#include <deque> // STL deque

namespace GSML
{

#define DEBUG_GSML_DATA_FILE_INDEX
  
  class DataFileIndex
  {
  public:

    class Config
    {
    public:
      Config& operator= (const Config& src_config)
      {
	filename_base = src_config.filename_base;
	filename_ext = src_config.filename_ext;
	working_dir = src_config.working_dir;
      }
    
      string filename_base;
      string filename_ext;
      string working_dir;
    };
    
    struct file_stuff
    {
      DataFile* datafile;
      dev_t device;
      ino_t inode;
    };
    
    class Except : public BasicExcept {};
    class ExceptUnsupported : public Except {};
    class ExceptLogicalError : public Except {};
    class ExceptNoFileFound : public Except {};
    class ExceptDeleteError : public Except {};
    
    typedef std::deque <file_stuff> :: iterator iterator;
    typedef std::deque <file_stuff> :: const_iterator const_iterator;
    
    DataFileIndex (const DataFileIndex::Config& i_config);
    ~DataFileIndex ();

    // check the working dir and create if it doesn't exists
    bool Configure (bool correct_minor_problems);

    void RegisterFile (DataFile* file);

    // generates the temp-filename and inserts it into the file-list
    void RegisterTempFile (DataFile* file);
  
    // or do we need a "unregister_file" method ?? - I think no ....
    void DeleteUnusedIndexes ();
    
    bool IsInIndex (string fname);
    
    const_iterator Begin ();
    const_iterator End ();
    
    void Debug ();
    
  protected:
    
    bool AddDataFile (DataFile* n_file);
    
    // states
    bool m_configured;  // is true if the working dir is chekced to exist and writeable
    int m_next_temp_file_number;  // next number between file_base and file_ext that should be used
  
    std::deque <file_stuff> m_file_index;
    
    // config
    Config m_config;
  };

} // end namespace GSML

#endif //GSML_DATA_FILE_INDEX
