/*  GSML - General Sound Manipulation Library
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   OSS Mixer Node implementation (very limitted - 16bit only)
 */

#ifndef GSML_IO_OSS_H
#define GSML_IO_OSS_H

#include "IO.hh"

#include <sys/ioctl.h> // all OSS specific stuff
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>  

#include <sys/time.h>  // needed for select
#include <sys/types.h> // dito


namespace GSML {
  
  // the OSS implementation which uses our Job to perform the sound card IO
  class OSSMixerNode : public HWMixerNode, public GSMX_utils::Threads::Job
  {
  public:
    OSSMixerNode ();
    virtual ~OSSMixerNode ();
    
    // HWMixerNode interface
    virtual bool Init ();
    
    virtual const vector<string>& Interfaces ();
    virtual const string& Interface ();
    virtual bool SetInterface (const string& n_interface);
    
    // MixerNode interface
    
    virtual track_index PlaybackChannels ();
    virtual track_index CaptureChannels ();
    
    virtual int PlaybackPeriodSize ();
    virtual int CapturePeriodSize ();
    
    virtual int PlaybackPeriods ();
    virtual int CapturePeriods ();
    
    virtual int PlaybackXRuns ();
    virtual int CaptureXRuns ();
    
    virtual void SetPlayback (bool state);
    virtual void SetCapture (bool state);
    
    virtual bool IsPlayback ();
    virtual bool IsCapture ();
    
    virtual void StartIO (Mixer& source);
    virtual void StopIO ();
    
    virtual bool IsIORunning ();
    
    virtual sample_index GetCurrentPos ();
    
    virtual void Debug ();
    
  protected:
    
    bool reget_card_params ();
    bool internal_config_params (bool overwrite_params);
    
    bool IsReadable (int test_fd) {
      fd_set fs;
      timeval tv;
      
      tv.tv_sec = 0;
      tv.tv_usec = 0;
      
      FD_ZERO (&fs);
      FD_SET (test_fd, &fs);
      
      int ret = select (1, &fs, NULL, NULL, &tv);
      return (ret != 0);
    }
    
    bool IsWriteable (int test_fd) {
      fd_set fs;
      timeval tv;
      
      tv.tv_sec = 0;
      tv.tv_usec = 0;
      
      FD_ZERO (&fs);
      FD_SET (test_fd, &fs);
      
      int ret = select (1, NULL, &fs, NULL, &tv);
      return (ret != 0);
    }
    
    // the second thread ...
    void Loop ();
    
    SampleBufferDefault* pl_buffer;
    pcm_16bit::value_type* pl_interleaved;
    int pl_processed;
    bool pl_vaild;
    
    SampleBufferDefault* ca_buffer;
    pcm_16bit::value_type* ca_interleaved;
    int ca_processed;
    
    // states
    Mixer* m_mixer;
    
    bool initialized;
    bool io_running;
    bool cap_running; // only indicated whether cp was started via card->Start ()

    int oss_fd;
    
    track_index oss_channels; 
    int oss_period_size;
    int oss_periods;
    
    bool stop_requested;
    bool m_is_first_loop;
    bool m_worked_in_last_loop;
    bool m_using_realtime_sched;
    
    /* frame counting */
    sample_index current_playback_period;
    sample_index current_capture_period;
    
    int pl_xruns;
    int ca_xruns;
    
    // config
    string cname;
    
    bool do_playback;
    bool do_capture;
  };

} // end namespace GSMP
  

#endif // GSMP_IO_OSS_H
