/*  GSML - General Sound Manipulation Library
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Base interface and 16bit / 24/8bit implementation of the buffer class
 *   we pass the sound data arround.
 *
 *   Very old-style C / C++. Will be replaced with a new dynamic-type and
 *   streaming concept in a future release ... .
 */

#ifndef GSML_SAMPLE_BUFFER
#define GSML_SAMPLE_BUFFER

#include "include.hh"
#include "typedefs.hh"
#include "Basics.hh"

namespace GSML
{
  // #define DEBUG_GSML_SAMPLE_BUFFER // for debuging ...
  
  // 1 channel (MONO!!) pcm-buffer we don't pass interleaved data at any time ...
  class SampleBuffer : public SigC::Object
    {
    public:
      class Except : public BasicExcept
      {
      public:
	sample_index used_length;
	sample_index length;
      };
      
      class ExceptNoData : public Except {};
      class ExceptFull : public Except {};
      class ExceptUnsupported : public Except {};
      
      virtual void WriteData (sample_index write_length,
			      pcm_8bit::value_type* source) = 0;
      
      virtual void WriteData (sample_index write_length,
			      pcm_16bit::value_type* source) = 0;
      
      virtual void WriteData (sample_index write_length,
			      pcm_24_8bit::value_type* source) = 0;
      
      virtual void ReadData (sample_index index,
			     sample_index read_length,
			     pcm_8bit::value_type* destination) const = 0;
      
      virtual void ReadData (sample_index index,
			     sample_index read_length,
			     pcm_16bit::value_type* destination) const = 0;
      
      virtual void ReadData (sample_index index,
			     sample_index read_length,
			     pcm_24_8bit::value_type* destination) const = 0;
      
      virtual void Zero (sample_index length) = 0; // really zero the buffer
      virtual void ZeroUnused () = 0; // helper method to clear trailing garbage
      
      virtual void Clear () = 0; // should only clear the used_length
      
      virtual sample_index UsedLength () const = 0;
      virtual void SetUsedLength (sample_index new_used_length) = 0; // only for direct data_access !! ...
      
      virtual sample_index Length () const = 0;
      
    private:
      
      // unallowed!
      SampleBuffer& operator= (SampleBuffer& src)
      {
      }
    };

  /*
    class SampleBuffer8Bit : public SampleBuffer
    {
    }
  */

  class SampleBuffer16Bit : public SampleBuffer
    {
    public:
      SampleBuffer16Bit (sample_index new_length);
      ~SampleBuffer16Bit ();

      void WriteData (sample_index write_length,
		      pcm_8bit::value_type* source);

      void WriteData (sample_index write_length,
		      pcm_16bit::value_type* source);
      
      void WriteData (sample_index write_length,
		      pcm_24_8bit::value_type* source);
      
      void ReadData (sample_index index,
		     sample_index read_length,
		     pcm_8bit::value_type* destination) const;
      
      void ReadData (sample_index index,
		     sample_index read_length,
		     pcm_16bit::value_type* destination) const;
      
      void ReadData (sample_index index,
		     sample_index read_length,
		     pcm_24_8bit::value_type* destination) const;
      
      void Zero (sample_index zero_length);
      void ZeroUnused ();
      
      void Clear ();
      
      sample_index UsedLength () const;
      void SetUsedLength (sample_index new_used_length);
      sample_index Length () const;
      
      SampleBuffer16Bit& operator= (SampleBuffer16Bit& src);
      
      pcm_16bit::value_type* data;

    private:
      sample_index length;
      sample_index used_length;
    };

  class SampleBuffer24_8Bit : public SampleBuffer
    {
    public:
      SampleBuffer24_8Bit (sample_index new_length, bool verb_dtor=false);
      ~SampleBuffer24_8Bit ();

      void WriteData (sample_index write_length,
		      pcm_8bit::value_type* source);

      void WriteData (sample_index write_length,
		      pcm_16bit::value_type* source);
      
      void WriteData (sample_index write_length,
		      pcm_24_8bit::value_type* source);
      
      void ReadData (sample_index index,
		     sample_index read_length,
		     pcm_8bit::value_type* destination) const;
      
      void ReadData (sample_index index,
		     sample_index read_length,
		     pcm_16bit::value_type* destination) const;
      
      void ReadData (sample_index index,
		     sample_index read_length,
		     pcm_24_8bit::value_type* destination) const;
      
      void Zero (sample_index zero_length);
      void ZeroUnused ();
      
      void Clear ();
      
      sample_index UsedLength () const;
      void SetUsedLength (sample_index new_used_length);
      sample_index Length () const;
      
      SampleBuffer24_8Bit& operator= (SampleBuffer24_8Bit& src);
      
      pcm_24_8bit::value_type* data;

    private:
      sample_index length;
      sample_index used_length;
      bool m_verb_dtor;
    };
  
  /* The SigC Handle (reference counting - kinda) */
  typedef SigC::Handle <SampleBuffer, SigC::Scopes::Extend> SBHandle;
  /* The default sample-buffer to use (at least internally) */
  typedef SampleBuffer24_8Bit SampleBufferDefault;
  
} // end namespace GSML

#endif //GSML_SAMPLE_BUFFER
