/*  GSML - General Sound Manipulation Library
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   some very basic GSML helpers
 */

#include "Basics.hh"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <sched.h>    // Linux/POSIX scheduler / realtime scheduling ...
#include <sys/resource.h> // set_priority

#include <sstream> // for stringstream

GSML::Timer::Timer ()
{
  gettimeofday (&m_start, NULL);
}

void GSML::Timer::Reset ()
{
  gettimeofday (&m_start, NULL);
}

long long GSML::Timer::Delta ()
{
  long long ctime;
  
  timeval t_time;
  gettimeofday (&t_time, NULL);
  
  ctime = (t_time.tv_sec * 1000000) + t_time.tv_usec;
  ctime -= ( (m_start.tv_sec * 1000000) + m_start.tv_usec);
  
  return ctime;
}

GSML::AutoTimer::AutoTimer (const string& i_text = "")
{
  m_text = i_text;
}

GSML::AutoTimer::~AutoTimer ()
{
  cout << ">> AutoTimer: \"" << m_text << "\" took: " << m_timer.Delta () << " us" << endl;
}

long long GSML::AutoTimer::Delta ()
{
  return m_timer.Delta ();
}

int GSML::FindUniqueName (string& fname, const string& base, const string& ext,
			  int first_tried_index = 0)
{
  stringstream t_fname;
  
  int t_index = first_tried_index;
  bool t_found = false;
  while (!t_found)
    {
      t_fname . str ("");
      t_fname << base << t_index << ext;
      cout << "testing: (statting): " << t_fname . str () << endl;
      struct stat t_stat;
      if (stat (t_fname . str () . c_str (), &t_stat) < 0)
	t_found = true;
      else
	++ t_index;
    }
  fname = t_fname . str ();
  return t_index;
}

void GSML::StopInDebugger ()
{
  pid_t m_pid = getpid ();
  kill (m_pid, SIGTRAP);
}

bool GSML::ThreadHelper::EnableRealtimeScheduling ()
{
  int t_pri = sched_get_priority_min (SCHED_FIFO);
  cout << "ThreadHelper:: priority_min: " << t_pri << endl;
  if (t_pri >= 0) {
    sched_param t_params;
    t_params.sched_priority = t_pri;
    
    int error = sched_setscheduler (0, SCHED_FIFO, &t_params);
    
    if (error < 0)
      cout << "ThreadHelper:: WARNING POSIX realtime-scheduling not available (not root?)! ..."
	   << endl;
    return error >= 0;
  }
  return false;
}

void GSML::ThreadHelper::USleep (int delay)
{
  /* From the nanosleep man-page:
   * 
   * If  the process is scheduled under a real-time policy like
   * SCHED_FIFO or SCHED_RR, then pauses of up to 2 ms will  be
   * performed as busy waits with microsecond precision.
   */
  
  // temp disable the sched_yield feature - we call it too often with such a little value ...
  delay = limit_min (delay, 5000); 
  if (delay < 500)
    sched_yield ();
  else
    usleep (delay);
}

bool GSML::ThreadHelper::SetPriority (int priority)
{
  return setpriority (PRIO_PROCESS, 0, priority) >= 0;
}

// seems to work
string GSML::itos (const int& i)
{
  string s;
  
  char* temp_str = new char [STRING_TEMP_LENGTH];
  sprintf (temp_str, "%i", i);
  s = temp_str;
  delete [] temp_str;
  
  return s;
}

// if (works (above)) {works(this)}
string GSML::itos (const unsigned int& i)
{
  string s;
  
  char* temp_str = new char [STRING_TEMP_LENGTH];
  sprintf (temp_str, "%u", i);
  s = temp_str;
  delete [] temp_str;
  
  return s;
}

// untested ...
string GSML::dtos (const double& d)
{
  string s;

  char* temp_str = new char [STRING_TEMP_LENGTH];
  sprintf (temp_str, "%f", d);
  s = temp_str;
  delete [] temp_str;

  return s;
}
