/*  GSML - General Sound Manipulation Library
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Base interface and 16bit / 24/8bit implementation of the buffer class
 *   we pass the sound data arround.
 *
 *   Very old-style C / C++. Will be replaced with a new dynamic-type and
 *   streaming concept in a future release ... .
 */

#include "SampleBuffer.hh"

using namespace GSML;

/****************************
 * the 16 bit impementation *
 ****************************/

SampleBuffer16Bit::SampleBuffer16Bit (sample_index new_length)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit: " << new_length << endl;
#endif
  
  length = new_length;
  used_length = 0;
  data = new pcm_16bit::value_type [length];
}

SampleBuffer16Bit::~SampleBuffer16Bit ()
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "~SampleBuffer16Bit: " << endl;
#endif
  delete [] data;
}


void SampleBuffer16Bit::WriteData (sample_index write_length,
				   pcm_8bit::value_type* source)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::WriteData (8bit)" << endl;
#endif
  if (used_length + write_length > length) //buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  
  for (sample_index count = 0; count < write_length; ++count)
    {
      data [used_length + count] ( (int16) source [count]  << 8 );
    }
  used_length += write_length;
}

void SampleBuffer16Bit::WriteData (sample_index write_length,
				   pcm_16bit::value_type* source)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::WriteData (16bit)" << endl;
#endif
  if (used_length + write_length > length) //buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  memcpy (&data[used_length], source, write_length * 2); //*2 cause count are bytes!!
    used_length += write_length;
}

void SampleBuffer16Bit::WriteData (sample_index write_length,
				   pcm_24_8bit::value_type* source)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::WriteData (24_8bit)" << endl;
#endif
  if (used_length + write_length > length) //buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  for (sample_index count = 0; count < write_length; ++count)
    {
      data [used_length + count] ( pcm_24_8bit::Convert2int16 ( source [count] ) );
    }
  used_length += write_length;
}

void SampleBuffer16Bit::ReadData (sample_index index,
				  sample_index read_length,
				  pcm_8bit::value_type* destination) const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::ReadData (8bit) ";
  cout << "index= " << index << ", read_length= " << read_length << endl;
#endif
  if (index + read_length > used_length) //enough data available??
    {
      SampleBuffer::ExceptNoData e;
      e.length = length;
      e.used_length = used_length;
      e.info = "read: not enough data available";
      throw e;
    }    
  
  for (sample_index count = 0; count < read_length; count++)
    {
      destination [count] ( data [index + count] >> 8 );
    }
}

void SampleBuffer16Bit::ReadData (sample_index index,
				  sample_index read_length,
				  pcm_16bit::value_type* destination) const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::ReadData (16bit) ";
  cout << "index= " << index << ", read_length= " << read_length << endl;
#endif
  if (index + read_length > used_length) //enough data available??
    {
      SampleBuffer::ExceptNoData e;
      e.length = length;
      e.used_length = used_length;
      e.info = "read: not enough data available";
      throw e;
    }
  
  memcpy (destination, &(data [index]), read_length * 2); //*2 because count are bytes!!
}
  
void SampleBuffer16Bit::ReadData (sample_index index,
				  sample_index read_length,
				  pcm_24_8bit::value_type* destination) const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::ReadData (24_8bit) ";
  cout << "index= " << index << ", read_length= " << read_length << endl;
#endif
  if (index + read_length > used_length) //enough data available??
    {
      SampleBuffer::ExceptNoData e;
      e.length = length;
      e.used_length = used_length;
      e.info = "read: not enough data available";
      throw e;
    }
  
  for (sample_index count = 0; count < read_length; count++)
    {
      destination [count] ( (int32) data [index + count] << 8 );
    }
}

void SampleBuffer16Bit::SetUsedLength (sample_index new_used_length)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::SetUsedLength " << new_used_length << endl;
#endif
  used_length = new_used_length;
}

sample_index SampleBuffer16Bit::Length () const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::GetLength " << endl;
#endif
  return length;
}

sample_index SampleBuffer16Bit::UsedLength () const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::GetUsedLength " << endl;
#endif
  return used_length;
}

void SampleBuffer16Bit::Zero (sample_index zero_length)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::Zero " << endl;
#endif
  
  if (used_length + zero_length > length) // buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  
  memset (data + used_length, 0, zero_length * sizeof (pcm_16bit::value_type) );
  used_length += zero_length;
}

void SampleBuffer16Bit::ZeroUnused ()
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::ZeroUnused " << endl;
#endif
  
  memset (data + used_length, 0, (length - used_length) * sizeof (pcm_16bit::value_type) );
  used_length = length;
}

void SampleBuffer16Bit::Clear ()
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer16Bit::Clear " << endl;
#endif
  used_length = 0;
}

SampleBuffer16Bit& SampleBuffer16Bit::operator= (SampleBuffer16Bit& src)
{
  length = src.length;
  used_length = src.used_length;
  
  memcpy (data, src.data, used_length * 2); //*2 cause count are bytes!!
}

/******************************
 * the 24/8 bit impementation *
 ******************************/

SampleBuffer24_8Bit::SampleBuffer24_8Bit (sample_index new_length, bool verb_dtor=false)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit: " << new_length << endl;
#endif
  
  length = new_length;
  used_length = 0;
  m_verb_dtor = verb_dtor;
  data = new pcm_24_8bit::value_type [length];
}

SampleBuffer24_8Bit::~SampleBuffer24_8Bit ()
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "~SampleBuffer24_8Bit: " << endl;
#endif
  
  delete [] data;
  
  if (m_verb_dtor)
    cout << "deleted: " << this << endl;
}


void SampleBuffer24_8Bit::WriteData (sample_index write_length,
				     pcm_8bit::value_type* source)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::WriteData (8bit)" << endl;
#endif
  if (used_length + write_length > length) //buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  
  for (sample_index count = 0; count < write_length; ++count)
    data [used_length + count] ( (int32) source [count]  << 16 );
  used_length += write_length;
}

void SampleBuffer24_8Bit::WriteData (sample_index write_length,
				     pcm_16bit::value_type* source)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::WriteData (16bit)" << endl;
#endif
  if (used_length + write_length > length) //buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  
  for (sample_index count = 0; count < write_length; ++count)
    data [used_length + count] ( (int32) source [count]  << 8 );
  used_length += write_length;
}

void SampleBuffer24_8Bit::WriteData (sample_index write_length,
				     pcm_24_8bit::value_type* source)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::WriteData (24_8bit)" << endl;
#endif
  if (used_length + write_length > length) //buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  
  memcpy (&data[used_length], source, write_length * sizeof (pcm_24_8bit::value_type) );
  used_length += write_length;
}

void SampleBuffer24_8Bit::ReadData (sample_index index,
				    sample_index read_length,
				    pcm_8bit::value_type* destination) const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::ReadData (8bit) ";
  cout << "index= " << index << ", read_length= " << read_length << endl;
#endif
  if (index + read_length > used_length) //enough data available??
    {
      SampleBuffer::ExceptNoData e;
      e.length = length;
      e.used_length = used_length;
      e.info = "read: not enough data available";
      throw e;
    }
  for (sample_index count = 0; count < read_length; ++count)
    {
      destination [count] ( pcm_24_8bit::Convert2int8 (data [index + count] ) );
    }
}

void SampleBuffer24_8Bit::ReadData (sample_index index,
				    sample_index read_length,
				    pcm_16bit::value_type* destination) const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::ReadData (16bit) ";
  cout << "index= " << index << ", read_length= " << read_length << endl;
#endif
  if (index + read_length > used_length) //enough data available??
    {
      SampleBuffer::ExceptNoData e;
      e.length = length;
      e.used_length = used_length;
      e.info = "read: not enough data available";
      throw e;
    }
  for (sample_index count = 0; count < read_length; ++count)
    {
      destination [count] ( pcm_24_8bit::Convert2int16 (data [index + count] ) );
    }
}
  
void SampleBuffer24_8Bit::ReadData (sample_index index,
				    sample_index read_length,
				    pcm_24_8bit::value_type* destination) const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::ReadData (24_8bit) ";
  cout << "index= " << index << ", read_length= " << read_length << endl;
#endif
  if (index + read_length > used_length) //enough data available??
    {
      SampleBuffer::ExceptNoData e;
      e.length = length;
      e.used_length = used_length;
      e.info = "read: not enough data available";
      throw e;
    }
  
  memcpy (destination, &(data [index]), read_length * sizeof (pcm_24_8bit::value_type) );
}

void SampleBuffer24_8Bit::SetUsedLength (sample_index new_used_length)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::SetUsedLength " << new_used_length << endl;
#endif
  used_length = new_used_length;
}

sample_index SampleBuffer24_8Bit::Length () const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::GetLength " << endl;
#endif
  return length;
}

sample_index SampleBuffer24_8Bit::UsedLength () const
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::GetUsedLength " << endl;
#endif
  return used_length;
}

void SampleBuffer24_8Bit::Zero (sample_index zero_length)
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::Zero" << endl;
#endif
  if (used_length + zero_length > length) // buffer big enough??
    {
      SampleBuffer::ExceptFull e;
      e.length = length;
      e.used_length = used_length;
      e.info = "write: buffer too small";
      throw e;
    }
  
  memset (data + used_length, 0, zero_length * sizeof (pcm_24_8bit::value_type) );
  
  used_length += zero_length;
}

void SampleBuffer24_8Bit::ZeroUnused ()
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::ZeroUnused" << endl;
#endif
  
  memset (data + used_length, 0, (length - used_length) * sizeof (pcm_24_8bit::value_type) );
  used_length = length;
}

void SampleBuffer24_8Bit::Clear ()
{
#ifdef DEBUG_GSML_SAMPLE_BUFFER
  cout << "SampleBuffer24_8Bit::Clear " << endl;
#endif
  used_length = 0;
}

SampleBuffer24_8Bit& SampleBuffer24_8Bit::operator= (SampleBuffer24_8Bit& src)
{
  length = src.length;
  used_length = src.used_length;
  
  memcpy (data, src.data, used_length * 4); //*4 cause count are bytes!!
}
