/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Interfaces for the plugins activated by th GSMP menusystem
 */

#ifndef GSMP_ACTIVATOR_HH
#define GSMP_ACTIVATOR_HH

namespace GSMP
{
  class App;
  class Project;
  class View;

  namespace Menu
  {

    class Activator
    { // Activator for SimpleItems, MutatorItems
    private:
      friend class SimpleItem;
      friend class MutatorItem;
      virtual void Execute () = 0;
    protected:
      // derived classes MUST have this Interface:
      Activator (App* i_app, Project* i_project, View* i_view)
      {
	m_app = i_app;
	m_project = i_project;
	m_view = i_view;
      }
     
      virtual ~Activator () {};

      App* m_app;
      Project* m_project;
      View* m_view;
    };
    
    class StatusActivator
    { // Activator for CheckedItems (status = "try to check Item")
    public:
      bool IsParasite () {return parasite;}
    private:
      friend class CheckedItem;
      friend class MenuData;
      virtual bool Execute (bool status) = 0; // returns new status
      bool parasite;
    protected:
      // derived classes MUST have this Interface:
      StatusActivator (App* i_app, Project* i_project, View* i_view)
      {
	m_app = i_app;
	m_project = i_project;
	m_view = i_view;
	parasite = false;
      }

      virtual ~StatusActivator () {};
      
      void MakeParasite () {parasite = true;}
      void DeParasite () {parasite = false;}

      App* m_app;
      Project* m_project;
      View* m_view;
    }; 

    class PredicateActivator
    { // Activator for SelectionItems (returnvalue = "accept selection")
    private:
      friend class SelectionItem;
      virtual bool Execute () = 0; // returns new status
    protected:
      // derived classes MUST have this Interface:
      PredicateActivator (App* i_app, Project* i_project, View* i_view)
      {
	m_app = i_app;
	m_project = i_project;
	m_view = i_view;
      }
      
      virtual ~PredicateActivator () {};

      App* m_app;
      Project* m_project;
      View* m_view;
    };

  } // end namespace Menu
} // end namespace GSMP

#endif // GSMP_ACTIVATOR_HH
