/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Class for real-time FX prehearing.
 */

#ifndef GSMP_FX_MIXERSOURCE
#define GSMP_FX_MIXERSOURCE

#include "FXBase.hh"
#include "IO.hh"
#include <map>

namespace GSMP
{

  class PlugHandle
  {
  public:
    GSML::track_index Track ();
  private:
    friend class FXMixerSource;

    int key;
    GSML::track_index m_track;
    LinearMonoFXPlug* fx;
  };

  class FXMixerSource : public GSML::MixerSource
  {
  public:
    FXMixerSource (GSML::DataLayer* i_layer);
    virtual ~FXMixerSource ();

    void SetDataLayer (GSML::DataLayer* layer);

    PlugHandle* AttachPlug (LinearMonoFXPlug* plug, GSML::track_index to_track);
    void DetachPlug (PlugHandle* plug);    

    virtual GSML::sample_index MinStart () const;
    virtual GSML::sample_index MaxEnd () const;

    virtual bool IsInputable () const;


    SigC::Signal0 <void> SignalAllUnpluged;    
  private:
   
    virtual void StartO_impl ();
    virtual void StartIO_impl ();
    virtual void StopIO_impl ();
    
    virtual void SetReadPos_impl (GSML::track_index track, GSML::sample_index n_pos, bool is_master);
    virtual void SetWritePos_impl (GSML::track_index track, GSML::sample_index n_pos, bool is_master);
    
    virtual bool Read_impl (GSML::track_index track, GSML::sample_index length, GSML::SampleBufferDefault* dest,
			    bool is_master);

    virtual bool Write_impl (GSML::track_index track, GSML::sample_index length, GSML::SampleBufferDefault* src,
			     GSML::sample_index buffer_index, bool is_master);

    virtual void Debug_impl () const;

    class track_data
    {
    public:
      GSML::TrackLayer::Selection* selection;
      GSML::TrackLayer::Selection::const_iterator current_segment;
      GSML::TrackLayer::Selection::const_iterator end;
      vector <LinearMonoFXPlug*> plug_vector;
      map <int, LinearMonoFXPlug*> m_plugs;
      bool did_fx;
    };

    vector <track_data> m_data;

    SigC::Threads::Mutex m_data_;

    void RebuildPlugVector (GSML::track_index track);
    void BuildTrackData (); // Also does UnplugAll and BuildSelection;
    void BuildSelection ();
    void BuildSelection (GSML::track_index track);
    void UnplugAll ();

    int current_key;
    GSML::DataLayer* m_layer;
  };   

}

#endif // GSMP_FX_MIXERSOURCE
