/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   GUI base.
 */

#ifndef GSMP_GUI_HH
#define GSMP_GUI_HH

#include "Base.hh"

namespace GSMP
{

  class App;
 
  namespace GUI
  {
    class Window;

    class Kit : public SigC::Object
    { // Abstract Base class for Gui stuff;
    public:
      SigC::Signal0<void> SignalGUIRefresh; // a signal to refresh the gui during idle cycles
      SigC::Signal0<bool> SignalAppExit;    // signaling an exit of the whole app - preventable by returning false

      virtual string ID () = 0;       // this returns a string which identifies the gui toolkit, eg. "GTK".
                                      // This string will be concatenated to the name of any visualisation Plugin
                                      // which is to be loaded by the App, to make shure it works with the App's
                                      // toolkit.
    protected:
      Kit (int argc, char *argv[])
      {
	DEBUG__ (GSMP, normal)
	  cout << "Created GUI Toolkit" << endl;
      }

      virtual ~Kit ()
      {
	DEBUG__ (GSMP, normal)
	  cout << "Destroyed GUI Toolkit" << endl;
      }

      int SlotGUIRefresh ()
      {
	SignalGUIRefresh.emit ();
	return 1;
      }
 
    private:
      friend class App;
      virtual void Run (App* i_app) = 0;   // Handles Execution to main event loop of the toolkit
      virtual void Quit () = 0;  // Signal Kit to finish up end return from Run ()....
    };

    class Window : public SigC::Object
    { // Abstract base class for drawn Windows
    public:
      virtual void PopTop () = 0; // pops Window to Top of Desktop;
      virtual void Hide () = 0; // "iconize" win
      virtual bool Kill () = 0; // delete Window; returns false, whenever this call did not succed.

      virtual void SetTitle (string title) = 0;
      virtual string Title () = 0;

      SigC::Signal0 <void> SignalDestroy; // Window recieved destroy event
    protected:
      Window ()
      {
	DEBUG__ (GSMP, detail)
	  cout << "Created GUI Window" << endl;
      }

    };

    class InitScreen
    { // BaseClass for some kind of splashscreen....
    public:
      virtual void Update (string status_msg, double progress) = 0;
      virtual ~InitScreen () {}
    };

  } // end namespace GUI;
} // end namespace GSMP

#endif // GSMP_GUI_HH
