/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* This code is mostly adapted from CoolPlayer's CPI_Player_CoDec_MPEG.c:
 * CoolPlayer - Blazing fast audio player.
 *   Copyright (C) 2000-2001 Niek Albers
 *   Copyright (C) 2001 Robert Leslie
 *
 *   Also released under the terms of the GPL v.2
 *
 * Because Robert Leslie pointed me to it to get an reference implementaion
 *   - THANKS!.
 *
 * Feel free to use it as mid-layer C++ API!
 *
 */

/* Short Description:
 *   Midlevel and C++ API for the libMAD !!! (mp3 library)
 */

#ifndef MAD_HELPER_H
#define MAD_HELPER_H

#include "generated/config.hh"

#if EXT_FIO_MP3 == 1

#include <iostream.h>
#include <stdio.h>
#include <string>
#include <deque>

#include <unistd.h>

// mutex
#include <sigc++/thread.h>

// I WANT A REAL OO only OS !!!!!!!!!!!

extern "C" {
#include "mad.h" // GPL mp3 lib
}

class MADHelper
{
public:
  
  // frame start table (needed (??) for efficent seek) / can be shared between MADHelpers ...
  struct FrameCache {
    
    FrameCache () {
      cout << "FrameCache: CTor: " << this << endl;
      used = 0;
    }
    
    ~FrameCache () {
      cout << "FrameCache: DTor: " << this << endl;
      if (used > 0)
	cout << "~FrameCache: BUG: in dtor and used > 0 ... !?!" << endl;
    }
    
    void Use () {
      SigC::Threads::MLock a (lock);
      ++ used;
    }
    
    void Unuse () {
      SigC::Threads::MLock a (lock);
      -- used;
    }
    
    SigC::Threads::Mutex lock;
    unsigned int used;
    
    std::deque <long> frames;
  };
  
  MADHelper ();
  ~MADHelper ();
  
  bool Open (const std::string& fname, MADHelper* cache_partner = 0);
  bool Close ();
  
  long Length ();
  int Tracks ();
  int Rate ();
  
  bool Read (int length, mad_fixed_t* buffer);
  bool Seek (long pos);
  
protected:
  
  void Cleanup ();
  bool ParseXing (struct mad_bitptr ptr, unsigned int bitlen);
  bool ScanHeader ();
  
  bool DecodeNext (bool frame_too);
  
  // scale MAD output
  void PackPCM (mad_fixed_t* pcm, unsigned int nsamples, mad_fixed_t const* ch1, mad_fixed_t const* ch2);
  
  std::string m_fname;
  int m_fd;                                 /* stream handle */
  
  long m_size;                              /* stream size, in bytes */
  
  signed long m_frames;                     /* number of current decoded frame */
  long m_byte_of_mp3frame;                  /* byte pos in mp3-stream */
  int m_sample_of_mp3frame;                 /* pcm_frame of current mp3-frame */
  int m_pcm_frames;                         /* total pcm_frames in file */
  
  int m_samplerate;                         /* samplerate */
  int m_tracks;                             /* tracks / channels */
  
  struct xing {
    long flags;
    unsigned long frames;
    unsigned long bytes;
    unsigned char toc [100];
    long scale;
  };
  xing m_xing;                              /* Xing VBR tag data */
  
  mad_stream m_stream;                      /* MAD stream structure */
  mad_frame m_frame;                        /* MAD frame structure */
  mad_synth m_synth;                        /* MAD synth structure */
  
  unsigned int m_samplecount;               /* samples output from current frame */
  
  mad_timer_t m_timer;                      /* current playing time position */
  mad_timer_t m_length;                     /* total playing time of current stream */
  unsigned long m_rate;                     /* bitrate sum for computing average */
  
  FrameCache* m_frame_cache;
  
  unsigned int m_buflen;                    /* input stream buffer vaild length */
  unsigned char m_buffer [40000];           /* input stream buffer - Rene: main src */ 
};

#ifdef TO_BE_USED

# define SAMPLE_DEPTH   16


enum {
  XING_FRAMES = 0x00000001L,
  XING_BYTES  = 0x00000002L,
  XING_TOC    = 0x00000004L,
  XING_SCALE  = 0x00000008L
};

#endif // TO_BE_USED

#endif // EXT_FIO_MP3 == 1

#endif // MAD_HELPER_H
