/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#include "Project.hh"
#include "Values.hh"

namespace GSMP
{

  class ParameterDialogAction
  {
  public:
    virtual FXBase* Build () = 0; // if implementation returns NULL, nothing will be done
  };

  class ParameterDialog : public SigC::Object
  {
  public:
    ParameterDialog (ParameterDialogAction* i_action, Project* i_project);
    virtual ~ParameterDialog ();

    // Hacks, TODO: replace theese Functions with Add (<some abstract base class>)
    virtual void AddDouble (double min, double max, double default_val, double* result, string label) = 0;
    virtual void AddCount (int min, int max, int default_val, int* result, string label) = 0;
    virtual void AddCheck (bool default_state, bool* result, string label) = 0;
  
    virtual void Show () = 0; // draws the dialog

  protected:
    void Launch ();

  private:
    ParameterDialogAction* m_action;
    Project* m_project;
  };




  class EnhParameterDialog;

  class DialogAction : public SigC::Object
  {
  public:
    virtual void Launch () = 0;
    virtual bool IsEnabled () = 0; // may be launched ?
    SigC::Signal1 <void, bool> SignalEnabledChanged;

    virtual void Init (EnhParameterDialog* dialog) = 0;  // called by EnhParameterDialog constr. 
    virtual void Reset (EnhParameterDialog* dialog) = 0; // reset to default

    virtual ~DialogAction () {};
  };


  class EnhParameterDialog : public SigC::Object
  {
  public:
    EnhParameterDialog (DialogAction* i_action);

    virtual void AddValue (Value* value, string group = "") = 0;

    virtual void Reset (string group = "") = 0; // Removes all widgets in group, "" removes all) = 0;
    virtual void Show () = 0; // draws the dialog

    virtual ~EnhParameterDialog ();
  protected:
    DialogAction* m_action;
  };

};
