/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#ifndef GSMP_PRE_HEAR_FX
#define GSMP_PRE_HEAR_FX

#include "Values.hh"
#include "FXBase.hh"
#include "FXMixerSource.hh"

namespace GSMP
{

  template <class PLUG_TYPE>
  class LinearMonoFXPlugFactory 
  {
  public:
    virtual PLUG_TYPE* Create () = 0;
  };

  template <class PLUG>
  class PrehearButton : public CheckVal, public SigC::Object
  {
  public:
    typedef void (PLUG::*DoubleFunc) (double*);
    typedef void (PLUG::*IndexFunc) (GSML::sample_index*);
    typedef void (PLUG::*BoolFunc) (bool*);


    PrehearButton (GSMP::Project* i_project, LinearMonoFXPlugFactory<PLUG>& i_factory);
    ~PrehearButton ();

    // Set Interactions, changes apply on next prehearing
    void SetInteractiveDouble (DoubleFunc d, double* src, Value& parameter);
    void SetInteractiveIndex (IndexFunc i, GSML::sample_index* src, Value& parameter);
    void SetInteractiveBool (BoolFunc b, bool* src, Value& parameter);

    // CheckVal Interface
    virtual bool State ();
    virtual bool Set (bool state);
    virtual bool Default ();

  private:
    struct InteractiveDouble
    {
      InteractiveDouble (DoubleFunc i_d, double* i_src, Value& i_parameter)
	: d (i_d), parameter (i_parameter) {src = i_src;}

      DoubleFunc d;
      double* src;
      Value& parameter;
    };

    struct InteractiveIndex
    {
      InteractiveIndex (IndexFunc i_i, GSML::sample_index* i_src, Value& i_parameter)
	: i (i_i), parameter (i_parameter) {src = i_src;}

      IndexFunc i;
      GSML::sample_index* src;
      Value& parameter;
    };

   struct InteractiveBool
    {
      InteractiveBool (BoolFunc i_b, bool* i_src, Value& i_parameter)
	: b (i_b), parameter (i_parameter) {src = i_src;}

      BoolFunc b;
      bool* src;
      Value& parameter;
    };


    struct ActivePlug
    {
      PLUG* plug;
      PlugHandle* handle;
    };

    vector <InteractiveDouble*> m_doubles;
    vector <InteractiveIndex*> m_indices;
    vector <InteractiveBool*> m_bools;

    vector <ActivePlug> m_active_plugs;

    void SlotAllUnplugged ();
    void Plug2ActiveTracks ();
    void UnplugAll ();

    bool m_state;

    GSMP::Project* m_project;
    FXMixerSource& m_fx_source;
    LinearMonoFXPlugFactory<PLUG>& m_factory;
  };

}

#include "Templates/PreHearFX.cc"

#endif // GSMP_PRE_HEAR_FX
