/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#ifndef GSMP_PRE_HEAR_FX
#  error "Never include this File in your code directly"
#  error "This file is automatically included by PreHearFX.hh"
#endif

#include <sigc++/bind.h>

namespace GSMP
{

  template <class PLUG>
  PrehearButton <PLUG>::PrehearButton (GSMP::Project* i_project, LinearMonoFXPlugFactory<PLUG>& i_factory)
    : m_fx_source (i_project -> FXFilter ()), m_factory (i_factory)
  {
    label = "Prehear on Playback";
    m_project = i_project;
    m_state = false;
    m_fx_source.SignalAllUnpluged.connect (slot (this, &PrehearButton::SlotAllUnplugged));
  }

  template <class PLUG>
  PrehearButton <PLUG>::~PrehearButton ()
  {
    UnplugAll ();

    for (vector <InteractiveDouble*>::iterator i = m_doubles.begin (); i != m_doubles.end (); i++)
      delete (*i);
    for (vector <InteractiveIndex*>::iterator j = m_indices.begin (); j != m_indices.end (); j++)
      delete (*j);
    for (vector <InteractiveBool*>::iterator k = m_bools.begin (); k != m_bools.end (); k++)
      delete (*k);
  }


  template <class PLUG>
  void PrehearButton <PLUG>::SetInteractiveDouble (DoubleFunc d, double* src, Value& parameter)
  {
    InteractiveDouble* t_double = new InteractiveDouble (d, src, parameter);
    m_doubles.push_back (t_double);
  }

  template <class PLUG>
  void PrehearButton <PLUG>::SetInteractiveIndex (IndexFunc i, GSML::sample_index* src, Value& parameter)
  {
    InteractiveIndex* t_index = new InteractiveIndex (i, src, parameter);
    m_indices.push_back (t_index);
  }

  template <class PLUG>
  void PrehearButton <PLUG>::SetInteractiveBool (BoolFunc b, bool* src, Value& parameter)
  {
    InteractiveBool* t_bool = new InteractiveBool (b, src, parameter);
    m_bools.push_back (t_bool);
  }


  template <class PLUG>
  bool PrehearButton <PLUG>::State ()
  {
    return m_state;
  }

  template <class PLUG>
  bool PrehearButton <PLUG>::Set (bool state)
  {
    if (state != m_state)
      {
	if (!state)
	  UnplugAll ();
	else
	  Plug2ActiveTracks ();

	m_state = state;
	ValueChanged ();
      }
    return m_state;
  }

  template <class PLUG>
  bool PrehearButton <PLUG>::Default ()
  {
    return false;
  }

  template <class PLUG>
  void PrehearButton <PLUG>::SlotAllUnplugged ()
  {
    cout << "SlotAllUnplugged" << endl;

    for (vector <ActivePlug>::iterator i = m_active_plugs.begin (); i != m_active_plugs.end (); i++)
      {
	delete i -> handle;
	delete i -> plug;
      }

    m_active_plugs.clear ();

    m_state = false;
    ValueChanged (); 
  }

  template <class PLUG>
  void PrehearButton <PLUG>::Plug2ActiveTracks ()
  {
    cout << "Plug2ActiveTracks" << endl;
    GSML::DataLayer* t_layer = m_project -> CurrentLayerPtr ();
    GSML::track_index max_tracks = min (t_layer -> Tracks (), m_fx_source.PlaybackTracks ());


    for (GSML::track_index i = 0; i < max_tracks; i++)
      {
	if ((*t_layer) [i] -> selection.IsActive ())
	  {
	    ActivePlug t_plug;
	    t_plug . plug = m_factory.Create ();
	    t_plug . handle = m_fx_source.AttachPlug (t_plug . plug, i);
	    m_active_plugs.push_back (t_plug);

	    // Connect Signals
	    for (vector <InteractiveDouble*>::iterator c_double = m_doubles.begin (); c_double != m_doubles.end (); c_double++)
	      {
		InteractiveDouble* t_double = *c_double;
		t_double -> parameter.ValueChanged.connect (bind (slot (t_plug.plug, t_double -> d), t_double -> src));
	      }

	    for (vector <InteractiveIndex*>::iterator c_index = m_indices.begin (); c_index != m_indices.end (); c_index++)
	      {
		InteractiveIndex* t_index = *c_index;
		t_index -> parameter.ValueChanged.connect (bind (slot (t_plug.plug, t_index -> i), t_index -> src));
	      }

	    for (vector <InteractiveBool*>::iterator c_bool = m_bools.begin (); c_bool != m_bools.end (); c_bool++)
	      {
		InteractiveBool* t_bool = *c_bool;
		t_bool -> parameter.ValueChanged.connect (bind (slot (t_plug.plug, t_bool -> b), t_bool -> src));
	      }
	  }
      }
  }

  template <class PLUG>
  void PrehearButton <PLUG>::UnplugAll ()
  {
    cout << "UnplugAll" << endl;

    for (vector <ActivePlug>::iterator i = m_active_plugs.begin (); i != m_active_plugs.end (); i++)
      {
	m_fx_source.DetachPlug (i -> handle);
	delete i -> plug;
      }

    m_active_plugs.clear ();
  }

}; // end namespace GSMP
