/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#ifndef GSMP_VALUES_HH
#define GSMP_VALUES_HH

#include "deque"
#include "Basics.hh"

namespace GSMP
{

  class Value
  {
  public:
    string label;
    virtual ~Value () {};

    SigC::Signal0<void> ValueChanged;
    SigC::Signal0<void> Changed; // including Limits !
  };

  class Scaleable : public Value
  {
  public:
    enum Behavior
      {
	LINEAR,
	SQUARE,
	CUBIC,
	LOG,
	EXP
      };

    virtual Behavior Scale () = 0;
  };

  class Selection : public Value
  {
  public:
    typedef deque <string>::const_iterator const_iterator;

    const_iterator begin ();
    const_iterator end ();

    virtual void Select (const_iterator i);
  protected:
    deque <string> preselections;
  };

  class CounterValue : public Value
  {
  public:
    virtual int Min () = 0;
    virtual int Max () = 0;
    virtual int Default () = 0;
    virtual int Set (int val) = 0;
  };

  class ShortTime : public Scaleable
  { // usualy shown in ms or samples
  public:
    virtual GSML::sample_index Get () = 0;
    virtual GSML::sample_index Min () = 0;
    virtual GSML::sample_index Max () = 0;
    virtual GSML::sample_index Default () = 0;
    virtual GSML::sample_index Set (GSML::sample_index) = 0;
    virtual GSML::sample_index MinStepping () = 0;

    virtual int SamplesPerSecond () = 0;
  };

  class Ratio : public Value
  { // usualy shown as percentage or x:y
  public:
    virtual double Min () = 0;
    virtual double Max () = 0;
    virtual double Default () = 0;
    virtual double Set (double val) = 0;
    virtual double MinStepping () = 0;
  };

  class VolumeFactor : public Value
  { // always square scale
  public:
    virtual double Get () = 0;
    virtual double Min () = 0;
    virtual double Max () = 0;
    virtual double Default () = 0;
    virtual double Set (double val) = 0;
  };

  class VolumeArray : public Value
  { // TODO ; for equs etc;
  public:
  
  };

  class Frequency : public Scaleable
  { // Absolute, pi = 0.5 * samplerate
  public:
    virtual double Get () = 0;
    virtual double Min () = 0;
    virtual double Max () = 0;
    virtual double Default () = 0;
    virtual double Set (double val) = 0;

    virtual int SamplesPerSecond () = 0;
  };

  class FrequencyFactor : public Scaleable
  { // Relative
  public:
    virtual double Get () = 0;
    virtual double Min () = 0;
    virtual double Max () = 0;
    virtual double Default () = 0;
    virtual double Set (double val) = 0;
  };

  class CheckVal : public Value
  { // a simple yes/no question
  public:
    virtual bool State () = 0;
    virtual bool Set (bool state) = 0;
    virtual bool Default () = 0;
  };

} // end namespace GSMP

#endif // GSMP_VALUES_HH
