/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Helper Classes to configure single values with constant ranges
 */

#include "Values.hh"

namespace GSMP
{

  using namespace GSML;

  class SingleCounter : public CounterValue
  {
  public:
    SingleCounter (int min_v, int max_v, int def_v, int& result, string name);

    virtual int Min ();
    virtual int Max ();
    virtual int Default ();
    virtual int Set (int val);

  private:
    int m_min;
    int m_max;
    int m_def;
    int &res;
  };

  class SingleVolume : public VolumeFactor
  {
  public:
    SingleVolume (double min_v, double max_v, double def_v, double& result, string name);

    virtual double Get ();
    virtual double Min ();
    virtual double Max ();
    virtual double Default ();
    virtual double Set (double val);

  private:
    double& res;
    double m_min, m_max, def;
  };

  class SingleCheck : public CheckVal
  {
  public:
    SingleCheck (bool def_state, bool& result, string name);

    virtual bool State ();
    virtual bool Set (bool state);
    virtual bool Default ();

  private:
    bool& res;
    bool def;
  };

  class SingleShortTime : public ShortTime
  {
  public:
    SingleShortTime (sample_index min_t, sample_index max_t, sample_index def_t, sample_index& result,
		     int samples_per_second, string name, sample_index stepping = 1, bool linear = true);

    virtual sample_index Get ();
    virtual sample_index Min ();
    virtual sample_index Max ();
    virtual sample_index Default ();
    virtual sample_index Set (GSML::sample_index);
    virtual sample_index MinStepping ();

    virtual Scaleable::Behavior Scale ();
    virtual int SamplesPerSecond ();
  private:
    sample_index m_min;
    sample_index m_max;
    sample_index def;
    sample_index& res;
    int sps;
    sample_index step;
    bool lin;
  };


  class SingleFrequency : public Frequency
  { // logarithmic scale
  public:
    SingleFrequency (double min_f, double max_f, double def_f, double& result, int samples_per_second, string name);

    virtual double Get ();
    virtual double Min ();
    virtual double Max ();
    virtual double Default ();
    virtual double Set (double val);

    virtual Scaleable::Behavior Scale ();
    virtual int SamplesPerSecond ();
  private:
    double& res;
    double m_min, m_max, def;
    int sps;
  };
}
