/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   View interface.
 */

#ifndef GSMP_VIEW_HH
#define GSMP_VIEW_HH

// GSMP stuff
#include "App.hh"
#include "FindMemCorruption.hh"

namespace GSMP
{
  class ViewImpl
  {
  public:
    virtual void SetStart (GSML::sample_index n_start) = 0;
    virtual GSML::sample_index Start () const = 0;
    
    virtual void SetZoom (GSML::sample_index n_zoom_factor) = 0;
    virtual void SetZoomAll () = 0; // auto-rescale to zoom-all
    virtual int Zoom () const = 0;
    
    virtual GSML::sample_index Width () const = 0; // visible screen width
    virtual GSML::sample_index Length () const = 0; // visible real length
    
    virtual GSML::sample_index Pointer () const = 0; // mouse-pointer pos
    
    // TODO methods to "dock" some FX Parameter config stuff...
  
    virtual void SetTrackVisible (GSML::track_index track_no, bool status) = 0;
    virtual bool IsTrackVisible (GSML::track_index track_no) const = 0;

    virtual string Name () const;

    virtual ViewImpl* Clone (View* n_view) = 0;
    
    SigC::Signal1<void, GSML::sample_index> SignalZoomChanged;
    SigC::Signal1<void, GSML::sample_index> SignalStartChanged;
    
    SigC::Signal1<void, GSML::sample_index> SignalPointerMoved;

    virtual ~ViewImpl () {}    
  protected:
    ViewImpl (View* i_view);
    View* m_view;
  };


  class View : public Menu::MenuData
  {
  public:

    Project* GetProject ();
    const Project* GetProject () const;

    SigC::Signal0 <void> SignalViewImplExchanged;
    SigC::Signal0 <void> SignalWindowExchanged;

    SigC::Signal0 <bool, GUIRequestMarshaller<bool> > SignalEraseRequest;  // someone is going to kill this View, preventable by returning false.
    SigC::Signal0 <void> SignalErased;

    ViewImpl* GetViewImpl () {return m_current_view;}   // A valid View always has one...
    string ViewImplType () {return m_current_impl_type;}

    void SwitchToViewImpl (const string& view_impl_type);

    bool IsVisible (); // Is this View displayed in any Window ?
    SimpleWindow* MyWindow (); // if not visible: 0

  private:
    friend class Project;
    friend class SimpleWindow;
    friend class MainWindow;

    View (Project* i_project);
    View (const View& copy_from);
    ~View ();

    void SetWindow (SimpleWindow* new_window);

  protected:
    Project*   m_project;
    App*       m_app;
    ViewImpl*  m_current_view;
    string     m_current_impl_type;
    SimpleWindow* m_win;

    FindMemCorruption m_fmc;
  };


}; // end namespace GSMP

#endif // GSMP_VIEW_HH
