/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *  abstract (not depending on Toolkit) Window interface.
 */

#ifndef GSMP_WINDOW_HH__
#define GSMP_WINDOW_HH__

// GSMP stuff
#include "GUI.hh"
#include "Menu.hh"

namespace GSMP
{

  class App;
  class View;
  class Project;

  class SimpleWindow : public SigC::Object
  {
  public:
    View* GetView ();
    const View* GetView () const;
    virtual GUI::Window* Win () = 0;

    // ViewImpl has been exchanged. If Object is a MainWindow, this could also
    // mean, the View has been exchanged ! 
    SigC::Signal0 <void> SignalViewImplChanged;
    // Window deleted !
    SigC::Signal0 <void> SignalDeleted;
 protected:
    SimpleWindow (App* i_app, View* i_view);
    virtual ~SimpleWindow ();

    // TODO: reconnect SlotNameChanged !

    void SlotNameChanged ();
    void SlotViewErased ();
    void SlotProjectErased ();

    virtual void ViewErased_Impl () { Win () -> Kill (); }
    virtual void ProjectErased_Impl () { Win () -> Kill (); }

    Menu::MenuCollection* m_main_menu_l;
    Menu::MenuCollection* m_main_menu_r;

    SigC::Connection view_impl_changed;
    SigC::Connection name_changed;
    SigC::Connection project_erased;
    SigC::Connection view_erased;

    App* m_app;
    View* m_view;
  };

  class MainWindow : public SimpleWindow
  {
  public:
    void SwitchToView (View* new_view);
 
  protected:
    MainWindow (App* i_app, View* i_view);
    virtual ~MainWindow ();

    void SwitchToView_impl (View* new_view);

    virtual void ViewErased_Impl ();
    virtual void ProjectErased_Impl ();
  };

}; // end namespace GSMP

#endif // GSMP_WINDOW_HH__
