/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#include "DefaultWindows.hh"

using namespace GSMP;
using namespace Gtk;

DefaultMainWindowGtk::DefaultMainWindowGtk (App* i_app, View* i_view)
  : MainWindow (i_app, i_view)
{
  DEBUG__(GSMP_GTK, detail)
    cout << "instanciating DefaultMainWindowGtk" << endl;

  m_menubar = new MenuBarGtk (i_view, m_main_menu_l, m_main_menu_r);
  m_statusbar = new StatusBar (i_view);

  m_impl = dynamic_cast <Gtk::Widget*> (i_view -> GetViewImpl ());
  if (!m_impl)
    {
      cout << "WARNING: ViewImpl is not a gtk widget !" << endl;
    }

  m_box.set_spacing (0);
  m_box.set_border_width (0);

  m_box.pack_start ( *(manage (m_menubar)), false, false, 0);
  m_box.pack_start (*m_impl);
  m_box.pack_start ( *(manage (m_statusbar)), false, false, 0);
  add (m_box);

  SignalViewImplChanged . connect (slot (this, &DefaultMainWindowGtk::SlotViewImplChanged));

  SetTitle (i_view -> GetProject () -> Name ());

  // set_policy etc
  set_name ("GSMP");
  //  set_default_size (600, 350);

  show_all ();
}

void DefaultMainWindowGtk::SlotViewImplChanged ()
{
  DEBUG__(GSMP_GTK, detail)
    cout << "SlotViewImplChanged" << endl;

  m_box . remove (*m_menubar);
  m_box . remove (*m_impl);
  m_box . remove (*m_statusbar);
  m_impl = dynamic_cast <Gtk::Widget*> (GetView () -> GetViewImpl ());

  m_menubar = new MenuBarGtk (GetView (), m_main_menu_l, m_main_menu_r);
  m_statusbar = new StatusBar (GetView ());
  
  m_box.pack_start ( *(manage (m_menubar)), false, false, 0);
  m_box.pack_start (*m_impl);
  m_box.pack_start ( *(manage (m_statusbar)), false, false, 0);

  show_all ();
}

DefaultMainWindowGtk::~DefaultMainWindowGtk ()
{
  DEBUG__(GSMP_GTK, detail)
    cout << "Destroying DefaultMainWindowGtk" << endl;
}

gint DefaultMainWindowGtk::delete_event_impl(GdkEventAny*)
{
  m_box . remove (*m_impl);
  return 0;
}

GUI::Window* DefaultMainWindowGtk::Win ()
{
  return this;
}

GUI::Window* DefaultSimpleWindowGtk::Win ()
{
  return this;
}
