/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *
 * Baseclass for Gtk-- Dialogs which can be displayed and destroyed via
 * a checked menu item
 *
 */

#include "Dialog.hh"

using namespace GSMP; 


Dialog:: Dialog (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view, string i_location)
: StatusActivator (i_app, i_project, i_view)
{
  m_location = i_location;
  m_dialog = 0;
}

Dialog::~Dialog ()
{
  if (m_dialog)
    {
      slot_dialog_deleted . disconnect ();
      delete m_dialog;
    }
}

bool Dialog::Execute (bool state)
{
  if (state && (!m_dialog) )
    {
      m_dialog = Create ();

      slot_dialog_deleted = m_dialog -> destroy . connect (slot (this, &Dialog::SlotDialogDeleted) );
    }
  else if (m_dialog && (!state) )
    {
      slot_dialog_deleted . disconnect ();
      delete m_dialog;
      m_dialog = 0;
    }

  MakeParasite ();
  return (m_dialog);
}

void Dialog::SlotDialogDeleted ()
{
  Menu::CheckedItemFinder t_item (*(m_app -> GetManager () ), m_location);

  Project* t_project = (m_project) ? m_project : ( *(m_app -> BeginProjects () ) );
  View* t_view = (m_view) ? m_view : ( *(t_project -> BeginViews () ) );
  t_item . Get () -> SetCheck (t_view, false);
}

