/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The File dialog.
 */

#include "FileDialog.hh"
#include "ExtFileIO_impl.hh"

using namespace GSMP;

FileDialog::FileDialog (Project* i_project, int i_mode)
{
  m_project = i_project;
  m_mode = i_mode;
  
  switch (m_mode)
    {
    case 0: set_title ("File Import");
      break;
    case 1: set_title ("File Export (currently .wav only)");
      break;
    case 2: set_title ("File Mix-Export (currently .wav only)");
      break;
    default:
      set_title ("File ??? (programmer made a typo ;-)");
    }
  
  get_ok_button ()->clicked.connect (slot (this, &FileDialog::slot_ok ) );
  get_cancel_button ()->clicked.connect (destroy.slot() );
  
  set_filename ("*.wav");
  
  show_all ();
}

FileDialog::~FileDialog ()
{
  cout << "YEAH: We get deleted ... (FileDialog)" << endl;
}

void FileDialog::slot_ok ()
{
  string fname = get_filename ();
  
  string ext = fname;
  
  string::size_type idx = fname.rfind ('.');
  if (idx != string::npos)
    ext = fname.substr (idx + 1);
  
  cout << "Extension: " << ext << endl;
  
  switch (m_mode)
    {
    case 0:
      {
	cout << "do_import" << endl;
	
	ImportFile* t_file_obj = 0;
	
#if EXT_FIO_MP3 == 1
	if (ext == "mp3")
	  t_file_obj = new MP3ImportFile ();
	else
#endif
#if EXT_FIO_OGGVORBIS == 1
	if (ext == "ogg")
	  t_file_obj = new OggVorbisImportFile ();
	else 
#endif
	if (ext == "wav")
	  t_file_obj = new WavImportFile ();
	else
	  {
	    cout << "Error: no Importer for: " << ext << ". Trying wav." << endl;
	    t_file_obj = new WavImportFile ();
	  }
	
	FileImporter* t_importer = new FileImporter (fname, t_file_obj);
	m_project -> DoFX (t_importer);
      }
      break;
    case 1:
      {
	cout << "do_export" << endl;
	WavExportFile* t_file_obj = new WavExportFile ();
	FileExporter* t_exporter = new FileExporter (fname, t_file_obj);
	
	m_project -> DoFX (t_exporter);
      }
      break;
    case 2:
      {
	cout << "do_mix_export" << endl;
	WavExportFile* t_file_obj = new WavExportFile ();
	MixExporter* t_exporter = new MixExporter (fname, t_file_obj);
	
	m_project -> DoFX (t_exporter);
      }
      break;
    default:
      cout << "BUG: mode not coded ..." << endl;
    }
  destroy ();
}

void ImportDialogX :: Execute ()
{
  FileDialog* dialog = manage (new FileDialog (m_project, 0) );
}

void ExportDialogX :: Execute ()
{
  FileDialog* dialog = manage (new FileDialog (m_project, 1) );
}

void MixExportDialogX :: Execute ()
{
  FileDialog* dialog = manage (new FileDialog (m_project, 2) );
}
