/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The IO control dialog.
 */

#include "IODialog.hh"
#include "sstream"
#include "IO.hh"

#include <gtk--/table.h>


using namespace GSMP;
using namespace GSML;

IODialog::IODialog (Project* i_project)
  : start_io (*(i_project -> BeginViews ()), string ("IO>Start IO")),
    stop_io (*(i_project -> BeginViews ()), string ("IO>Stop IO")),
    do_playback (*(i_project -> BeginViews ()), string ("IO>Do Playback")),
    do_capture (*(i_project -> BeginViews ()), string ("IO>Do Capture")),
    sel_play (*(i_project -> BeginViews ()), string ("IO>Play selections only")),
    sel_capture (*(i_project -> BeginViews ()), string ("IO>Capture in selections only")),
    hear_fx (*(i_project -> BeginViews ()), string ("IO>Prehear FX")),
    do_loop (*(i_project -> BeginViews ()), string ("IO>Loop IO")),
    do_stop_at_end (*(i_project -> BeginViews ()), string ("IO>Stop at end")),
    close_button ("Close"),
    io_status ("IO status"),
    pl_label ("", 0.0),
    ca_label ("", 0.0)
{
  m_project = i_project;

  connection_view_erased = (*(i_project -> BeginViews ())) -> SignalErased . connect (slot (this, &IODialog::SlotViewErased));

  string t_title = string ("IO Control") + m_project -> Name (); 
  set_title (t_title);

  // *(m_project -> BeginViews ())

  //time.set_label ( m_project -> PlaybackTime ());

  get_vbox () -> set_border_width (10);
  //get_vbox () -> set_spacing (10);

  start_stop_box . add (start_io);
  start_stop_box . add (stop_io);
  start_stop_box . set_border_width (5);

  Gtk::VBox* t_io_box = manage (new Gtk::VBox);

  queue.set_bar_style (GTK_PROGRESS_DISCRETE);

  t_io_box -> pack_start (pl_label);
  t_io_box -> pack_start (queue);
  t_io_box -> pack_start (ca_label);

  io_status.add (*t_io_box);

  get_vbox () -> add (start_stop_box);
  get_vbox () -> add (do_playback);
  get_vbox () -> add (do_capture);
  get_vbox () -> add (sel_play);
  get_vbox () -> add (sel_capture);
  get_vbox () -> add (hear_fx);
  get_vbox () -> add (do_loop);
  get_vbox () -> add (do_stop_at_end);
  get_vbox () -> add (do_capture);
  get_vbox () -> add (time);
  get_vbox () -> add (io_status);
  
  close_button.clicked.connect (destroy.slot ());
  get_action_area () -> pack_start (close_button);

  show_all ();
  
  m_project -> SignalTimeRefresh.connect (slot (this, &IODialog::SlotTimeRefresh));
  //SlotTimeRefresh ();
  old_blocks = 0;
}

void IODialog::SlotTimeRefresh ()
{
  time.set (m_project -> PlaybackTime ());
  const GSML::Mixer::io_status& status =  m_project -> GetMixer () -> GetIOStatus ();

  pl_label.set (string ("PL: XRuns: ") + itos (status.pl_xruns)
	       + string (", Queue Outs: ") + itos (status.pl_data_queue_outs) );
  
  ca_label.set (string ("Ca: XRuns: ") + itos (status.ca_xruns)
	       + string (", Queue length: ") + itos (status.ca_data_queue) );

  if (old_blocks != status.pl_data_queue_max)
    {
      old_blocks = status.pl_data_queue_max;
      queue.set_discrete_blocks (status.pl_data_queue_max);
    }

  queue.set_percentage (GSML::limit_max ( ((double) status.pl_data_queue) / ((double) status.pl_data_queue_max), 1.0) );
  
  show_all ();
}

void IODialog::SlotViewErased ()
{
  cout << "IODialog::SlotViewErased";
  connection_view_erased . disconnect ();
  if (m_project -> BeginViews () != m_project -> EndViews ())
    {
      View* t_view = *(m_project -> BeginViews ()); 
      connection_view_erased = t_view -> SignalErased . connect (slot (this, &IODialog::SlotViewErased));

      start_io . SetView (t_view);
      stop_io . SetView (t_view);
      do_playback . SetView (t_view);
      do_capture . SetView (t_view);
      sel_play . SetView (t_view);
      sel_capture . SetView (t_view);
      hear_fx . SetView (t_view);
      do_loop . SetView (t_view);
      do_stop_at_end . SetView (t_view);
    }
  else
    cout << "No more views left...." << endl;
}

Gtk::Widget* IODialogX::Create ()
{
  return new IODialog (m_project);
}
