/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Layer history dialog.
 */

#include "LayerHistoryDialog.hh"

#include <gtk--/button.h>

using namespace GSMP;
using namespace GSML;

LayerHistoryDialog::LayerHistoryDialog (Project* i_project)
{
  m_project = i_project;
  internal_select = false;
  
  string t_title = string ("Layer History ") + m_project -> Name ();

  set_title (t_title);
  set_usize (450, 280);
  
  get_vbox () -> set_border_width (10);
  get_vbox () -> set_spacing (10);
  
  m_scrolled_window = manage (new Gtk::ScrolledWindow () );
  m_scrolled_window -> set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  get_vbox () -> pack_start (*m_scrolled_window);
  
  static const gchar *titles[] = {"No.", "Comment", "Tracks", "Length", "Fixated", NULL};
  m_clist = manage (new Gtk::CList (titles) );
  
  m_clist->column (1) . set_auto_resize (true);
  m_clist->column (2) . set_auto_resize (true);
  m_clist->column (3) . set_auto_resize (true);
  
  m_clist -> select_row.connect (slot (this, &LayerHistoryDialog::slot_selected) );
  m_scrolled_window -> add (*m_clist);
  
  l_prepared_layer = manage (new Gtk::Label ("Prepared Layer:") );
  get_vbox () -> pack_start (*l_prepared_layer, false);
  
  Gtk::Button* t_button = manage (new Gtk::Button ("Close") );
  t_button -> clicked .connect (destroy . slot() );
  get_action_area () -> pack_start (*t_button);
  
  m_project -> SignalCurrentLayerChanged () . connect (slot (this, &LayerHistoryDialog::SlotCurrentLayerChanged) );
  m_project -> SignalNewLayerPrepared () . connect (slot (this, &LayerHistoryDialog::SlotNewLayerPrepared) );
  m_project -> SignalNewLayerFixated () . connect (slot (this, &LayerHistoryDialog::SlotNewLayerFixated) );
  m_project -> SignalNewLayerCanceled () . connect (slot (this, &LayerHistoryDialog::SlotNewLayerCanceled) );
  
  build_list ();

  show_all ();
}

LayerHistoryDialog::~LayerHistoryDialog ()
{
  cout << "YEAH: We get deleted ... (LayerHistoryDialog)" << endl;
}

void LayerHistoryDialog::SlotCurrentLayerChanged (DataLayer* n_layer)
{
  build_list ();
}

void LayerHistoryDialog::SlotNewLayerPrepared (DataLayer* n_layer)
{
  string text = "Prepared Layer: " + itos ( (int)n_layer);
  l_prepared_layer -> set_text (text);
}

void LayerHistoryDialog::SlotNewLayerFixated ()
{
  string text = "Prepared Layer:";
  l_prepared_layer -> set_text (text);
}

void LayerHistoryDialog::SlotNewLayerCanceled ()
{
  string text = "Prepared Layer:";
  l_prepared_layer -> set_text (text);
}

void LayerHistoryDialog::slot_selected (gint row, gint column, GdkEvent* event)
{
  cout << "LayerHistoryDialog::slot_selected:" << endl;
  if (internal_select)
    return;
  
  Data::iterator i = m_project -> BeginLayer ();
  while (row != 0 && i != m_project -> EndLayer () )
    {
      -- row;
      ++ i;
    }
  
  if (i != m_project->EndLayer () )
    {
      cout << "Setting layer ..." << endl;
      m_project -> SetCurrentLayer (i);
    }
  else
    cout << "Error: layer invaild!" << endl;
}

void LayerHistoryDialog::build_list ()
{
  internal_select = true;
  
  m_clist -> clear ();
  
  Data::iterator i = m_project->BeginLayer ();
  for (; i != m_project->EndLayer (); ++ i)
    {
      vector<std::string> text;
      text . push_back (itos (i - m_project -> BeginLayer () ) );
      text . push_back ( (*i) -> Comment () );
      text . push_back (itos ( (*i) -> Tracks () ) );
      text . push_back (itos ( (int) (*i) -> Length () ) );
      text . push_back (itos ( (int) (*i) -> IsFixated () ) );
      
      m_clist -> rows () . push_back (text);
      
      if (i == m_project -> CurrentLayer () )
	{
	  m_clist -> rows () . back () . select ();
	  m_clist -> cell (i - m_project -> BeginLayer(), 0) . moveto ();
	}
    }
  internal_select = false;
}


Gtk::Widget* LayerHistoryDialogX::Create ()
{
  return new LayerHistoryDialog (m_project);
}
