/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Mixer dialog.
 */

#include "Dialog.hh"

#include <gtk--/box.h>
#include <gtk--/scale.h>
#include <gtk--/label.h>
#include <gtk--/drawingarea.h> 
#include <gtk--/dialog.h>

class MixerScale : public Gtk::VBox
{
public:
  
  MixerScale ();
  ~MixerScale ();
  
  void SetValue (GSML::pcm_default::value_type value);
  
  SigC::Signal1 <void, GSML::pcm_default::value_type> SignalChanged;
  
protected:
  
  void SlotAdjustmentChanged ();
  
  double dB_fs;
  
  bool internal_changed;
  
  Gtk::VScale m_scale;
  Gtk::Label m_label;
  
  Gtk::Adjustment* m_adjustment;
};


class LEDArray : public Gtk::DrawingArea
{
public:
  LEDArray (int i_width, int i_height, double i_max_perc);
  ~LEDArray ();
  
  void SetKeepPeaks (bool state);
  bool IsKeepPeaks ();
  
  void SetPeakDelay (int time);
  int PeakDelay ();
  
  void SetFallOff (bool state);
  bool IsFallOff ();
  
  void SetFallOffStep (double step);
  double FallOffStep ();
  
  void Clear ();
  void Draw (double perc);
  
protected:
  virtual gint configure_event_impl (GdkEventConfigure* event);
  virtual gint expose_event_impl (GdkEventExpose* event);
  
  Gdk_Window win;
  Gdk_Visual visual;
  Gdk_Pixmap pixmap;
  
  Gdk_GC back_gc;
  Gdk_GC marker_gc;
  Gdk_GC green_gc;
  Gdk_GC red_gc;
  
  int m_width;
  int m_height;
  bool configured;
  
  double m_max_perc;
  double m_last_perc;
  
  GSML::Timer m_peak_timer;
  bool m_keep_peaks;
  int m_peak_delay;
  double m_peak_perc;
  
  bool m_do_falloff;
  double m_falloff_step;
};

class MixerTrackMatrixWidget : public Gtk::VBox
{
public:
  MixerTrackMatrixWidget (GSML::UniversalMixerParams* i_mixer_params, GSML::Mixer* i_mixer, GSML::track_index i_track, bool is_pl);
  ~MixerTrackMatrixWidget ();

  void GUIRefresh ();

private:
  void SlotChannelsChanged (GSML::track_index n_channels, bool do_show);
  
  void SlotTrackValueChanged (GSML::track_index track, GSML::pcm_default::value_type value);
  void SlotMatrixValueChanged (GSML::track_index track, GSML::track_index channel, GSML::pcm_default::value_type value);
  void SlotTrackMuteChanged (GSML::track_index track, bool n_state);
  
  void SlotTrackScaleValueChanged (GSML::pcm_default::value_type value);
  void SlotMatrixScaleValueChanged (GSML::pcm_default::value_type value, GSML::track_index channel);
   
  Gtk::HBox m_hbox;
  MixerScale m_scale;
  LEDArray m_vu_meter;
  
  vector<MixerScale*> m_scales;
  
  bool internal_changed;
  bool m_io_was_running;
  
  GSML::UniversalMixerParams* m_mixer_params;
  GSML::Mixer* m_mixer;
  GSML::track_index m_track;
  bool m_is_pl;
};

class MixerTrackMatrixDialog : public Gtk::Dialog
{
public:
  MixerTrackMatrixDialog (GSMP::Project* i_project, GSML::UniversalMixerParams* i_mixer_params, GSML::Mixer* i_mixer, GSML::track_index i_track, bool is_pl);
  ~MixerTrackMatrixDialog ();
  
private:
  
  void SlotTracksChanged (GSML::track_index tracks);
  void SlotGUIRefresh ();
  
  MixerTrackMatrixWidget* m_track_matrix;
  
  GSML::UniversalMixerParams* m_mixer_params;
  GSML::Mixer* m_mixer;
  GSML::track_index m_track;
  GSMP::Project* m_project;
  bool m_is_pl;
};

class MixerTrackWidget : public Gtk::VBox
{
public:
  MixerTrackWidget (GSMP::Project* i_project, GSML::UniversalMixerParams* i_mixer_params, GSML::Mixer* i_mixer,
		    GSML::track_index i_track, bool is_pl);
  ~MixerTrackWidget ();

  void GUIRefresh ();
  
private:
  
  void SlotValueChanged (GSML::track_index track, GSML::pcm_default::value_type value);
  void SlotMuteChanged (GSML::track_index track, bool n_state);
  
  void SlotScaleValueChanged (GSML::pcm_default::value_type value);
  void SlotMuteButtonToggled ();
  
  void SlotChmButtonToggled ();
  
  void SlotMTMDDeleted ();

  Gtk::HBox m_hbox;
  MixerScale m_scale;
  LEDArray m_vu_meter;
  Gtk::ToggleButton m_mute_button;
  Gtk::ToggleButton m_chm_button;
  
  bool internal_changed;
  bool m_io_was_running;
  
  GSML::UniversalMixerParams* m_mixer_params;
  GSML::Mixer* m_mixer;
  GSML::track_index m_track;
  bool m_is_pl;
  
  GSMP::Project* m_project;
  MixerTrackMatrixDialog* m_mixer_track_matrix_dialog;
};

class MixerTrackDialog : public Gtk::Dialog
{
public:
  MixerTrackDialog (GSMP::Project* i_project, GSML::UniversalMixerParams* i_params, bool is_pl);
  ~MixerTrackDialog ();
  
private:
  
  void build_tracks ();
  void SlotTracksChanged (GSML::track_index tracks);
  
  void SlotGUIRefresh ();
  
  Gtk::HBox m_hbox;
  
  deque <MixerTrackWidget*> m_tracks;
  
  GSMP::Project* m_project;
  GSML::UniversalMixerParams* m_mixer_params;
  bool m_is_pl;
};

class MixerChannelWidget : public Gtk::VBox
{
public:
  MixerChannelWidget (GSML::UniversalMixerParams* i_mixer_params, GSML::Mixer* i_mixer,
		      GSML::track_index i_channel, bool is_pl);
  ~MixerChannelWidget ();
  
  void GUIRefresh ();
  
private:
  
  void SlotValueChanged (GSML::track_index channel, GSML::pcm_default::value_type value);
  void SlotMuteChanged (GSML::track_index channel, bool n_state);
  
  void SlotScaleValueChanged (GSML::pcm_default::value_type value);
  void SlotMuteButtonToggled ();
  
  Gtk::HBox m_hbox;
  MixerScale m_scale;
  LEDArray m_vu_meter;
  Gtk::ToggleButton m_mute_button;
  
  bool internal_changed;
  bool m_io_was_running;
  
  GSML::UniversalMixerParams* m_mixer_params;
  GSML::Mixer* m_mixer;
  GSML::track_index m_channel;
  bool m_is_pl;
};

class MixerChannelDialog : public Gtk::Dialog
{
public:
  MixerChannelDialog (GSMP::Project* i_project, GSML::UniversalMixerParams* i_mixer_params, bool is_pl);
  ~MixerChannelDialog ();
  
private:
  
  void build_channels ();
  void SlotChannelsChanged (GSML::track_index channels);
  
  void SlotGUIRefresh ();
  
  Gtk::HBox m_hbox;
  
  deque <MixerChannelWidget*> m_channels;
  
  GSMP::Project* m_project;
  GSML::UniversalMixerParams* m_mixer_params;
  bool m_is_pl;
};

class PLChannelDialogX : public Dialog
{
public:
  PLChannelDialogX (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Dialog (i_app, i_project, i_view, string ("Dialogs>Playback channel mix")) {};
  
  virtual Gtk::Widget* Create ();
};

class CAChannelDialogX : public Dialog
{
public:
  CAChannelDialogX (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Dialog (i_app, i_project, i_view, string ("Dialogs>Capture channel mix")) {};
  
  virtual Gtk::Widget* Create ();
};

class PLTrackDialogX : public Dialog
{
public:
  PLTrackDialogX (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Dialog (i_app, i_project, i_view, string ("Dialogs>Playback track mix")) {};
  
  virtual Gtk::Widget* Create ();
};

class CATrackDialogX : public Dialog
{
public:
  CATrackDialogX (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Dialog (i_app, i_project, i_view, string ("Dialogs>Capture track mix")) {};
  
  virtual Gtk::Widget* Create ();
};


