/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#include "ParamDialogs.hh"

#include <gtk--/radiomenuitem.h> 

#include <math.h>
#include <sstream>
#include <streambuf.h>

using namespace Gtk;

ParameterDialogGtk::ParameterDialogGtk (GSMP::ParameterDialogAction* i_action, GSMP::Project* i_project, string i_title)
  : ParameterDialog (i_action, i_project),
    ok_button ("Ok"),
    cancel_button ("Cancel")
{
  set_title (i_title);
  get_vbox () -> set_border_width (10);

  Gtk::HBox* t_area = get_action_area ();

  ok_button.clicked.connect (slot (this, &ParameterDialogGtk::SlotOkClicked));
  cancel_button.clicked.connect (destroy.slot ());

  t_area -> pack_start (ok_button);
  t_area -> pack_start (cancel_button);
}

void  ParameterDialogGtk::AddDouble (double min, double max, double default_val, double* result, string label)
{
  Gtk::VBox* t_box = get_vbox ();

  Gtk::Label* t_label = manage (new Gtk::Label (label));
  Gtk::Adjustment* t_adjustment = manage (new Gtk::Adjustment (default_val, min, max));
  DoubleValueSlider* t_slider = manage (new DoubleValueSlider (*t_adjustment, result));

  *result = default_val;

  t_box -> pack_start (*t_label);
  t_box -> pack_start (*t_slider);
}

void ParameterDialogGtk::AddCount (int min, int max, int default_val, int* result, string label)
{
  Gtk::VBox* t_box = get_vbox ();

  Gtk::Label* t_label = manage (new Gtk::Label (label));
  Gtk::Adjustment* t_adjustment = manage (new Gtk::Adjustment (default_val, min, max));
  Counter* t_counter = manage (new Counter (*t_adjustment, result));

  *result = default_val;

  t_box -> pack_start (*t_label);
  t_box -> pack_start (*t_counter);
}

void ParameterDialogGtk::AddCheck (bool default_state, bool* result, string label)
{
  Gtk::VBox* t_box = get_vbox ();

  Check* t_check = manage (new Check (default_state, result, label));

  *result = default_state;

  t_box -> pack_start (*t_check);
}



void  ParameterDialogGtk::Show ()
{
  show_all ();
}

void ParameterDialogGtk::SlotOkClicked ()
{
  Launch ();
  destroy ();
}


DoubleValueSlider::DoubleValueSlider (Gtk::Adjustment& adjustment, double* dst_value)
  : Gtk::HScale (adjustment)
{
  m_dst_value = dst_value;
  get_adjustment () -> value_changed.connect (slot (this, &DoubleValueSlider::SlotValueChanged)); 
}

void DoubleValueSlider::SlotValueChanged ()
{
  *m_dst_value = get_adjustment () -> get_value ();
}


Counter::Counter (Gtk::Adjustment& adjustment, int* dst_value)
  : Gtk::SpinButton (adjustment, 1)
{
  m_dst_value = dst_value;
  get_adjustment () -> value_changed.connect (slot (this, &Counter::SlotValueChanged));
}

void Counter::SlotValueChanged ()
{
  *m_dst_value = (int) ( get_adjustment () -> get_value () );
}


Check::Check (bool i_state, bool* dst_state, string label)
  : Gtk::CheckButton (label)
{
  set_active (i_state);
  m_dst_state = dst_state;
  toggled.connect (slot (this, &Check::SlotToggled)); 
}

void Check::SlotToggled ()
{
  *m_dst_state = get_active ();
}

/********************************************************************************************************/

/**************************/

class Hack1 : public VolumeFactorInput
{
public:
  Hack1 (GSMP::VolumeFactor* i_f) : VolumeFactorInput (i_f) {}

  virtual string Name () {return string ("Hack1");}
  virtual bool Match (string input) {return (input == "h1");}
  virtual Gtk::Widget* CreateInput () {return new Gtk::Button ("Hack 1");}
  virtual string str () {return string ("h1");}
  virtual string str (double val) {return string ("xx");}
};

class Hack2 : public VolumeFactorInput
{
public:
  Hack2 (GSMP::VolumeFactor* i_f) : VolumeFactorInput (i_f) {}

  virtual string Name () {return string ("Hack2");}
  virtual bool Match (string input) {return (input == "h2");}
  virtual Gtk::Widget* CreateInput () {return new Gtk::Button ("Hack 2");}
  virtual string str () {return string ("h2");}
  virtual string str (double val) {return string ("xx");}
  virtual unsigned int Priority () {return 1;}
};

class Hack3 : public VolumeFactorInput
{
public:
  Hack3 (GSMP::VolumeFactor* i_f) : VolumeFactorInput (i_f) {}

  virtual string Name () {return string ("Hack3");}
  virtual bool Match (string input) {return (input == "h3");}
  virtual Gtk::Widget* CreateInput () {return new Gtk::Button ("Hack 3");}
  virtual string str () {return string ("h3");}
  virtual string str (double val) {return string ("xx");}
  virtual bool Inlined () {return false;}
};


/**************************/

class GenericSlider : public Gtk::HBox
{
public:
  GenericSlider ();

  void SetBehavior (GSMP::Scaleable::Behavior behavior);
  void SetLimits (double min, double max, double default_value);
  void SetTexts (string min, string max, string default_value);
  void SetStepping (double step, double master);
  void SnapToSteps (bool state);

  void Set (double value);
  void Set2Min ();
  void Set2Max ();
  void Set2Def ();

  SigC::Signal1 <void, double> Changed;

private:
  gint SlotPressedContext (GdkEventButton* event);
  void SlotIncClicked ();
  void SlotDecClicked ();
  int GetCurrentStep (); 

  void SlotValueChanged ();

  Gtk::HScale scale;
  Gtk::Button inc;
  Gtk::Button dec;
  Gtk::Button conf; 
  Gtk::Button chain;
  Gtk::Menu context;
  Gtk::Menu chain_menu;
  Gtk::RadioMenuItem_Helpers::Group group;

  double m_min;
  double m_max;
  double m_step;
  double m_default;
  double m_step_master;
  GSMP::Scaleable::Behavior m_behavior;
  bool snap;

  int current_step;

  double m_value;

  double Real (double virt); // slider pos in [0,1]  -> value in [min,max];
  double Virt (double real); // value in [min,max] -> slider pos in [0,1];

  bool internal_change; // recursion killing
  bool recalc_current_step; // work around lousy gtk-- scale precission
};

/*********************************************************/

class VolumeSliderWidget : public GenericSlider
{
public:
  VolumeSliderWidget (VolumeFactorInput& input, GSMP::VolumeFactor* fact);

private:
  void SlotSliderChanged (double new_value);
  void SlotValueChanged ();
  void SlotLimitsChanged ();

  bool internal_changed; // recursion killing

  GSMP::VolumeFactor* m_factor;
  VolumeFactorInput& m_input;
};

class DbInput : public VolumeFactorInput
{
public:
  DbInput (GSMP::VolumeFactor* fact) : VolumeFactorInput (fact) {}

  virtual string Name () {return string ("dB");}
  virtual bool Match (string input);
  virtual Gtk::Widget* CreateInput ();
  virtual string str ();
  virtual string str (double val);
  virtual unsigned int Priority () {return 10;}
};

class VolPercentInput : public VolumeFactorInput
{
public:
  VolPercentInput (GSMP::VolumeFactor* fact) : VolumeFactorInput (fact) {}

  virtual string Name () {return string ("%");}
  virtual bool Match (string input);
  virtual Gtk::Widget* CreateInput ();
  virtual string str ();
  virtual string str (double val);
};


/**************************/

class FrequencySliderWidget : public GenericSlider
{
public:
  FrequencySliderWidget (FrequencyInput& input, GSMP::Frequency* freq);

private:
  void SlotSliderChanged (double new_value);
  void SlotValueChanged ();
  void SlotLimitsChanged ();

  bool internal_changed; // recursion killing

  GSMP::Frequency* m_freq;
  FrequencyInput& m_input;
};


class HzInput : public FrequencyInput
{
public:
  HzInput (GSMP::Frequency* freq) : FrequencyInput (freq) {samples_per_second = freq -> SamplesPerSecond ();}

  virtual string Name () {return string ("Hz");}
  virtual bool Match (string input);
  virtual Gtk::Widget* CreateInput ();
  virtual string str ();
  virtual string str (double val);
  virtual unsigned int Priority () {return 10;}

private:
  double samples_per_second;
};


/**************************/

class ShortTimeSliderWidget : public GenericSlider
{
public:
  ShortTimeSliderWidget (ShortTimeInput& input, GSMP::ShortTime* time);

private:
  void SlotSliderChanged (double new_value);
  void SlotValueChanged ();
  void SlotLimitsChanged ();

  bool internal_changed; // recursion killing

  GSMP::ShortTime* m_time;
  ShortTimeInput& m_input;
};

class MsInput : public ShortTimeInput
{
public:
  MsInput (GSMP::ShortTime* time) : ShortTimeInput (time) {samples_per_second = time -> SamplesPerSecond ();}

  virtual string Name () {return string ("ms");}
  virtual bool Match (string input);
  virtual Gtk::Widget* CreateInput ();
  virtual string str ();
  virtual string str (GSML::sample_index val);
  virtual unsigned int Priority () {return 10;}

private:
  double samples_per_second;
};


/*************************************/

EnhParameterDialogGtk::EnhParameterDialogGtk (GSMP::DialogAction* i_action, string i_title)
  : EnhParameterDialog (i_action),
    ok_button ("Ok"),
    reset_button ("Reset"),
    cancel_button ("Cancel")
{
  set_title (i_title);
  HBox* t_area = get_action_area ();

  ok_button.set_sensitive (m_action -> IsEnabled ());
  m_action -> SignalEnabledChanged.connect (slot (this, &EnhParameterDialogGtk::SlotOkEnabledChanged));

  cancel_button.clicked.connect (destroy.slot ());
  reset_button.clicked.connect (bind (slot (m_action, &GSMP::DialogAction::Reset), this));
  ok_button.clicked.connect (slot (this, &EnhParameterDialogGtk::SlotOkClicked));

  t_area -> pack_start (ok_button);
  t_area -> pack_start (reset_button);
  t_area -> pack_start (cancel_button);

  m_box . set_spacing (5);
  m_box . set_border_width (10);
  get_vbox () -> pack_start (m_box);

  m_action -> Init (this);
}

EnhParameterDialogGtk::~EnhParameterDialogGtk ()
{
  for (map <string, deque <GenericInput*> > :: iterator i = m_active_inputs.begin (); i != m_active_inputs.end (); i++)
    for (deque <GenericInput*> :: iterator j = i -> second.begin (); j != i -> second.end (); j++)
      delete (*j);
}

void EnhParameterDialogGtk::SlotOkClicked ()
{
  m_action -> Launch ();
  destroy ();
}

void EnhParameterDialogGtk::SlotOkEnabledChanged (bool new_state)
{
  ok_button.set_sensitive (new_state);
}

void EnhParameterDialogGtk::AddValue (GSMP::Value* value, string group = "")
{
  GSMP::VolumeFactor* volume_factor;
  GSMP::Frequency* frequency_val;
  GSMP::CheckVal* check_val;
  GSMP::ShortTime* short_time;

  Widget* t_new_widget = 0;

  // Try to cast config value

  if ((frequency_val = dynamic_cast <GSMP::Frequency*> (value)))
    {
      GenericInputWidget* t_widget = new GenericInputWidget (value, value -> label);

      GenericInput* t_input = new HzInput (frequency_val);
      t_widget -> Add (t_input);
      m_active_inputs [group] . push_back (t_input);

      t_widget -> Show ();
      t_new_widget = t_widget;
    }

  else if ((short_time = dynamic_cast <GSMP::ShortTime*> (value)))
    {
      GenericInputWidget* t_widget = new GenericInputWidget (value, value -> label);

      GenericInput* t_input = new MsInput (short_time);
      t_widget -> Add (t_input);
      m_active_inputs [group] . push_back (t_input);

      t_widget -> Show ();
      t_new_widget = t_widget;
    }

  else if ((volume_factor = dynamic_cast <GSMP::VolumeFactor*> (value)))
    {
      GenericInputWidget* t_widget = new GenericInputWidget (value, value -> label);

      GenericInput* t_input = new VolPercentInput (volume_factor);
      t_widget -> Add (t_input);
      m_active_inputs [group] . push_back (t_input);

      // Hack !!!
      if (volume_factor -> Min () >= 0.0)
	{
	  t_input = new DbInput (volume_factor);
	  t_widget -> Add (t_input);
	  m_active_inputs [group] . push_back (t_input);
	}

      t_widget -> Show ();
      t_new_widget = t_widget;
    }


  else if (check_val = dynamic_cast <GSMP::CheckVal*> (value))
    t_new_widget = new CheckedInput (check_val, check_val -> label);

  if (!t_new_widget)
    {
      cout << "Warning : Config value of unknown type !" << endl;
      return;
    }
  // Now Add new widget to Dialog

  if (group == "")
    m_box.pack_start (* manage (t_new_widget));

  else
    {
      Gtk::Frame* t_frame;

      cout << "Frame" << endl;

      map <string, Frame* > :: iterator f = m_groups.find (group);
      if (f == m_groups.end ())
	{
	  cout  << "New Frame" << endl;

	  t_frame = new Frame (group);
	  //t_frame -> set_shadow_type (GTK_SHADOW_IN);
	  VBox* t_vbox = new VBox;
	  t_vbox -> set_spacing (5);
	  t_vbox -> set_border_width (5);
	  t_frame -> add (*manage (t_vbox));
	  m_box . pack_start (* manage (t_frame));
	  m_groups [group] = t_frame;
	}
      else
	t_frame = f -> second;

      VBox* t_box = dynamic_cast <VBox*> (t_frame -> get_child ());
      t_box -> pack_start (* manage (t_new_widget));
    }

}

void EnhParameterDialogGtk::Reset (string group = "")
{
  if (group == "")
    {
      m_box.children () . clear ();

      for (map <string, deque <GenericInput*> > :: iterator i = m_active_inputs.begin (); i != m_active_inputs.end (); i++)
	for (deque <GenericInput*> :: iterator j = i -> second.begin (); j != i -> second.end (); j++)
	    delete (*j);
      
      m_active_inputs.clear ();
      m_groups.clear ();
    }
  else
    {
      map <string, Frame* > :: iterator f = m_groups.find (group);
      if (f == m_groups.end ())
	{
	  cout << "Critical: ParameterDialogGtk was told to reset non-existent group " << group << endl;
	  return;
	}

      m_box.remove (*(f -> second));
      m_groups.erase (f);

      map <string, deque <GenericInput*> > :: iterator i = m_active_inputs.find (group);
      if (i == m_active_inputs.end ())
	{
	  cout << "Critical: ParameterDialogGtk holds no input deque for group " << group << endl;
	  return;
	
	  for (deque <GenericInput*> :: iterator j = i -> second.begin (); j != i -> second.end (); j++)
	    delete (*j);
 
	  m_active_inputs.erase (i);
	}
    }

  Show ();
}

void EnhParameterDialogGtk::Show ()
{
  show_all ();
}





CheckedInput::CheckedInput (GSMP::CheckVal* i_val, const string& i_label)
  : CheckButton (i_label, 0.0)
{
  m_val = i_val;
  set_active (m_val -> State ());
  internal_change = false;

  toggled.connect (slot (this, &CheckedInput::SlotToggled));
  m_val -> ValueChanged . connect (slot (this, &CheckedInput::SlotChanged));
}


void CheckedInput::SlotToggled ()
{
  if (internal_change)
    return;

  internal_change = true;
  bool t_state = get_active ();
  if (t_state != m_val -> Set (t_state))
    set_active (!t_state);
  internal_change = false;
}

void CheckedInput::SlotChanged ()
{
  if (internal_change)
    return;

  internal_change = true;
  set_active (m_val -> State () );
  internal_change = false;
}





GenericInputWidget::GenericInputWidget (GSMP::Value* i_value, const string& i_label)
  : Table (3, 2, false), label (i_label)
{
  m_value = i_value;

  max_priority = 0;
  max_priority_input = 0;
  active_input = 0;

  internal_change = false;

  available_inputs.set_usize (75,0);
  available_inputs.set_menu (*(manage (new Gtk::Menu) ));

  entry.set_usize (75,0);
  entry.activate.connect ( slot (this, &GenericInputWidget::SlotActivate));

  m_value -> ValueChanged.connect (slot (this, &GenericInputWidget::SlotValueChanged));
  m_value -> Changed.connect (slot (this, &GenericInputWidget::SlotValueChanged));
}

void GenericInputWidget::Add (GenericInput* input)
{
  using namespace Menu_Helpers;

  m_inputs.push_back (input);

  if (input -> Priority () >= max_priority)
    {
      max_priority = input -> Priority ();
      max_priority_input = input;
    }

  MenuList& t_list = available_inputs.get_menu () -> items ();
  t_list.push_back (MenuElem (input -> Name ()));
  t_list.back () -> activate.connect (bind (slot (this, &GenericInputWidget::Draw), input));

  current_input = 0;
}

void GenericInputWidget::Show ()
{
  Draw (max_priority_input);
  show_all ();
}

void GenericInputWidget::Draw (GenericInput* input)
{
  using namespace Menu_Helpers;

  cout << "Draw" << endl;

  if (internal_change)
    return;

  internal_change = true;

  if (current_input)
    remove (*current_input);
  current_input = input -> CreateInput ();

  if (active_input)
    {
      if (input -> Inlined ())
	{
	  if (!inlined)
	    {
	      remove (label);
	      remove (entry);
	      remove (available_inputs);
	      attach (label, 0, 3, 0, 1, (GTK_FILL|GTK_EXPAND), 0);
	      attach (available_inputs, 0, 1, 1, 2, 0, 0);
	      attach (entry, 2, 3, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
	      inlined = true;
	    }
	  attach (*(manage (current_input)), 1, 2, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
	}
      else
	{
	  if (inlined)
	    {
	      remove (label);
	      remove (entry);
	      remove (available_inputs);
	      attach (label, 0, 1, 0, 1, (GTK_FILL|GTK_EXPAND), 0);
	      attach (available_inputs, 1, 2, 0, 1, 0, 0);
	      attach (entry, 2, 3, 0, 1, (GTK_FILL|GTK_EXPAND), 0);
	      inlined = false;
	    }
	  attach (*(manage (current_input)), 0, 3, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
	}
    }
  else
    { // drawing for the very first time
      if (input -> Inlined ())
	{
	  attach (label, 0, 3, 0, 1, (GTK_FILL|GTK_EXPAND), 0);
	  attach (available_inputs, 0, 1, 1, 2, 0, 0);
	  attach (*(manage (current_input)), 1, 2, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
	  attach (entry, 2, 3, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
	  inlined = true;
	}
      else
	{
	  attach (label, 0, 1, 0, 1, (GTK_FILL|GTK_EXPAND), 0);
	  attach (available_inputs, 1, 2, 0, 1, 0, 0);
	  attach (*(manage (current_input)), 0, 3, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
	  attach (entry, 2, 3, 0, 1, (GTK_FILL|GTK_EXPAND), 0);
	  inlined = false;
	}
    }

  active_input = input;
  entry.set_text (input -> str ());

  int pos = 0;

  for (deque <GenericInput*> :: iterator i = m_inputs.begin (); i != m_inputs.end (); i++, pos++)
    if ((*i) == input)
	available_inputs . set_history (pos);


  show_all ();
  internal_change = false;
}

void GenericInputWidget::SlotActivate ()
{
  if (internal_change)
    return;

  string t_match = entry.get_text ();
  if (! active_input -> Match (t_match))
    {
      if (max_priority_input -> Match (t_match))
	Draw (max_priority_input);
      else
	{
	  for (deque <GenericInput*> :: iterator i = m_inputs.begin (); i != m_inputs.end (); i++)
	    if ((*i) -> Match (t_match))
	      {
		Draw (*i);
		return;
	      }
	}
    }
}

void GenericInputWidget::SlotValueChanged ()
{
  internal_change = true;
  entry.set_text (active_input -> str ());
  internal_change = false;
}

VolumeFactorInput::VolumeFactorInput (GSMP::VolumeFactor* fact)
{
  m_factor = fact;
}

FrequencyInput::FrequencyInput (GSMP::Frequency* freq)
{
  m_freq = freq;
}

ShortTimeInput::ShortTimeInput (GSMP::ShortTime* time)
{
  m_time = time;
}





GenericSlider::GenericSlider ()
  : inc ("+"), dec ("-"), conf ("v"), chain ("ch")
{
  using namespace Menu_Helpers;

  scale.set_draw_value(false);
  scale.set_usize (250, 0); 
  scale.get_adjustment () -> set_upper (1.0);
  scale.get_adjustment () -> set_lower (0.0);
  scale.set_digits (5);
  scale.get_adjustment () -> value_changed.connect (slot (this, &GenericSlider::SlotValueChanged));
 
  m_min = 0;
  m_max = 1;
  m_step = 0.01;
  m_default = 1;
  m_step_master = m_default;
  snap = false;
  m_behavior = GSMP::Scaleable::LINEAR;

  m_value = m_default;
  current_step = 0;

  MenuList& t_list = context.items ();
  t_list.push_back (MenuElem ("min"));
  t_list.back () -> activate.connect (slot (this, &GenericSlider::Set2Min));

  t_list.push_back (MenuElem ("max"));
  t_list.back () -> activate.connect (slot (this, &GenericSlider::Set2Max));

  t_list.push_back (MenuElem ("default"));
  t_list.back () -> activate.connect (slot (this, &GenericSlider::Set2Def));

  t_list.push_back (SeparatorElem ());
  t_list.push_back (MenuElem ("copy"));
  t_list.push_back (SeparatorElem ());

  t_list.push_back (RadioMenuElem (group, "linear"));
  t_list.back () -> activate.connect (bind (slot (this, &GenericSlider::SetBehavior), GSMP::Scaleable::LINEAR));

  t_list.push_back (RadioMenuElem (group, "square"));
  t_list.back () -> activate.connect (bind (slot (this, &GenericSlider::SetBehavior), GSMP::Scaleable::SQUARE));

  t_list.push_back (RadioMenuElem (group, "cubic"));
  t_list.back () -> activate.connect (bind (slot (this, &GenericSlider::SetBehavior), GSMP::Scaleable::CUBIC));

  t_list.push_back (RadioMenuElem (group, "log"));
  t_list.back () -> activate.connect (bind (slot (this, &GenericSlider::SetBehavior), GSMP::Scaleable::LOG));

  t_list.push_back (RadioMenuElem (group, "exp"));
  t_list.back () -> activate.connect (bind (slot (this, &GenericSlider::SetBehavior), GSMP::Scaleable::EXP));

  inc.clicked.connect (slot (this, &GenericSlider::SlotIncClicked));
  dec.clicked.connect (slot (this, &GenericSlider::SlotDecClicked));

  conf.button_press_event.connect (slot (this, &GenericSlider::SlotPressedContext));

  pack_start (scale, true);
  pack_start (dec, false);
  pack_start (inc, false);
  pack_start (conf, false);
  pack_start (chain, false);

  internal_change = false;
  recalc_current_step = true;

  Set (m_value);

  show_all ();
}

void GenericSlider::SetBehavior (GSMP::Scaleable::Behavior behavior)
{
  using namespace Menu_Helpers;

  if (internal_change)
    return;

  cout << "SET BEHAVIOR !!!!" << behavior <<  endl;
  m_behavior = behavior;

  // set option menu
  MenuList::iterator pos = context.items () . begin ();

  switch (m_behavior)
    {
    case GSMP::Scaleable::EXP:
      pos ++;
    case GSMP::Scaleable::LOG:
      pos ++;
    case GSMP::Scaleable::CUBIC:
      pos ++;
    case GSMP::Scaleable::SQUARE:
      pos ++;
    case GSMP::Scaleable::LINEAR:
      pos ++;
      pos ++;
      pos ++;
      pos ++;
      pos ++;
      pos ++;
    };

  internal_change = true;
  (*pos) -> activate (); 
  internal_change = false;

  Set (m_value);
}


void GenericSlider::SetLimits (double min, double max, double default_value)
{
  m_min = min;
  m_max = max;
  m_default = default_value;
  Set (default_value);
}

void GenericSlider::SetTexts (string min, string max, string default_value)
{
  using namespace Menu_Helpers;

  MenuList::iterator pos = context.items () . begin ();

  MenuItem& t_item_min = **(pos++);
  t_item_min.remove ();
  t_item_min.add_label (min);

  MenuItem& t_item_max = **(pos++);
  t_item_max.remove ();
  t_item_max.add_label (max);

  MenuItem& t_item_def = **pos;
  t_item_def.remove ();
  t_item_def.add_label (default_value);
}

void GenericSlider::SetStepping (double step, double master)
{
  m_step = step;
  m_step_master = master;
  Set (m_value);
}

void GenericSlider::SnapToSteps (bool state)
{
  snap = state;
  Set (m_value);
}


void GenericSlider::Set (double value)
{
  scale.get_adjustment () -> set_value (Virt (value));
}


void GenericSlider::Set2Min ()
{
  Set (m_min);
}

void GenericSlider::Set2Max ()
{
  Set (m_max);
}

void GenericSlider::Set2Def ()
{
  Set (m_default);
}

gint GenericSlider::SlotPressedContext (GdkEventButton* event)
{
  context.popup (event -> button, event -> time);
  return 0;
}

void GenericSlider::SlotIncClicked ()
{
  if (m_value < m_max)
    {
      int t_step = ++current_step;
      double t_val = min ( ( ((double) t_step) * m_step ) + m_step_master , m_max);
      recalc_current_step = false;
      Set (t_val);
      recalc_current_step = true;
    }
}

void GenericSlider::SlotDecClicked ()
{
  if (m_value > m_min)
    {
      int t_step = --current_step;
      double t_val = max ( ( ((double) t_step) * m_step ) + m_step_master , m_min);
      recalc_current_step = false;
      Set (t_val);
      recalc_current_step = true;
    }
}


void GenericSlider::SlotValueChanged ()
{
  double t_val = scale.get_adjustment () -> get_value ();

  m_value = Real (t_val);

  if (recalc_current_step)
    {
      double t_step = (m_value - m_step_master) / m_step;
      current_step = (int) t_step;

      if (snap)
	{
	  m_value = min (max ( ( ((double) current_step) * m_step ) + m_step_master , m_min) , m_max);
	}
    }

  cout << current_step << "\t" << t_val << "\t--> " << m_value << "\t--> " << Virt (m_value) << endl;
  Changed (m_value);
}




double GenericSlider::Real (double val)
{
  double l = m_max - m_min;
  
  double x_trans;

  switch (m_behavior)
    {
    case GSMP::Scaleable::LINEAR:
      x_trans = val;
      break;
    case GSMP::Scaleable::SQUARE:
      x_trans = val*val;
      break;
    case GSMP::Scaleable::CUBIC:
      x_trans = val*val*val;
      break;
    case GSMP::Scaleable::EXP:
      x_trans = exp (val * log (2.0)) - 1.0;
      break;
    case GSMP::Scaleable::LOG:
      x_trans = log (((exp (1.0)- 1.0) * val ) + 1.0);
      break;
    };

  return (x_trans * l) + m_min;
}


double GenericSlider::Virt (double val)
{
  double l = m_max - m_min;
  double x_retrans = (val - m_min) / l;

  switch (m_behavior)
    {
    case GSMP::Scaleable::LINEAR:
      return x_retrans;
      break;
    case GSMP::Scaleable::SQUARE:
      return sqrt (x_retrans);
      break;
    case GSMP::Scaleable::CUBIC:
      return pow (x_retrans, 1.0/3.0);
      break;
    case GSMP::Scaleable::EXP:
      return  log (x_retrans + 1.0) / log (2.0);
      break;
    case GSMP::Scaleable::LOG:
      return (exp (x_retrans) - 1.0) / (exp (1.0) - 1.0); 
      break;
    };
}


VolumeSliderWidget::VolumeSliderWidget (VolumeFactorInput& input, GSMP::VolumeFactor* fact)
  : m_input (input)
{
  m_factor = fact;

  SetLimits (m_factor -> Min (), m_factor -> Max (), m_factor -> Default ());
  SetTexts (m_input.str (m_factor -> Min ()), m_input.str (m_factor -> Max ()), m_input.str (m_factor -> Default ()));
  SetStepping ((m_factor -> Max () - m_factor -> Min ()) / 500, m_factor -> Default ());
  SnapToSteps (false);
  SetBehavior (GSMP::Scaleable::SQUARE);

  Set (m_factor -> Get ());

  internal_changed = false;

  m_factor -> ValueChanged.connect (slot (this, &VolumeSliderWidget::SlotValueChanged));
  m_factor -> Changed.connect (slot (this, &VolumeSliderWidget::SlotLimitsChanged));
  Changed.connect (slot (this, &VolumeSliderWidget::SlotSliderChanged)); 
}

void VolumeSliderWidget::SlotSliderChanged (double new_value)
{
  if (internal_changed)
    return;

  cout << "VolumeSliderWidget::SlotSliderChanged" << endl;

  internal_changed = true;
  m_factor -> Set (new_value);
  internal_changed = false;
}

void VolumeSliderWidget::SlotValueChanged ()
{
  if (internal_changed)
    return;

  cout << "VolumeSliderWidget::SlotValueChanged" << endl;

  internal_changed = true;
  Set (m_factor -> Get ());
  internal_changed = false;
}

void VolumeSliderWidget::SlotLimitsChanged ()
{
  cout << "VolumeSliderWidget::SlotLimitsChanged" << endl;

  SetLimits (m_factor -> Min (), m_factor -> Max (), m_factor -> Default ());
  SetTexts (m_input.str (m_factor -> Min ()), m_input.str (m_factor -> Max ()), m_input.str (m_factor -> Default ()));
  SetStepping ((m_factor -> Max () - m_factor -> Min ()) / 500, m_factor -> Default ());
  Set (m_factor -> Get ());
}



FrequencySliderWidget::FrequencySliderWidget (FrequencyInput& input, GSMP::Frequency* freq)
  : m_input (input)
{
  m_freq= freq;

  SetLimits (m_freq -> Min (), m_freq -> Max (), m_freq -> Default ());
  SetTexts (m_input.str (m_freq -> Min ()), m_input.str (m_freq -> Max ()), m_input.str (m_freq -> Default ()));
  SetStepping ((m_freq -> Max () - m_freq -> Min ()) / 500, m_freq -> Default ());
  SnapToSteps (false);
  SetBehavior (m_freq -> Scale ());

  Set (m_freq -> Get ());

  internal_changed = false;

  m_freq -> ValueChanged.connect (slot (this, &FrequencySliderWidget::SlotValueChanged));
  m_freq -> Changed.connect (slot (this, &FrequencySliderWidget::SlotLimitsChanged));
  Changed.connect (slot (this, &FrequencySliderWidget::SlotSliderChanged)); 
}

void FrequencySliderWidget::SlotSliderChanged (double new_value)
{
  if (internal_changed)
    return;

  cout << "FrequencySliderWidget::SlotSliderChanged" << endl;

  internal_changed = true;
  m_freq -> Set (new_value);
  internal_changed = false;
}

void FrequencySliderWidget::SlotValueChanged ()
{
  if (internal_changed)
    return;

  cout << "FrequencySliderWidget::SlotValueChanged" << endl;

  internal_changed = true;
  Set (m_freq -> Get ());
  internal_changed = false;
}

void FrequencySliderWidget::SlotLimitsChanged ()
{
  cout << "FrequencySliderWidget::SlotLimitsChanged" << endl;

  SetLimits (m_freq -> Min (), m_freq -> Max (), m_freq -> Default ());
  SetTexts (m_input.str (m_freq -> Min ()), m_input.str (m_freq -> Max ()), m_input.str (m_freq -> Default ()));
  SetStepping ((m_freq -> Max () - m_freq -> Min ()) / 500, m_freq -> Default ());
  SetBehavior (m_freq -> Scale ());
  Set (m_freq -> Get ());
}




ShortTimeSliderWidget::ShortTimeSliderWidget (ShortTimeInput& input, GSMP::ShortTime* time)
  : m_input (input)
{
  m_time = time;

  SetLimits (m_time -> Min (), m_time -> Max (), m_time -> Default ());
  SetTexts (m_input.str (m_time -> Min ()), m_input.str (m_time -> Max ()), m_input.str (m_time -> Default ()));
  SetStepping (m_time -> MinStepping (), m_time -> Default ());
  SnapToSteps (true);
  SetBehavior (m_time -> Scale ());

  Set (m_time -> Get ());

  internal_changed = false;

  m_time -> ValueChanged.connect (slot (this, &ShortTimeSliderWidget::SlotValueChanged));
  m_time -> Changed.connect (slot (this, &ShortTimeSliderWidget::SlotLimitsChanged));
  Changed.connect (slot (this, &ShortTimeSliderWidget::SlotSliderChanged)); 
}

void ShortTimeSliderWidget::SlotSliderChanged (double new_value)
{
  if (internal_changed)
    return;

  cout << "ShortTimeSliderWidget::SlotSliderChanged" << endl;

  internal_changed = true;
  m_time -> Set ((GSML::sample_index) new_value);
  internal_changed = false;
}

void ShortTimeSliderWidget::SlotValueChanged ()
{
  if (internal_changed)
    return;

  cout << "ShortTimeSliderWidget::SlotValueChanged" << endl;

  internal_changed = true;
  Set (m_time -> Get ());
  internal_changed = false;
}

void ShortTimeSliderWidget::SlotLimitsChanged ()
{
  cout << "ShortTimeSliderWidget::SlotLimitsChanged" << endl;

  SetLimits (m_time -> Min (), m_time -> Max (), m_time -> Default ());
  SetTexts (m_input.str (m_time -> Min ()), m_input.str (m_time -> Max ()), m_input.str (m_time -> Default ()));
  SetStepping (m_time -> MinStepping (), m_time -> Default ());
  SetBehavior (m_time -> Scale ());
  Set (m_time -> Get ());
}





bool DbInput::Match (string input)
{
  cout << "INPUT " << input << endl; 

  stringstream s;
  s << input;

  double val;
  string text;
  string unit;

  bool got_val = false;
  bool got_text = false;
  bool got_unit = false;

  if ((s >> val))
    {
      cout << "  got val: " << val << endl;
      got_val = true;
    }
  else 
    {
      s.clear ();
      if ((s >> text))
	{
	  cout << "  got text: " << text << endl;
	  got_text = true; 
	}
    }

  if ((s >> unit))
    {
      cout << "  got unit: " << unit << endl;
      got_unit = true;
    }

  if (!s.eof ())
    return false;

  if (got_text && text != "-inf")
    return false;

  if (got_unit && unit != "dB")
    return false;

  if (! (got_val || got_text))
    return false;

  if (got_text)
    m_factor -> Set (0);
  else
    m_factor -> Set (exp ((val/10.0) * log (2)));

  return true;
}


Gtk::Widget* DbInput::CreateInput ()
{
  return new VolumeSliderWidget (*this, m_factor);
}


string DbInput::str ()
{
  return str (m_factor -> Get ());
}


string DbInput::str (double val)
{
  if (val == 0)
    return string ("-inf dB");

  val = 10.0 * (log (val) / log (2));

  stringstream s;
  s.setf(ios::fixed, ios::floatfield);
  s.precision (2);
  s << val << " dB";

  return s.str ();
}



bool VolPercentInput::Match (string input)
{
  cout << "INPUT " << input << endl; 

  stringstream s;
  s << input;

  double val;
  string unit;

  bool got_val = false;
  bool got_unit = false;

  if ((s >> val))
    {
      cout << "  got val: " << val << endl;
      got_val = true;
    }
  else 
    return false;

  if ((s >> unit))
    {
      cout << "  got unit: " << unit << endl;
      got_unit = true;
    }

  if (!s.eof ())
    return false;

  if (got_unit && unit != "%")
    return false;

  m_factor -> Set (val/100.0);
  return true;
}


Gtk::Widget* VolPercentInput::CreateInput ()
{
  return new VolumeSliderWidget (*this, m_factor);
}


string VolPercentInput::str ()
{
  return str (m_factor -> Get ());
}


string VolPercentInput::str (double val)
{
  stringstream s;
  s.setf(ios::fixed, ios::floatfield);
  s.precision (2);

  s << (val * 100.0) << " %";

  return s.str ();
}





bool HzInput::Match (string input)
{
  cout << "INPUT " << input << endl; 

  stringstream s;
  s << input;

  double val;
  string unit;

  bool got_val = false;
  bool got_unit = false;

  if ((s >> val))
    {
      cout << "  got val: " << val << endl;
      got_val = true;
    }
  else 
    return false;

  if ((s >> unit))
    {
      cout << "  got unit: " << unit << endl;
      got_unit = true;
    }

  if (!s.eof ())
    return false;

  if (got_unit && unit != "Hz")
    return false;

  m_freq -> Set (2.0 * M_PI * val / samples_per_second);
  return true;
}


Gtk::Widget* HzInput::CreateInput ()
{
  return new FrequencySliderWidget (*this, m_freq);
}


string HzInput::str ()
{
  return str (m_freq -> Get ());
}


string HzInput::str (double val)
{
  stringstream s;
  s.setf(ios::fixed, ios::floatfield);
  s.precision (2);

  s << (val * samples_per_second / (2.0 * M_PI)) << " Hz";

  return s.str ();
}





bool MsInput::Match (string input)
{
  cout << "INPUT " << input << endl; 

  stringstream s;
  s << input;

  double val;
  string unit;

  bool got_val = false;
  bool got_unit = false;

  if ((s >> val))
    {
      cout << "  got val: " << val << endl;
      got_val = true;
    }
  else 
    return false;

  if ((s >> unit))
    {
      cout << "  got unit: " << unit << endl;
      got_unit = true;
    }

  if (!s.eof ())
    return false;

  if (got_unit && unit != "ms")
    return false;

  m_time -> Set (val * samples_per_second / 1000.0);
  return true;
}


Gtk::Widget* MsInput::CreateInput ()
{
  return new ShortTimeSliderWidget (*this, m_time);
}


string MsInput::str ()
{
  return str (m_time -> Get ());
}


string MsInput::str (GSML::sample_index val)
{
  stringstream s;
  s.setf(ios::fixed, ios::floatfield);
  s.precision (2);

  s << (val * 1000.0 / samples_per_second) << " ms";

  return s.str ();
}
