/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#include "ToolkitBasics.hh"
#include "ParameterDialog.hh"

#include <gtk--/dialog.h>
#include <gtk--/adjustment.h>
#include <gtk--/scale.h>
#include <gtk--/spinbutton.h>
#include <gtk--/checkbutton.h>
#include <gtk--/frame.h>
#include <gtk--/table.h> 
#include <gtk--/optionmenu.h> 
#include <gtk--/label.h>
#include <gtk--/entry.h>

class ParameterDialogGtk : public GSMP::ParameterDialog, public Gtk::Dialog
{
public:
  ParameterDialogGtk (GSMP::ParameterDialogAction* i_action, GSMP::Project* i_project, string i_title);

  virtual void AddDouble (double min, double max, double default_val, double* result, string label);
  virtual void AddCount (int min, int max, int default_val, int* result, string label);
  virtual void AddCheck (bool default_state, bool* result, string label);

  virtual void Show ();

private:
  void SlotOkClicked ();

  Gtk::Button ok_button;
  Gtk::Button cancel_button;
};

class DoubleValueSlider : public Gtk::HScale
{
public:
  DoubleValueSlider (Gtk::Adjustment& adjustment, double* dst_value);

private:
  void SlotValueChanged ();

  double* m_dst_value;
};

class Counter : public Gtk::SpinButton
{
public:
  Counter (Gtk::Adjustment& adjustment, int* dst_value);

private:
  void SlotValueChanged ();

  int* m_dst_value;
};

class Check : public Gtk::CheckButton
{
public:
  Check (bool i_state, bool* dst_state, string label);

private:
  void SlotToggled ();

  bool* m_dst_state;
};



/****************************************************************************************************/

class GenericInput;

class EnhParameterDialogGtk : public GSMP::EnhParameterDialog,  public Gtk::Dialog
{
public:
  EnhParameterDialogGtk (GSMP::DialogAction* i_action, string i_title);
  ~EnhParameterDialogGtk ();

  virtual void AddValue (GSMP::Value* value, string group = "");

  virtual void Reset (string group = ""); // Removes all widgets in group, "" removes all
  virtual void Show (); // draws the dialog

private:
  void SlotOkClicked ();
  void SlotOkEnabledChanged (bool new_state);

  Gtk::Button ok_button;
  Gtk::Button reset_button;
  Gtk::Button cancel_button;

  Gtk::VBox m_box;

  map <string, Gtk::Frame*> m_groups;
  map <string, deque <GenericInput*> > m_active_inputs;
};

class CheckedInput : public  Gtk::CheckButton
{
public:
  CheckedInput (GSMP::CheckVal* i_val, const string& i_label);

private:
  void SlotToggled ();
  void SlotChanged ();

  GSMP::CheckVal* m_val;
  bool internal_change;
};


class GenericInput// : public SigC::Object
{
public:
  virtual string Name () = 0;
  virtual bool Match (string input) = 0;
  virtual Gtk::Widget* CreateInput () = 0;
  virtual ~GenericInput () {}
  virtual unsigned int Priority () {return 0;}
  virtual bool Inlined () {return true;} // display widget between type-selector and entry box 
  virtual string str () = 0;
};

class GenericInputWidget : public Gtk::Table
{
public:
  GenericInputWidget (GSMP::Value* i_value, const string& i_label);
 
  void Add (GenericInput* input);
  void Show ();
private:
  void Draw (GenericInput* input);
  void SlotActivate (); // New Text set

  void SlotValueChanged ();

  deque <GenericInput*> m_inputs;
  unsigned int max_priority;

  GenericInput* max_priority_input;
  GenericInput* active_input;

  bool inlined; // current visible input has inlined style
  bool internal_change; // signal recursion killing

  Gtk::Label label;
  Gtk::Entry entry;
  Gtk::OptionMenu available_inputs;

  Gtk::Widget* current_input;

  GSMP::Value* m_value;
};

class VolumeFactorInput : public GenericInput
{
public:
  VolumeFactorInput (GSMP::VolumeFactor* fact);

  virtual string str (double val) = 0;
protected:
  GSMP::VolumeFactor* m_factor;
};


class FrequencyInput : public GenericInput
{
public:
  FrequencyInput (GSMP::Frequency* freq);

  virtual string str (double val) = 0;
protected:
  GSMP::Frequency* m_freq;
};

class ShortTimeInput : public GenericInput
{
public:
  ShortTimeInput (GSMP::ShortTime* time);

  virtual string str (GSML::sample_index val) = 0;
protected:
  GSMP::ShortTime* m_time;
};
