/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The status-bar.
 */

#include "StatusBar.hh"

using namespace Gtk;
using namespace GSMP;
using namespace GSML;

StatusBar::StatusBar (View* i_view)
  : HBox (false, 0),
    cancel (i_view, "Edit>Stop computation"),
    follow_io (i_view, "View>auto follow>IO position marker"),
    follow_curpos (i_view, "View>auto follow>current position marker")
{
  m_view = i_view;
  m_project = m_view -> GetProject ();

  set_border_width (0);
  
  m_progress.set_percentage (0.0);
  m_progress.set_show_text (true);
  
  m_project_message.set (m_project -> message_info . GetMessage ());
  m_mouse_position.set (m_project -> Time (m_view -> GetViewImpl () -> Start ()));
  m_project_length.set (m_project -> Time (m_project -> Length ()));
  follow_io.SetLabel ("fIO");
  follow_curpos.SetLabel ("fCP");
  cancel.SetLabel ("Cancel");

  pack_start (follow_curpos, false);
  pack_start (follow_io, false);
  
  pack_start ( *(manage (FrameWidget (m_project_message))));
  pack_start ( *(manage (FrameWidget (m_mouse_position))));
  pack_start ( *(manage (FrameWidget (m_project_length))));

  if (m_project -> progress_info () . begin () != m_project -> progress_info () . end())
    InsertProgress ( *((m_project -> progress_info () . end ()) - 1) );    

  show_all ();

  slot_pointer_moved = m_view -> GetViewImpl () -> SignalPointerMoved . connect (slot (this, &StatusBar::SlotPointerMoved));
  m_project -> SignalCurrentLayerChanged () . connect (slot (this, &StatusBar::SlotLayerChanged));
  m_project -> message_info . SignalUpdated.connect (slot (this, &StatusBar::SlotMessageInfoUpdated));
  m_project -> progress_info () . SignalInfoPushed.connect (slot (this, &StatusBar::SlotInfoPushed));
  m_project -> progress_info () . SignalInfoUpdated.connect (slot (this, &StatusBar::SlotInfoUpdated));
  m_project -> progress_info () . SignalInfoPopped.connect (slot (this, &StatusBar::SlotInfoPopped));
}

Gtk::Frame* StatusBar::FrameWidget (Gtk::Widget& widget)
{
  Gtk::Frame* r_frame = new Gtk::Frame ();
  r_frame -> set_shadow_type (GTK_SHADOW_IN);
  r_frame -> add (widget);
  return r_frame;
}

void StatusBar::SlotPointerMoved (sample_index pos)
{
  m_mouse_position.set (m_project -> Time (pos));
}

void StatusBar::SlotLayerChanged (DataLayer* ignored)
{
  m_project_length.set (m_project -> Time (m_project -> Length ()));
}

void StatusBar::SlotMessageInfoUpdated ()
{
  m_project_message.set (m_project -> message_info . GetMessage ());
}

void StatusBar::SlotInfoPushed (const GSML::Data::ProgressInfo::Info* info)
{
  if (!progress_shown)
    InsertProgress (info);
  else
    SlotInfoUpdated (info);
}

void StatusBar::SlotInfoUpdated (const GSML::Data::ProgressInfo::Info* info)
{
  m_progress.set_percentage (info -> Progress ());  
  m_progress.show ();
}

void StatusBar::SlotInfoPopped ()
{
  if (m_project -> progress_info () . begin () != m_project -> progress_info () . end())
    SlotInfoUpdated ( *((m_project -> progress_info () . end ()) - 1) );
  else
    RemoveProgress ();    
}

void StatusBar::InsertProgress (const GSML::Data::ProgressInfo::Info* info)
{
  pack_end (cancel, false);
  pack_end (m_progress, false);
  progress_shown = true;
  SlotInfoUpdated (info);
}

void StatusBar::RemoveProgress ()
{
  remove (m_progress);
  remove (cancel);
  show_all ();
  progress_shown = false;
}
