/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#include "ToolkitBasics.hh"

#include <gtk--/menushell.h>
#include "SplashScreen.hh"

using namespace Gtk;
using namespace Menu_Helpers;
using namespace GSMP;
using namespace GSMP::Menu;


string GtkKit :: ID ()
{
  return string ("gtk");
}

void GtkKit :: Run (GSMP::App* i_app)
{
  DEBUG__(GSMP_GTK, normal)
    cout << "starting gtk toolkit" << endl;

  m_app = i_app;
  connection_call_app_init = Gtk::Main::idle.connect (slot (this, &GtkKit::CallAppInit));
  m_kit.run ();
}

void GtkKit :: Quit ()
{
  Gtk::Main::quit();
}


GtkKit :: GtkKit (int argc, char *argv[])
  : m_kit (argc, argv), Kit (argc,argv)
{
  DEBUG__(GSMP_GTK, normal)
    cout << "initialized gtk toolkit" << endl;

  m_kit.timeout . connect (SigC::slot(this, &Kit::SlotGUIRefresh), 40);
}

gint GtkKit::CallAppInit ()
{
  connection_call_app_init . disconnect ();
  SplashScreen* t_splash = new SplashScreen (m_kit);
  m_app -> Init (t_splash);
  delete t_splash;
  return 0;
}

/*********************************/


void WindowGtk  :: PopTop ()
{
  DEBUG__(GSMP_GTK, detail)
    cout << "WindowGtk: recieved PopTop ()" << endl;
}

void WindowGtk  :: Hide ()
{
  DEBUG__(GSMP_GTK, detail)
    cout << "WindowGtk: recieved Hide ()" << endl;
}

bool WindowGtk  :: Kill ()
{
  DEBUG__(GSMP_GTK, detail)
    cout << "WindowGtk: recieved Kill ()" << endl;
  destroy ();
  return true;
}

void WindowGtk :: SetTitle (string title)
{
  cout << "WindowGtk :: SetTitle ... " << title << endl;
  set_title (title);
}

string WindowGtk :: Title ()
{
  return string ("Not Implemented !!!");
}


WindowGtk  :: WindowGtk ()
{
  DEBUG__(GSMP_GTK, detail)
    cout << "WindowGtk created" << endl;

  destroy.connect (SignalDestroy.slot ());
}

/*********************************/


MenuGtk::MenuGtk (View* i_view, MenuCollection* content) 
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "instanciating MenuGtk" << endl;

  m_view = i_view;
  m_app = i_view -> GetProject () -> GetApp ();
  m_collection = content;

  DEBUG__(GSMP_GTK, low_level)
    cout << "Crunching MenuCollector" << endl;

  MenuCollection::iterator last = (m_collection -> end ()) - 1;
  for (MenuCollection::iterator i = m_collection -> begin (); i != m_collection -> end (); i++)
    AppendSubMenuItem (this, (*i).SubMenu (), m_view, (i != last));

  m_collection -> SimpleItemStateChanged.connect (slot (this, &MenuGtk::SlotSimpleItemStateChanged)); 
  m_collection -> CheckedItemStateChanged.connect (slot (this, &MenuGtk::SlotCheckedItemStateChanged)); 
  m_collection -> SignalCheckedChanged.connect (slot (this, &MenuGtk::SlotCheckedChanged)); 

  show_all ();
}

Gtk::MenuItem* MenuGtk::Locate (int submenu_no, GSMP::Menu::ItemPath* path)
{
  int t_offset = 0;
  MenuCollection::iterator t_pos = m_collection -> begin ();

  for (int i = 0; i < submenu_no; i++)
    {
      t_offset += (*t_pos).SubMenu () -> my_items . size ();
      t_pos ++;
    }

  return LocateMenuItem (path, this, t_offset);
}


void MenuGtk::SlotSimpleItemStateChanged  (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::SimpleItem& item)
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "ItemStateChanged" << endl;

  Gtk::MenuItem* t_item = Locate (submenu_no, path);
  t_item -> set_sensitive (item.IsEnabled (m_view));  
}

void MenuGtk::SlotCheckedChanged  (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item)
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "CheckedChanged" << endl;

  CheckedMenuItem* t_item = dynamic_cast <CheckedMenuItem*> (Locate (submenu_no, path) );
  t_item -> SetCheck (item.IsChecked (m_view));
}


void MenuGtk::SlotCheckedItemStateChanged  (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item)
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "CheckedItemStateChanged" << endl;

  CheckedMenuItem* t_item = dynamic_cast <CheckedMenuItem*> (Locate (submenu_no, path) );
  t_item -> set_sensitive (item.IsEnabled (m_view));
}


/*********************************/


MenuBarGtk::MenuBarGtk (View* i_view, MenuCollection* i_l_menu, MenuCollection* i_r_menu) 
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "instanciating MenuBarGtk" << endl;

  m_view = i_view;
  m_app = i_view -> GetProject () -> GetApp ();
  m_l_menu = i_l_menu;
  m_r_menu = i_r_menu;


  DEBUG__(GSMP_GTK, low_level)
    cout << "Crunching MenuCollector" << endl;

  MenuList& bar = items ();
  for (MenuCollection::iterator i = m_l_menu -> begin (); i != m_l_menu -> end (); i++)
    {
      Gtk::Menu* t_menu = new Gtk::Menu;
      AppendSubMenuItem (t_menu, (*i).SubMenu (), m_view, false);
      bar.push_back (MenuElem((*i).Title (),"",*t_menu));
    }

  for (MenuCollection::iterator i = m_r_menu -> begin (); i != m_r_menu -> end (); i++)
    {
      Gtk::Menu* t_menu = new Gtk::Menu;
      AppendSubMenuItem (t_menu, (*i).SubMenu (), m_view, false);
      bar.push_back (MenuElem((*i).Title (),"",*t_menu));
      bar.back () -> right_justify ();
    }

  m_l_menu -> SimpleItemStateChanged.connect (bind (slot (this, &MenuBarGtk::SlotSimpleItemStateChanged) , true)); 
  m_r_menu -> SimpleItemStateChanged.connect (bind (slot (this, &MenuBarGtk::SlotSimpleItemStateChanged) , false)); 
  m_l_menu -> CheckedItemStateChanged.connect (bind (slot (this, &MenuBarGtk::SlotCheckedItemStateChanged) , true)); 
  m_r_menu -> CheckedItemStateChanged.connect (bind (slot (this, &MenuBarGtk::SlotCheckedItemStateChanged) , false)); 
  m_l_menu -> SignalCheckedChanged.connect (bind (slot (this, &MenuBarGtk::SlotCheckedChanged) , true)); 
  m_r_menu -> SignalCheckedChanged.connect (bind (slot (this, &MenuBarGtk::SlotCheckedChanged) , false)); 

  show_all ();
}



Gtk::MenuItem* MenuBarGtk::Locate (int submenu_no, GSMP::Menu::ItemPath* path, bool left)
{
  if (!left)
    submenu_no += m_l_menu -> size ();

  MenuList& t_bar = items ();
  Gtk::Menu* t_menu = (t_bar [submenu_no]) -> get_submenu ();

  return LocateMenuItem (path, t_menu);
}




void MenuBarGtk::SlotSimpleItemStateChanged  (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::SimpleItem& item, bool left)
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "ItemStateChanged" << endl;

  Gtk::MenuItem* t_item = Locate (submenu_no, path, left);
  t_item -> set_sensitive (item.IsEnabled (m_view));  
}

void MenuBarGtk::SlotCheckedChanged  (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item, bool left)
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "CheckedChanged" << endl;

  CheckedMenuItem* t_item = dynamic_cast <CheckedMenuItem*> (Locate (submenu_no, path, left) );
  t_item -> SetCheck (item.IsChecked (m_view));
}


void MenuBarGtk::SlotCheckedItemStateChanged  (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item, bool left)
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "CheckedItemStateChanged" << endl;

  CheckedMenuItem* t_item = dynamic_cast <CheckedMenuItem*> (Locate (submenu_no, path, left) );
  t_item -> set_sensitive (item.IsEnabled (m_view));
}



/*********************************/


CheckedMenuItem :: CheckedMenuItem (const string& label)
  : Gtk::CheckMenuItem (label)
{
  _activated = false;
  activate.connect (slot (this, &CheckedMenuItem::SlotActivate));
};

void CheckedMenuItem :: SlotActivate ()
{
  DEBUG__(GSMP_GTK, low_level)
    cout << "CheckedMenuItem :: SlotActivate ()" << endl;

  if (!_activated)
    SignalActivated ();
}

void CheckedMenuItem :: SetCheck (bool state)
{
  _activated = true;
  set_active (state);
  _activated = false;
}

/*********************************/


ButtonGtk::ButtonGtk (GSMP::View* i_view, const string& location)
  : m_item (*(i_view -> GetProject () -> GetApp () -> GetManager ()), location)
{
  m_view = i_view;
  slot_view_erased = m_view -> SignalErased . connect (slot (this, &ButtonGtk::SlotViewErased));
  slot_project_erased = m_view -> GetProject () -> SignalErased . connect (slot (this, &ButtonGtk::SlotProjectErased));
  
  shown = false;
  if (! m_item.Matched () )
    {
      m_item.SignalMatched.connect (slot (this, &ButtonGtk::SlotItemMatched));
      m_label = "-";
      m_sensitive = false;
    }
  else
    ExtractProperties ();

  m_item . SignalStateChanged.connect (slot (this, &ButtonGtk::SlotSimpleItemStateChanged));
  Show ();
  clicked .connect (slot (this, &ButtonGtk::SlotActivated));
}

void ButtonGtk::SlotSimpleItemStateChanged ()
{
  if (m_view)
    {
      m_sensitive = m_item . Get () -> IsEnabled (m_view);
      Show ();
    }
}

void ButtonGtk::SlotItemMatched ()
{
  if (m_view)
    {
      ExtractProperties ();
      Show ();
    }
}

void ButtonGtk::SlotViewErased ()
{
  m_sensitive = false;
  m_view = 0;
  Show ();
}

void ButtonGtk::SetView (GSMP::View* new_view)
{
  slot_view_erased.disconnect ();
  slot_project_erased.disconnect ();
  m_view = new_view;
  slot_view_erased = m_view -> SignalErased . connect (slot (this, &ButtonGtk::SlotViewErased));
  slot_project_erased = m_view -> GetProject () -> SignalErased . connect (slot (this, &ButtonGtk::SlotProjectErased));

  ExtractProperties ();
  Show ();
}

void ButtonGtk::SetLabel (const string& new_label)
{
  m_label = new_label;
  Show ();
}

void ButtonGtk::ExtractProperties ()
{
  const SimpleItem* t_item = m_item . Get ();
  m_label = t_item -> my_params.name;
  m_sensitive = t_item -> IsEnabled (m_view);
}

void ButtonGtk::Show ()
{
  if (shown)
    remove ();
  add_label (m_label, 0.5, 0.5);
  set_sensitive (m_sensitive);
  show ();
  shown = true;
}

void ButtonGtk::SlotActivated ()
{
  if (m_view && m_sensitive)
    m_item . Get () -> Activate (m_view);
}


/*********************************/


CheckButtonGtk::CheckButtonGtk (GSMP::View* i_view, const string& location)
  : m_item (*(i_view -> GetProject () -> GetApp () -> GetManager ()), location)
{
  m_view = i_view;
  external_activate = false;

  slot_view_erased = m_view -> SignalErased . connect (slot (this, &CheckButtonGtk::SlotViewErased));
  slot_project_erased = m_view -> GetProject () -> SignalErased . connect (slot (this, &CheckButtonGtk::SlotProjectErased));

  shown = false;
  if (! m_item.Matched () )
    {
      m_item.SignalMatched.connect (slot (this, &CheckButtonGtk::SlotItemMatched));
      m_label = "-";
      m_sensitive = false;
      m_checked = false;
    }
  else
    ExtractProperties ();

  m_item . SignalStateChanged.connect (slot (this, &CheckButtonGtk::SlotCheckedItemStateChanged));
  m_item . SignalCheckedChanged.connect (slot (this, &CheckButtonGtk::SlotCheckedChanged));
  Show ();
  clicked .connect (slot (this, &CheckButtonGtk::SlotActivated));
}

void CheckButtonGtk::SlotCheckedItemStateChanged ()
{
  if (m_view)
    {
      m_sensitive = m_item . Get () -> IsEnabled (m_view);
      Show ();
    }
}

void CheckButtonGtk::SlotCheckedChanged ()
{
  external_activate = true;
  if (m_view)
    {
      m_checked = m_item . Get () -> IsChecked (m_view);
      Show ();
    }
  external_activate = false;
}

void CheckButtonGtk::SlotItemMatched ()
{
  if (m_view)
    {
      ExtractProperties ();
      Show ();
    }
}

void CheckButtonGtk::SlotViewErased ()
{
  m_sensitive = false;
  m_checked = false;
  m_view = 0;
  Show ();
}

void CheckButtonGtk::SetView (GSMP::View* new_view)
{
  slot_view_erased.disconnect ();
  slot_project_erased.disconnect ();

  m_view = new_view;

  slot_view_erased = m_view -> SignalErased . connect (slot (this, &CheckButtonGtk::SlotViewErased));
  slot_project_erased = m_view -> GetProject () -> SignalErased . connect (slot (this, &CheckButtonGtk::SlotProjectErased));

  ExtractProperties ();
  Show ();
}

void CheckButtonGtk::SetLabel (const string& new_label)
{
  m_label = new_label;
  Show ();
}

void CheckButtonGtk::ExtractProperties ()
{
  const CheckedItem* t_item = m_item . Get ();
  m_label = t_item -> my_params.name;
  m_sensitive = t_item -> IsEnabled (m_view);
  m_checked = t_item -> IsChecked (m_view);
}

void CheckButtonGtk::Show ()
{
  if (shown)
    remove ();
  add_label (m_label, 0, 0);
  set_sensitive (m_sensitive);
  set_active (m_checked);
  show ();
  shown = true;
}

void CheckButtonGtk::SlotActivated ()
{
  if (!external_activate)
    if (m_view && m_sensitive)
      m_item . Get () -> Activate (m_view);
}

/*********************************/

ToggleButtonGtk::ToggleButtonGtk (GSMP::View* i_view, const string& location)
  : m_item (*(i_view -> GetProject () -> GetApp () -> GetManager ()), location)
{
  m_view = i_view;
  external_activate = false;

  slot_view_erased = m_view -> SignalErased . connect (slot (this, &ToggleButtonGtk::SlotViewErased));
  slot_project_erased = m_view -> GetProject () -> SignalErased . connect (slot (this, &ToggleButtonGtk::SlotProjectErased));
  shown = false;
  if (! m_item.Matched () )
    {
      m_item.SignalMatched.connect (slot (this, &ToggleButtonGtk::SlotItemMatched));
      m_label = "-";
      m_sensitive = false;
      m_checked = false;
    }
  else
    ExtractProperties ();

  m_item . SignalStateChanged.connect (slot (this, &ToggleButtonGtk::SlotCheckedItemStateChanged));
  m_item . SignalCheckedChanged.connect (slot (this, &ToggleButtonGtk::SlotCheckedChanged));
  Show ();
  clicked .connect (slot (this, &ToggleButtonGtk::SlotActivated));
}

void ToggleButtonGtk::SlotCheckedItemStateChanged ()
{
  if (m_view)
    {
      m_sensitive = m_item . Get () -> IsEnabled (m_view);
      Show ();
    }
}

void ToggleButtonGtk::SlotCheckedChanged ()
{
  external_activate = true;
  if (m_view)
    {
      m_checked = m_item . Get () -> IsChecked (m_view);
      Show ();
    }
  external_activate = false;
}

void ToggleButtonGtk::SlotItemMatched ()
{
  if (m_view)
    {
      ExtractProperties ();
      Show ();
    }
}

void ToggleButtonGtk::SlotViewErased ()
{
  m_sensitive = false;
  m_checked = false;
  m_view = 0;
  Show ();
}

void ToggleButtonGtk::SetView (GSMP::View* new_view)
{
  slot_view_erased . disconnect ();
  slot_project_erased . disconnect ();

  m_view = new_view;

  slot_view_erased = m_view -> SignalErased . connect (slot (this, &ToggleButtonGtk::SlotViewErased));
  slot_project_erased = m_view -> GetProject () -> SignalErased . connect (slot (this, &ToggleButtonGtk::SlotProjectErased));

  ExtractProperties ();
  Show ();
}

void ToggleButtonGtk::SetLabel (const string& new_label)
{
  m_label = new_label;
  Show ();
}

void ToggleButtonGtk::ExtractProperties ()
{
  const CheckedItem* t_item = m_item . Get ();
  m_label = t_item -> my_params.name;
  m_sensitive = t_item -> IsEnabled (m_view);
  m_checked = t_item -> IsChecked (m_view);
}

void ToggleButtonGtk::Show ()
{
  if (shown)
    remove ();
  add_label (m_label, 0.5, 0.5);
  set_sensitive (m_sensitive);
  set_active (m_checked);
  show ();
  shown = true;
}

void ToggleButtonGtk::SlotActivated ()
{
  if (!external_activate)
    if (m_view && m_sensitive)
      m_item . Get () -> Activate (m_view);
}

/**************************************************/


void AppendSubMenuItem (Gtk::Menu* menu, SubMenuItem* item, GSMP::View* for_view, bool end_seperator)
{
  using namespace SigC;
  using SigC::bind;

  MenuList& ml = menu -> items ();

  DEBUG__(GSMP_GTK, low_level)
    cout << "Appending SubMenuItem ==> Gtk::Menu" << endl;

  deque <GSMP::Menu::Item*> :: iterator j = (end_seperator) ? item -> my_items.end () : (item -> my_items.end ()) - 1;
  for (deque <GSMP::Menu::Item*> :: iterator i = item -> my_items.begin (); i != j; i++)
    {
      string t_name = (*i) -> my_params.name;

      SubMenuItem* submenu = dynamic_cast <GSMP::Menu::SubMenuItem*> (*i);
      SimpleItem* simpleitem = dynamic_cast <GSMP::Menu::SimpleItem*> (*i);
      CheckedItem* checked = dynamic_cast <GSMP::Menu::CheckedItem*> (*i);
      SelectionItem* selection = dynamic_cast <GSMP::Menu::SelectionItem*> (*i);

      if (submenu)
	{
	  DEBUG__(GSMP_GTK, low_level)
	    cout << "Found SubMenu" << endl;
	  Gtk::Menu* t_menu = new Gtk::Menu;
	  AppendSubMenuItem (t_menu, submenu, for_view, false);
	  ml.push_back (MenuElem(t_name, "", *t_menu)); 
	}

      else if (simpleitem)
	{
	  DEBUG__(GSMP_GTK, low_level)
	    cout << "Found SimpleItem" << endl;
	  ml.push_back (MenuElem (t_name));
	  ml.back () -> set_sensitive (simpleitem -> IsEnabled (for_view));
	  ml.back () -> activate.connect (bind (slot (simpleitem, &GSMP::Menu::SimpleItem::Invoke), for_view));
	}

      else if (checked)
	{
	  DEBUG__(GSMP_GTK, low_level)
	    cout << "Found CheckedItem" << endl;
	  ml.push_back (manage (new CheckedMenuItem (t_name)));
	  CheckedMenuItem* t_item = dynamic_cast <CheckedMenuItem*>  (ml.back ());
	  t_item -> set_sensitive (checked -> IsEnabled (for_view));
	  t_item -> set_active (checked -> IsChecked (for_view));
	  t_item -> SignalActivated.connect (bind (slot (checked, &GSMP::Menu::CheckedItem::ToggleCheck), for_view));
	  //t_item -> set_show_toggle (false);
	}

      else if (dynamic_cast <const GSMP::Menu::SeperatorItem *> (*i))
	{
	  DEBUG__(GSMP_GTK, low_level)
	    cout << "Found Sererator" << endl;
	  ml.push_back (SeparatorElem ());
	}

      else
	{
	  DEBUG__(GSMP_GTK, normal)
	    cout << "Warning: Found Item of unsupported type" << endl; 
	}
    }
}


Gtk::MenuItem* LocateMenuItem (GSMP::Menu::ItemPath* path, Gtk::Menu* menu, int path_front_offset)
{
  Gtk::Menu* t_menu = menu;
  MenuItem* current;
  for ( ItemPath::const_iterator i = path -> begin (); i != path -> end (); ++i)
    {
      MenuList& t_list = t_menu -> items ();
      current = t_list [(*i) + path_front_offset];
      path_front_offset = 0;
      if (current -> has_submenu ())
	t_menu = current -> get_submenu ();
    } 
  return current;
}
