/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#ifndef TOOL_KIT_BASICS_HH
#define TOOL_KIT_BASICS_HH

// GSMP
#include "GUI.hh"
#include "App.hh"
#include "View.hh"

// gtk--
#include <gtk--/main.h> 
#include <gtk--/window.h> 
#include <gtk--/menu.h>
#include <gtk--/menubar.h> 
#include <gtk--/checkbutton.h> 

class GtkKit : public GSMP::GUI::Kit
{
public:
  virtual string ID ();

  GtkKit (int argc, char *argv[]);
private:
  virtual void Run (GSMP::App* i_app);
  virtual void Quit ();

  Gtk::Main m_kit;

  gint CallAppInit ();

  GSMP::App* m_app;
  SigC::Connection connection_call_app_init;
};

class WindowGtk : public Gtk::Window, public GSMP::GUI::Window
{ 
public:
  virtual void PopTop (); // pops Window to Top of Desktop;
  virtual void Hide ();   // "iconize" win
  virtual bool Kill ();   // delete Window; returns false, whenever this call did not succed.

  virtual void SetTitle (string title);
  virtual string Title ();

  WindowGtk ();
  ~WindowGtk () {};
};

/* Single (context) menu  */
class MenuGtk : public Gtk::Menu
{
public:
  MenuGtk (GSMP::View* i_view, GSMP::Menu::MenuCollection* content); 
  
private:
  GSMP::App* m_app;
  GSMP::View* m_view;
  GSMP::Menu::MenuCollection* m_collection;

  Gtk::MenuItem* Locate (int submenu_no, GSMP::Menu::ItemPath* path);

  void SlotSimpleItemStateChanged (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::SimpleItem& item);
  void SlotCheckedChanged (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item);
  void SlotCheckedItemStateChanged (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item);
};

// Menubar, consisting of left and right menus 
class MenuBarGtk : public Gtk::MenuBar
{
public:
  MenuBarGtk (GSMP::View* i_view, GSMP::Menu::MenuCollection* i_l_menu, GSMP::Menu::MenuCollection* i_r_menu);

private:
  GSMP::View* m_view;
  GSMP::App* m_app;
  GSMP::Menu::MenuCollection* m_l_menu;
  GSMP::Menu::MenuCollection* m_r_menu;

  Gtk::MenuItem* Locate (int submenu_no, GSMP::Menu::ItemPath* path, bool left);

  void SlotSimpleItemStateChanged (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::SimpleItem& item, bool left);
  void SlotCheckedChanged (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item, bool left);
  void SlotCheckedItemStateChanged (int submenu_no, GSMP::Menu::ItemPath* path, const GSMP::Menu::CheckedItem& item, bool left);
};


// A signal-recursion-death save Version of gtk's checked menu item
class CheckedMenuItem : public Gtk::CheckMenuItem
{
public:
  CheckedMenuItem (const string& label);

  SigC::Signal0 <void> SignalActivated;

  void SetCheck (bool state);
protected:
  bool _activated;
  void SlotActivate ();
};


// A Button connected to the menu system
class ButtonGtk : public Gtk::Button
{
public:
  ButtonGtk::ButtonGtk (GSMP::View* i_view, const string& location);

  void SetView (GSMP::View* new_view);
  void SetLabel (const string& new_label);
protected:
  void SlotSimpleItemStateChanged ();
  void SlotItemMatched ();
  void SlotViewErased ();
  void SlotProjectErased ()  {SlotViewErased ();} 
  void SlotActivated ();

  void ExtractProperties (); // Extract Menu::SimpleItem's settings
  void Show ();              // (Re-) Draws the Button

  GSMP::View* m_view;
  GSMP::App* m_app;
  SigC::Connection slot_view_erased;
  SigC::Connection slot_project_erased;
  GSMP::Menu::SimpleItemFinder m_item;
  string m_label;
  bool m_sensitive;
  bool shown;
};


// An CheckButton
class CheckButtonGtk : public Gtk::CheckButton
{
public:
  CheckButtonGtk::CheckButtonGtk (GSMP::View* i_view, const string& location);

  void SetView (GSMP::View* new_view);
  void SetLabel (const string& new_label);
protected:
  void SlotCheckedItemStateChanged ();
  void SlotCheckedChanged ();
  void SlotItemMatched ();
  void SlotViewErased ();
  void SlotProjectErased () {SlotViewErased ();}
  void SlotActivated ();

  void ExtractProperties (); // Extract Menu::SimpleItem's settings
  void Show ();              // (Re-) Draws the Button

  GSMP::View* m_view;
  GSMP::App* m_app;
  SigC::Connection slot_view_erased;
  SigC::Connection slot_project_erased;
  GSMP::Menu::CheckedItemFinder m_item;
  string m_label;
  bool m_sensitive;
  bool shown;
  bool m_checked;
  bool external_activate; // Recursion killing
};

// An ToggleButton
class ToggleButtonGtk : public Gtk::ToggleButton
{
public:
  ToggleButtonGtk::ToggleButtonGtk (GSMP::View* i_view, const string& location);

  void SetView (GSMP::View* new_view);
  void SetLabel (const string& new_label);
protected:
  void SlotCheckedItemStateChanged ();
  void SlotCheckedChanged ();
  void SlotItemMatched ();
  void SlotViewErased ();
  void SlotProjectErased () {SlotViewErased ();}
  void SlotActivated ();

  void ExtractProperties (); // Extract Menu::SimpleItem's settings
  void Show ();              // (Re-) Draws the Button

  GSMP::View* m_view;
  GSMP::App* m_app;
  SigC::Connection slot_view_erased;
  SigC::Connection slot_project_erased;
  GSMP::Menu::CheckedItemFinder m_item;
  string m_label;
  bool m_sensitive;
  bool shown;
  bool m_checked;
  bool external_activate; // Recursion killing
};







// Appends GSMP::Menu::SubMenuItem entries to given Gtk::Menu
void AppendSubMenuItem (Gtk::Menu* menu, GSMP::Menu::SubMenuItem* item, GSMP::View* for_view, bool end_seperator);
Gtk::MenuItem* LocateMenuItem (GSMP::Menu::ItemPath* path, Gtk::Menu* menu, int path_front_offset = 0);



#endif // TOOL_KIT_BASICS_HH

