/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The track-info widget.
 */

#include "TrackInfo.hh"

using namespace GSML;
using namespace GSMP;
using namespace Gtk;

TrackInfo::TrackInfo (Project* i_project, track_index i_track_no)
  : Table (3, 2, true),
    mute ("Mute"),
    rec ("Rec"),
    active ("DoFX"),
    hide_track ("hide")
{
  m_project = i_project;
  m_track_no = i_track_no;

  name_external_changed = true;
  mute_external_changed = true;
  rec_external_changed = true;
  active_external_changed = false;

  set_row_spacings (2);
  set_col_spacings (2);
  set_border_width (2);

  m_dlayer = m_project -> CurrentLayerPtr ();
  m_tlayer = (*m_dlayer) [m_track_no];

  name.set_usize (75,0);
  name.set_text ( m_tlayer -> Name ()); 

  mute.set_active (m_project -> m_pl_mixer_params -> IsTrackMuted (m_track_no));
  rec.set_active (! (m_project -> m_ca_mixer_params -> IsTrackMuted (m_track_no)) );
  active.set_active (m_tlayer  -> selection.IsActive ());

  attach (name, 0, 2, 0, 1, (GTK_FILL|GTK_EXPAND), 0);
  attach (mute, 0, 1, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
  attach (rec, 1, 2, 1, 2, (GTK_FILL|GTK_EXPAND), 0);
  attach (active, 0, 1, 2, 3, (GTK_FILL|GTK_EXPAND), 0);
  attach (hide_track, 1, 2, 2, 3, (GTK_FILL|GTK_EXPAND), 0);

  show_all ();

  //m_project-> SignalCurrentLayerChanged.connect (slot (this, &TrackInfo::SlotDataLayerChanged));
  slot_name_changed = m_tlayer -> SignalNameChanged.connect (slot (this, &TrackInfo::SlotNameChanged));
  slot_active_changed = m_tlayer -> selection.SignalActiveChanged.connect (slot (this, &TrackInfo::SlotActiveChanged));

  m_project -> m_pl_mixer_params -> SignalTrackMuteChanged.connect (slot (this, &TrackInfo::SlotMuteChanged));
  m_project -> m_ca_mixer_params -> SignalTrackMuteChanged.connect (slot (this, &TrackInfo::SlotRecChanged));

  name.changed.connect (slot (this, &TrackInfo::SlotNameChangedUser));
  mute.toggled.connect (slot (this, &TrackInfo::SlotMuteChangedUser));
  rec.toggled.connect (slot (this, &TrackInfo::SlotRecChangedUser));
  active.toggled.connect (slot (this, &TrackInfo::SlotActiveChangedUser));
}

//void TrackInfo::SlotDataLayerChanged (DataLayer* new_layer)
void TrackInfo::SetDataLayer (GSML::DataLayer* new_layer)
{

  m_dlayer = new_layer;
  slot_name_changed.disconnect ();
  slot_active_changed.disconnect ();

  if (m_dlayer -> Tracks () > m_track_no)
    {
      m_tlayer = (*m_dlayer) [m_track_no];

      name_external_changed = false;
      active_external_changed = false;

      name.set_text (m_tlayer -> Name ());
      name.show ();
      active.set_active (m_tlayer -> selection.IsActive ());
      active.show ();

      name_external_changed = true;
      active_external_changed = true;

      slot_name_changed = m_tlayer -> SignalNameChanged.connect (slot (this, &TrackInfo::SlotNameChanged));
      slot_active_changed = m_tlayer -> selection.SignalActiveChanged.connect (slot (this, &TrackInfo::SlotActiveChanged));
    }
}

void TrackInfo::SlotNameChanged (const string& new_name)
{
  if (name_external_changed)
    {
      name.set_text (new_name);
      name.show ();
    }
}

void TrackInfo::SlotNameChangedUser ()
{
  if (name_external_changed)
    {
      name_external_changed = false;
      m_tlayer -> Name (name.get_text ());
      name_external_changed = true;
    }
}

void TrackInfo::SlotMuteChanged (track_index track, bool new_state)
{
  if (mute_external_changed && (track == m_track_no) )
    {
      mute.set_active (new_state);
      mute.show ();
    }
}


void TrackInfo::SlotMuteChangedUser ()
{
  if (mute_external_changed)
    {
      mute_external_changed = false;
      m_project -> m_pl_mixer_params -> SetTrackMute (m_track_no, mute.get_active ());
      mute_external_changed = true;
    }
}

void TrackInfo::SlotRecChanged (track_index track, bool new_state)
{
  if (rec_external_changed && (track == m_track_no) )
    {
      rec.set_active (!new_state);
      rec.show ();
    }
}

void TrackInfo::SlotRecChangedUser ()
{
  if (rec_external_changed)
    {
      rec_external_changed = false;
      m_project -> m_ca_mixer_params -> SetTrackMute (m_track_no, !rec.get_active ());
      rec_external_changed = true;
    }
}

void TrackInfo::SlotActiveChanged (bool new_state)
{
  if (active_external_changed)
    {
      active.set_active (new_state);
      active.show ();
    }
}

void TrackInfo::SlotActiveChangedUser ()
{
  if (active_external_changed)
    {
      active_external_changed = false;
      m_tlayer -> selection.SetActive (active.get_active ());
      active_external_changed = true;
    }
}


void TrackInfo::SlotHide ()
{
  cout << "Not Implemented !" << endl;
}
