/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The track manager dialog.
 */

#include "TrackManagerDialog.hh"

#include <gtk--/button.h>

using namespace GSMP;
using namespace GSML;

TrackManagerDialog::TrackManagerDialog (Project* i_project)
{
  m_project = i_project;
  
  string t_title = string ("TrackManager ") + m_project -> Name ();

  set_title (t_title);
  set_usize (280, 220);
  
  get_vbox () -> set_border_width (10);
  get_vbox () -> set_spacing (10);
  
  static const gchar *titles[] = {"No.", "Name", "Length", NULL};
  m_clist = manage (new UseableCList (titles) );
  
  m_clist -> column (1) . set_auto_resize (true);
  m_clist -> column (2) . set_auto_resize (true);
  
  m_clist -> set_reorderable (true); // test HACK .. ?!?
  m_clist -> row_moved . connect (slot (this, &TrackManagerDialog::SlotRowMoved) );
  
  m_scrolled_window = manage (new Gtk::ScrolledWindow () );
  m_scrolled_window -> set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  m_scrolled_window -> add (*m_clist);
  get_vbox () -> pack_start (*m_scrolled_window);
  
  m_insert_button = manage (new Gtk::Button ("Insert") );
  m_insert_button -> clicked . connect (slot (this, &TrackManagerDialog::SlotInsertButton) );
  m_button_box . pack_start (*m_insert_button);
  
  m_delete_button = manage (new Gtk::Button ("Delete") );
  m_delete_button -> clicked . connect (slot (this, &TrackManagerDialog::SlotDeleteButton) );
  m_button_box . pack_start (*m_delete_button);
  
  get_vbox () -> pack_start (m_button_box, false);
  
  Gtk::Button* t_button = manage (new Gtk::Button ("Close") );
  t_button -> clicked . connect (destroy . slot () );
  get_action_area () -> pack_start (*t_button);
  
  m_project -> SignalCurrentLayerChanged () . connect (slot (this, &TrackManagerDialog::SlotCurrentLayerChanged) );
  
  build_list (m_project -> CurrentLayerPtr () );

  show_all ();
  hack = 0;
}

TrackManagerDialog::~TrackManagerDialog ()
{
  cout << "YEAH: We get deleted ... (TrackManagerDialog)" << endl;
}

void TrackManagerDialog::build_list (DataLayer* layer)
{
  cout << "building list ..." << endl;
  m_clist -> clear ();
  
  for (DataLayer::iterator i = layer -> Begin (); i != layer -> End (); ++ i)
    {
      cout << "inserting: " << i - layer -> Begin () << endl;
      vector<std::string> text;
      text . push_back (itos (i - layer -> Begin () ) );
      text . push_back ( (*i) -> Name () );
      text . push_back (itos ( (int) (*i) -> Length () ) );
      m_clist -> rows () . push_back (text);
    }
  // dummy empty row - to make insert at end possible ...
  vector<std::string> text;
  text . push_back ("");
  text . push_back ("-none-");
  text . push_back (itos (hack) );
  m_clist -> rows () . push_back (text);
  cout << "build end" << endl;
  ++ hack;
}

void TrackManagerDialog::SlotCurrentLayerChanged (DataLayer* n_layer)
{
  build_list (n_layer);
}

void TrackManagerDialog::SlotInsertButton ()
{
  cout << "insert" << endl;
  
  // find selected row
  Gtk::CList_Helpers::SelectionList& t_list = m_clist -> selection ();
  if (t_list . begin () != t_list . end () )
    {
      track_index t_track = t_list . begin () -> get_row_num ();
      cout << "inserting Track at: " << t_track << endl;
      if (InternalEditStart ("insert track") ) {
	m_prepared_layer -> CreateTrack (t_track);
	InternalEditEnd ();
      }
      else {
	m_project -> message_info . PutMessage (string ("can't prepare - FX running?") );
      }
    }
  else
    m_project -> message_info . PutMessage (string ("no track selected") );
}

void TrackManagerDialog::SlotDeleteButton ()
{
  cout << "delete" << endl;
  
  bool selected = false;
  
  // find selected row
  Gtk::CList_Helpers::SelectionList& t_list = m_clist -> selection ();
  if (t_list . begin () != t_list . end () )
    {
      track_index t_track = t_list . begin () -> get_row_num ();
      if (t_track < m_project -> CurrentLayerPtr () -> Tracks () )
	{
	  selected = true;
	  cout << "deleting Track at: " << t_track << endl;
	  
	  if (InternalEditStart ("insert track") ) {
	    m_prepared_layer -> EraseTrack (t_track);
	    InternalEditEnd ();
	  }
	  else 
	    m_project -> message_info . PutMessage (string ("can't prepare - FX running?") );
	}
    }
  if (!selected)
    m_project -> message_info . PutMessage (string ("no track selected") );
}

void TrackManagerDialog::SlotRowMoved (gint p0, gint p1)
{
  cout << "row moved: " << p0 << " -> " << p1 << endl;
  DataLayer* t_layer = m_project -> CurrentLayerPtr ();
  p0 = limit_max (p0, t_layer -> Tracks () - 1);
  p1 = limit_max (p1, t_layer -> Tracks () - 1);
  cout << "row moved (limitted): " << p0 << " -> " << p1 << endl;
  
  if (p0 != p1)
    {
      if (InternalEditStart ("move track") ) {
	TrackLayer t_track ( *( (*m_prepared_layer) [p0] ) );
	
	m_prepared_layer -> EraseTrack (p0);
	m_prepared_layer -> CreateTrack (p1);
	(*(*m_prepared_layer) [p1] ) = t_track;
	
	InternalEditEnd ();
      }
      else 
	m_project -> message_info . PutMessage (string ("can't prepare - FX running?") );
    }
  else
    m_project -> message_info . PutMessage (string ("invaild track move") );
  
  build_list (m_project -> CurrentLayerPtr () );
  // m_clist -> clear ();
}

bool TrackManagerDialog::InternalEditStart (const string& undo_info)
{
  m_prepared_layer = m_project -> PrepareNewLayer ();
  // TODO: catch exceptions !!
  m_prepared_layer -> Comment (undo_info);
  m_project -> progress_info () . PushInfo ("track editing");
  
  return true;
}

void TrackManagerDialog::InternalEditEnd ()
{
  m_project -> progress_info () . PopInfo ();
  m_project -> FixateNewLayer ();
}

Gtk::Widget* TrackManagerDialogX::Create ()
{
  return new TrackManagerDialog (m_project);
}
