/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The view managment dialog.
 */

#include "ViewDialog.hh"

using namespace GSMP;
using namespace GSML;

ViewDialog::ViewDialog (App* i_app)
  : close_button ("Close")
{
  m_app = i_app;
  m_views = 0;

  set_title ("All Views");
  set_usize (450, 280);

  get_vbox ()->set_border_width (10);
  get_vbox ()->set_spacing (10);

  m_scrolled_window = manage (new Gtk::ScrolledWindow () );
  m_scrolled_window->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  get_vbox ()->pack_start (*m_scrolled_window);
  
  static const gchar *titles[] = {"Project", "View Type", "Start", "End", "Zoom", "Status", NULL};
  m_clist = manage (new Gtk::CList (titles) );

  m_clist->column (0).set_auto_resize (true);
  m_clist->column (1).set_auto_resize (true);
  m_clist->column (2).set_auto_resize (true);
  m_clist->column (3).set_auto_resize (true);
  m_clist->column (4).set_auto_resize (true);
  m_clist->column (4).set_auto_resize (true);

  m_scrolled_window->add (*m_clist);

  m_clist->select_row.connect (slot (this, &ViewDialog::SlotClicked));

  close_button.clicked.connect (destroy.slot ());
  get_action_area () -> pack_start (close_button);
  
  BuildAll ();

  m_app -> SignalProjectCreated.connect (slot(this, &ViewDialog::SlotProjectChanged));
  m_app -> SignalWindowCreated.connect (slot(this, &ViewDialog::SlotWindowChanged));

  show_all ();
}


void ViewDialog::BuildAll ()
{
  m_clist->clear (); 

  if (m_views)
    delete m_views;
  m_views = new vector <View*>;

  // kill old connections
  while (m_connections.size () != 0)
    {
      m_connections.top () . disconnect ();
      m_connections.pop ();
    }

  for (deque <Project*>::iterator i = m_app -> BeginProjects (); i != m_app -> EndProjects (); i++)
    {
      Project* t_project = (*i);

      m_connections . push ( t_project -> SignalNameChanged . connect (slot(this, &ViewDialog::SlotAnythingChanged)) );
      m_connections . push ( t_project -> SignalErased . connect (slot(this, &ViewDialog::SlotAnythingChanged)) );
      //   m_connections . push ( t_project -> SignalViewCreated . connect (slot(this, &ViewDialog::SlotAnythingChanged)) );

      
      for (deque <View*>::iterator j = t_project -> BeginViews () ; j != t_project -> EndViews (); j++)
	{
	  View* t_view = (*j);

	   m_connections.push ( t_view -> SignalErased . connect (slot(this, &ViewDialog::SlotAnythingChanged)) );

	  string status;
	  if (t_view -> IsVisible ())
	    {
	      status = "shown";
	      SimpleWindow* t_win = t_view -> MyWindow ();
	      m_connections.push ( t_win -> SignalDeleted . connect (slot(this, &ViewDialog::SlotAnythingChanged)) );
	    }
	  else
	    {
	      m_connections.push ( t_view -> SignalViewImplExchanged . connect (slot(this, &ViewDialog::SlotAnythingChanged)) );
	      status = "hidden";
	    }

	  ViewImpl* t_impl = t_view -> GetViewImpl ();
	
	  m_connections.push ( t_impl ->SignalZoomChanged. connect (slot(this, &ViewDialog::SlotIndexChanged)) );
	  m_connections.push ( t_impl ->SignalStartChanged. connect (slot(this, &ViewDialog::SlotIndexChanged)) );
    
	  vector<std::string> text;

	  text.push_back (t_project -> Name ());
	  text.push_back (t_impl -> Name ());
	  text.push_back (itos ((int) (t_impl -> Start () )));
	  text.push_back (itos ((int) (t_impl -> Start () + t_impl -> Length () )));
	  text.push_back (itos (t_impl -> Zoom () ));
	  text.push_back (status);

	  m_clist->rows ().push_back (text);
	  m_views -> push_back (t_view);
	}
    }
}

void ViewDialog::SlotAnythingChanged ()
{
  DEBUG__(GSMP, low_level)
    cout << "Rebuilding View List" << endl;
  BuildAll ();
}

void ViewDialog::SlotProjectChanged (Project* ignored)
{
  SlotAnythingChanged ();
}

void ViewDialog::SlotWindowChanged (SimpleWindow* ignored)
{
  SlotAnythingChanged ();
}

void ViewDialog::SlotIndexChanged (GSML::sample_index ignored)
{
  cout << "ViewDialog::SlotIndexChanged" << endl;
  SlotAnythingChanged ();
}



void ViewDialog::SlotClicked  (gint row, gint column, GdkEvent* event)
{
  View* t_view = (*m_views) [row];

  if (t_view -> IsVisible ())
    t_view -> MyWindow () -> Win () -> PopTop ();
  else
    m_app -> CreateMainWindow (t_view);
}


Gtk::Widget* ViewDialogX::Create ()
{
  return new ViewDialog (m_app);
}
