/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   
 */

#include "PluginInterface.hh"
#include "Data.hh"
#include "Activator.hh"
#include "Project.hh"
#include "ParameterDialog.hh"

class VolumeFader : public GSMP::LinearMonoFX
{
public:
  VolumeFader (double start_factor, double end_factor);

private:
  virtual void PrepareFX (GSML::sample_index length);
  virtual void ComputeBlock (GSML::SampleBufferDefault* src, GSML::SampleBufferDefault* dst, GSML::sample_index length);

  double m_start_factor;
  double m_delta_value;
  GSML::sample_index current_length;
};


/*********************************************************************************************************/

START_PLUGIN_INTERFACE;
PLUGIN_ITEM("Function","SilenceFXPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-09-19","Filters>Volume&G>Silence&N&100:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","InsertSilenceFXPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-09-19","Filters>Volume&G>Insert Silence&N&110:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","FadeInFXPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-09","Filters>Volume&G>Fade in:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","FadeOutFXPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-09","Filters>Volume&G>Fade out:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","VolumeShapeFXPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-09","Filters>Volume&G>Volume shape:Project:Item:State=on", NULL);
END_PLUGIN_INTERFACE;

class SilenceFXPlugin : public GSMP::Menu::Activator
{
public:
  SilenceFXPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class InsertSilenceFXPlugin : public GSMP::Menu::Activator
{
public:
  InsertSilenceFXPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};


class FadeInFXPlugin : public GSMP::Menu::Activator
{
public:
  FadeInFXPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ()
  {
    m_project -> DoFX (new VolumeFader (0,1));
  }
};

class FadeOutFXPlugin : public GSMP::Menu::Activator
{
public:
  FadeOutFXPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ()
  {
    m_project -> DoFX (new VolumeFader (1,0));
  }
};


class VolumeShapeFXPlugin : public GSMP::Menu::Activator
{
public:
  VolumeShapeFXPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ()
  {
    Action* t_action = new Action;
    SigC::Handle <GSMP::ParameterDialog, SigC::Scopes::Extend> t_dialog = m_project -> FXConfigDialog (t_action, string ("Volume Shaper"));
    t_dialog -> AddDouble (0, 3, 1, &(t_action -> s_val), string ("Start Volume"));
    t_dialog -> AddDouble (0, 3, 1, &(t_action -> e_val), string ("End Volume"));
    t_dialog -> Show ();
  }

private:
  class Action : public GSMP::ParameterDialogAction
  {
  public:
    virtual GSMP::FXBase* Build ()
    {
      return new VolumeFader (s_val, e_val);
    }

    double s_val;
    double e_val; 
  };
};

PLUGIN_FACTORY(Function,SilenceFXPlugin,GSMP::Menu::Activator,SilenceFXPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view));
PLUGIN_FACTORY(Function,InsertSilenceFXPlugin,GSMP::Menu::Activator,InsertSilenceFXPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view));
PLUGIN_FACTORY(Function,FadeInFXPlugin,GSMP::Menu::Activator,FadeInFXPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view));
PLUGIN_FACTORY(Function,FadeOutFXPlugin,GSMP::Menu::Activator,FadeOutFXPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view));
PLUGIN_FACTORY(Function,VolumeShapeFXPlugin,GSMP::Menu::Activator,VolumeShapeFXPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view));


/**********************************************************************************************************/

using namespace GSML;

void SilenceFXPlugin::Execute ()
{
  DataLayer* t_layer = m_project->PrepareNewLayer ();
  
  t_layer->Comment ("silence");
  m_project -> progress_info () . PushInfo ("silencing selection");
  
  for (track_index tr=0; tr < t_layer->Tracks (); ++tr)
    {
      TrackLayer* t_track = (*t_layer) [tr];
      TrackLayer::Selection& t_selection = t_track->selection;
      
      if (t_selection.IsActive () )
	{
	  TrackLayer::Selection::const_iterator i = t_track->selection.begin ();
	  
	  while (i != t_track->selection.end () )
	    {
	      sample_index t_start = i->start;
	      sample_index t_length = i->end - i->start;
	      
	      t_track->RemoveRegion (t_start, t_length);
	      
	      // create a entry witch has virtual-zero data by default ...
	      TrackLayer::Entry t_entry;
	      t_entry.length = t_length;
	      
	      t_track->InsertRawEntry (t_start, t_entry);
	      ++i;
	    }
	}
    }
  
  m_project -> progress_info () . PopInfo ();
  m_project -> FixateNewLayer ();
}

void InsertSilenceFXPlugin::Execute ()
{
  sample_index t_cur_pos;
  if (!m_project->CurrentLayerPtr ()->markers.GetMarker ("cur_pos", &t_cur_pos) )
    {
      cout << "can't get current pos marker ..." << endl;
      return;
    }
  
  DataLayer* t_layer = m_project->PrepareNewLayer ();
  
  t_layer->Comment ("insert silence");
  m_project -> progress_info () . PushInfo ("inserting silence");
  
  for (track_index tr=0; tr < t_layer->Tracks (); ++tr)
    {
      TrackLayer* t_track = (*t_layer) [tr];
      TrackLayer::Selection& t_selection = t_track->selection;
      
      if (t_selection.IsActive () )
	{
	  sample_index t_length = t_selection.SummedLength ();
	  
	  if (t_length > 0)
	    {
	      // create a entry witch has virtual-zero data by default ...
	      TrackLayer::Entry t_entry;
	      t_entry.length = t_length;
	      
	      t_track->InsertRawEntry (t_cur_pos, t_entry);
	    }
	     
	}
    }
  
  m_project -> progress_info () . PopInfo ();
  m_project -> FixateNewLayer ();
}

VolumeFader::VolumeFader (double start_factor, double end_factor)
  : LinearMonoFX (string ("fadeing"), string ("volume fade"))
{
  pcm_24_8bit type;
  double dBfs = type.dBfs ();

  m_start_factor = dBfs * start_factor;
  m_delta_value = dBfs * (end_factor - start_factor);
}

void VolumeFader::PrepareFX (sample_index length)
{
  current_length = length;
}

void VolumeFader::ComputeBlock (SampleBufferDefault* src, SampleBufferDefault* dst, sample_index length)
{
  double t_start = m_delta_value * (((double) Offset ()) / ((double) current_length));
  double t_end = m_delta_value * (((double) (Offset () + length) ) / ((double) current_length));

  pcm_24_8bit::value_type t_factor;
  pcm_24_8bit::value_type t_inc;

  t_factor ( (t_start + m_start_factor) );
  t_inc ( ((t_end - t_start) / ((double) length) ) );

  pcm_24_8bit::value_type* src_data = src -> data;
  pcm_24_8bit::value_type* dst_data = dst -> data;

  for (sample_index x = 0; x < length; x ++)
    {
      dst_data [x] = t_factor * src_data [x];
      t_factor = t_factor + t_inc;
    }
}
