/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *
 * Plugin collection for the very basic GSMP tasks
 *
 */

#include <sstream>

#include "generated/config.hh"
#include "PluginInterface.hh"
#include "IO.hh" // GSML mixer stuff
#include "IO_ALSA.hh" // ALSA mixer nodes
#include "IO_OSS.hh" // ALSA mixer nodes
#include "Activator.hh"
#include "App.hh"
#include "Data.hh"
#include "DataLayer.hh" 
#include "ParameterDialog.hh"


// Testing only
#include "ValuesGeneric.hh"
#include <math.h>

/********************************** Interface Part **********************************/

START_PLUGIN_INTERFACE;
#ifdef GSMP_WITH_ALSA
PLUGIN_ITEM("IO_Mixer_Node","ALSA","Ren Rebe <ren.rebe@gmx.net>","2001-06-11","", NULL);
#endif
PLUGIN_ITEM("IO_Mixer_Node","OSS","Ren Rebe <ren.rebe@gmx.net>","2001-10-22","", NULL);

// Project Menu
PLUGIN_ITEM("Function","NewProjectPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-07-23","Project>New&A&-1:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","ClearPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Project>Clear&A&0:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","CloseProjectPlugin", "Valentin Ziegler <ziegler@informatik.hu-berlin.de>","2001-10-10","Project>Close&A&2:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","Quit", "Valentin Ziegler <ziegler@informatik.hu-berlin.de>","2001-08-10","Project>Exit:App:Item:State=on", NULL);


// EditMenu
PLUGIN_ITEM("Function","UndoPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Edit>Undo&A:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","RedoPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Edit>Redo&A:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","CutPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Edit>Cut&B:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","CopyPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Edit>Copy&B:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","PastePlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Edit>Paste&B:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","SelAllPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03", "Edit>selection&D>select all&A&0:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","UnSelAllPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03", "Edit>selection&D>clear&A&1:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","InvertSelPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03", "Edit>selection&D>invert&A&2:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","AddTrackPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Edit>Add Track&F:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","RemoveTrackPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-07-22","Edit>Remove Track&F:Project:Item:State=on", NULL);

//PLUGIN_ITEM("Function","FX", "Ich","heute","Edit>Fake&G:Project:Item:State=on", NULL);
//PLUGIN_ITEM("Function","LoudnessFXPlugin", "Ich","heute","Edit>Loudness&G:Project:Item:State=on", NULL);

PLUGIN_ITEM("Function","CancelPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-08","Edit>Stop computation&G:Project:Item:State=off", NULL);

// IO Menu
PLUGIN_ITEM("Function","StartIOPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Start IO&A:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","StopIOPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Stop IO&A:Project:Item:State=off", NULL);
PLUGIN_ITEM("Function","SetPlaybackPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Do Playback&B:Project:Checked:Checked=checked", NULL);
PLUGIN_ITEM("Function","SetCapturePlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Do Capture&B&1:Project:Checked:State=on", NULL);
PLUGIN_ITEM("Function","SetPlaySelPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Play selections only&C&-2:Project:Checked:State=on&Checked=unchecked", NULL);
PLUGIN_ITEM("Function","SetCaptSelPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Capture in selections only&C&-1:Project:Checked:State=on&Checked=unchecked", NULL);
PLUGIN_ITEM("Function","SetFXHearPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Prehear FX&C:Project:Checked:State=on&Checked=unchecked", NULL);
PLUGIN_ITEM("Function","SetLoopPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Loop IO&C:Project:Checked:State=on&Checked=unchecked", NULL);
PLUGIN_ITEM("Function","SetStopEndPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Stop at end&C:Project:Checked:State=on&Checked=unchecked", NULL);
PLUGIN_ITEM("Function","RemixPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-03","IO>Playback remix:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","MixerChannelConfigPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-08-29","IO>Config Channels:Project:Item:State=on", NULL);
PLUGIN_ITEM("Function","ToggleALSAPlugin", "Ren Rebe <rene.rebe@gmx.net>","2001-08-25","IO>Toggle ALSA Card&zz:App:Item:State=on", NULL);

// View Menu

PLUGIN_ITEM("Function","ZoomInPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>zoom in&A&0:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","ZoomOutPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>zoom out&A&1:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","Zoom1Plugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>zoom&A&4>1$:1&B&0:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","Zoom100Plugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>zoom&A&4>1$:100&B&5:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","Zoom10000Plugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>zoom&A&4>1$:10000&B&6:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","ZoomAllPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>zoom&A&4>all data&D&3:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","ZoomSelectionPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>zoom&A&4>selection&D&4:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","DuplicateViewPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>duplicate view&E&0:View:Item:State=on", NULL);
PLUGIN_ITEM("Function","FollowIOPosPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>auto follow&H>IO position marker&A&0:View:Checked:Checked=unchecked", NULL);
PLUGIN_ITEM("Function","FollowCurPosPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-08-04","View>auto follow&H>current position marker&A&0:View:Checked:Checked=unchecked", NULL);
PLUGIN_ITEM("Function","AutoDelViewsPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-10-11","View>Behavior>delete Views without window:App:Checked:Checked=unchecked", NULL);

//PLUGIN_ITEM("Function","Check1Plugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-07-23","View>Sensitive Above&B&1:Project:Item:State=on", NULL);
//PLUGIN_ITEM("Function","Check2Plugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-07-23","View>Sensitive Below&B&0:View:Item:State=on", NULL);

// Help Menu
//PLUGIN_ITEM("Function","Check3Plugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-07-23","Help>Check This !&C&0:Project:Checked:State=on", NULL);
//PLUGIN_ITEM("Function","EnhDialogPlugin", "Valentin Ziegler <valentin.ziegler@student.hu-berlin.de>","2001-07-23","Help>EnhDialog:Project:Item:State=on", NULL);
END_PLUGIN_INTERFACE;


class NewProjectPlugin : public GSMP::Menu::Activator
{
public:
  NewProjectPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view)  {}
  
  virtual void Execute ();
};

class ClearProjectPlugin : public GSMP::Menu::Activator
{
public:
  ClearProjectPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view)  {}
  
  virtual void Execute ();
};


class CloseProjectPlugin : public GSMP::Menu::Activator
{
public:
  CloseProjectPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view)  {}
  
  virtual void Execute ();
};


class QuitPlugin : public GSMP::Menu::Activator
{
public:
  QuitPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class UndoPlugin : public GSMP::Menu::Activator
{
public:
  UndoPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class RedoPlugin : public GSMP::Menu::Activator
{
public:
  RedoPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class CutPlugin : public GSMP::Menu::Activator
{
public:
  CutPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class CopyPlugin : public GSMP::Menu::Activator
{
public:
  CopyPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class PastePlugin : public GSMP::Menu::Activator
{
public:
  PastePlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class SelAllPlugin : public GSMP::Menu::Activator
{
public:
  SelAllPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class UnSelAllPlugin : public GSMP::Menu::Activator
{
public:
  UnSelAllPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class InvertSelPlugin : public GSMP::Menu::Activator
{
public:
  InvertSelPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};


class AddTrackPlugin : public GSMP::Menu::Activator
{
public:
  AddTrackPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class RemoveTrackPlugin : public GSMP::Menu::Activator
{
public:
  RemoveTrackPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};


/*
  class FXPlugin : public GSMP::Menu::Activator
  {
  public:
  FXPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
  : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
  };

  class LoudnessFXPlugin : public GSMP::Menu::Activator
  {
  public:
  LoudnessFXPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
  : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
  };
*/

class CancelPlugin : public GSMP::Menu::Activator
{
public:
  CancelPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};


class StartIOPlugin : public GSMP::Menu::Activator
{
public:
  StartIOPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

class StopIOPlugin : public GSMP::Menu::Activator
{
public:
  StopIOPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};



class SetPlaybackPlugin : public GSMP::Menu::StatusActivator
{
public:
  SetPlaybackPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);
};

class SetCapturePlugin : public GSMP::Menu::StatusActivator
{
public:
  SetCapturePlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);
};

class SetPlaySelPlugin : public GSMP::Menu::StatusActivator
{
public:
  SetPlaySelPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);
};

class SetCaptSelPlugin : public GSMP::Menu::StatusActivator
{
public:
  SetCaptSelPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);
};


class SetFXHearPlugin : public GSMP::Menu::StatusActivator
{
public:
  SetFXHearPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);
};



class SetLoopPlugin : public GSMP::Menu::StatusActivator
{
public:
  SetLoopPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);
};

class SetStopEndPlugin : public GSMP::Menu::StatusActivator
{
public:
  SetStopEndPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);
};


class RemixPlugin : public GSMP::Menu::Activator
{
public:
  RemixPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ()
  {
    Action* t_action = new Action (m_project, m_app);
    SigC::Handle <GSMP::ParameterDialog, SigC::Scopes::Extend> t_dialog = m_project -> FXConfigDialog (t_action, string ("Playback Track Remix"));
    int t_max_channels =  m_app -> GetMixerNode () -> PlaybackChannels ();
    t_dialog -> AddCount (1, t_max_channels, min (t_max_channels, 2), &(t_action -> channels), string ("Channels"));
    t_dialog -> AddCheck (false, &(t_action -> interleaved), string ("Interleaved"));
    t_dialog -> AddCheck (false, &(t_action -> keep_vols), string ("Keep Volumes"));
    t_dialog -> AddCheck (true, &(t_action -> zero), string ("Zero other Chanels"));

    t_dialog -> Show ();
  }

private:
  class Action : public GSMP::ParameterDialogAction
  {
  public:
    Action (GSMP::Project* i_project, GSMP::App* i_app)
    {
      m_project = i_project;
      m_app = i_app;
    }

    virtual GSMP::FXBase* Build ()
    {
      for  (int tr = 0; tr < m_project -> m_pl_mixer_params->Tracks (); ++tr)
	{
	  GSML::pcm_default type;
	  GSML::pcm_default::value_type new_vol;
	  new_vol (0);
	  
	  if (keep_vols)
	    for (int chan = 0; chan < m_project->m_pl_mixer_params->Channels (); chan++)
	      new_vol = new_vol + m_project -> m_pl_mixer_params -> MatrixValue (tr, chan);
	  else
	    new_vol (type.dBfs ());
	  
	  for (int ch = 0; ch < m_project->m_pl_mixer_params->Channels (); ++ch)
	    {
	      bool match_channel = false;
	      if (ch < channels)
		if ((!interleaved) || (tr % channels == ch))
		  match_channel = true;
	      
	      if (match_channel)
		m_project -> m_pl_mixer_params -> SetMatrixValue (tr, ch, new_vol);
	      else
		if (zero)
		  m_project -> m_pl_mixer_params -> SetMatrixValue (tr, ch, (GSML::pcm_default::value_type) {0});
	    }
	}
      
      return 0;
    }

    int channels;
    bool interleaved;
    bool keep_vols;
    bool zero;

    GSMP::Project* m_project;
    GSMP::App* m_app;
  };
};


class MixerChannelConfigPlugin : public GSMP::Menu::Activator
{
public:
  MixerChannelConfigPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ()
  {
    Action* t_action = new Action (m_project, m_app);
    SigC::Handle <GSMP::ParameterDialog, SigC::Scopes::Extend> t_dialog = m_project -> FXConfigDialog (t_action, string ("Playback Track Remix"));
    
    t_dialog -> AddCount (1,
			  m_app -> GetMixerNode () -> PlaybackChannels (),
			  m_project->m_pl_mixer_params->Channels (),
			  &(t_action -> pl_channels),
			  string ("Playback Channels") );
    t_dialog -> AddCount (1,
			  m_app -> GetMixerNode () -> CaptureChannels (),
			  m_project->m_ca_mixer_params->Channels (),
			  &(t_action -> ca_channels),
			  string ("Capture Channels") );
    t_dialog -> Show ();
  }

private:
  class Action : public GSMP::ParameterDialogAction
  {
  public:
    Action (GSMP::Project* i_project, GSMP::App* i_app)
    {
      m_project = i_project;
      m_app = i_app;
    }

    virtual GSMP::FXBase* Build ()
    {
      m_project -> m_pl_mixer_params -> SetChannels (pl_channels);
      m_project -> m_ca_mixer_params -> SetChannels (ca_channels);
      return 0;
    }

    int pl_channels;
    int ca_channels;
    
    GSMP::Project* m_project;
    GSMP::App* m_app;
  };
};


class ZoomPlugin : public GSMP::Menu::Activator
{
public:
  ZoomPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view, GSML::sample_index zoom)
    : Activator (i_app, i_project, i_view)  {m_zoom = zoom;}

  virtual void Execute ();
private:
  GSML::sample_index m_zoom;
};

class ZoomAllPlugin : public GSMP::Menu::Activator
{
public:
  ZoomAllPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view)  {}
  
  virtual void Execute ();
};


class ZoomSelectionPlugin : public GSMP::Menu::Activator
{
public:
  ZoomSelectionPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view)  {}
  
  virtual void Execute ();
};


class DuplicateViewPlugin : public GSMP::Menu::Activator
{
public:
  DuplicateViewPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view)  {}
  
  virtual void Execute ();
};


class FollowMarkerPlugin : public GSMP::Menu::StatusActivator, public SigC::Object
{
public:
  FollowMarkerPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view, string i_marker);

  virtual bool Execute (bool state);
protected:
  virtual void Update (GSML::sample_index new_index) = 0;
private:
  void SlotMarkersChanged (const string& marker, GSML::sample_index old_pos, GSML::sample_index new_pos);
  void SlotLayerChanged (GSML::DataLayer* new_layer);

  SigC::Connection slot_markers_changed;
  string m_marker;
};

class FollowCurPosPlugin : public FollowMarkerPlugin
{
public:
  FollowCurPosPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : FollowMarkerPlugin (i_app, i_project, i_view, "cur_pos") {}
  
  virtual void Update (GSML::sample_index new_index);
};

class FollowIOPosPlugin : public FollowMarkerPlugin
{
public:
  FollowIOPosPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : FollowMarkerPlugin (i_app, i_project, i_view, "io_pos") {}
  
  virtual void Update (GSML::sample_index new_index);
};

class AutoDelViewsPlugin : public GSMP::Menu::StatusActivator, public SigC::Object
{
public:
  AutoDelViewsPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : StatusActivator (i_app, i_project, i_view) {}
  
  virtual bool Execute (bool state);

  void SlotWindowDeleted ();
  void SlotWindowCreated (GSMP::SimpleWindow* n_win);
};


// class C1Plugin : public GSMP::Menu::Activator
// {
// public:
//   C1Plugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
//     : Activator (i_app, i_project, i_view) {}
  
//   virtual void Execute ();
// };

// class C2Plugin : public GSMP::Menu::Activator
// {
// public:
//   C2Plugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
//     : Activator (i_app, i_project, i_view) {}
  
//   virtual void Execute ();
// };

// class C3Plugin : public GSMP::Menu::StatusActivator
// {
// public:
//   C3Plugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
//     : StatusActivator (i_app, i_project, i_view) {counter = 0;}
  
//   virtual bool Execute (bool status);

// private:
//   int counter;
// };

// class EnhDialogPlugin : public GSMP::Menu::Activator
// {
// public:
//   EnhDialogPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
//     : Activator (i_app, i_project, i_view)  {}
  
//   virtual void Execute ();

// private:
//   class Action : public GSMP::DialogAction
//   {
//   public:
//     virtual void Launch () {cout << "Launch !" << endl;}
//     virtual bool IsEnabled () {return true;} // may be launched ?
//     virtual void Init (GSMP::EnhParameterDialog* dialog)
//     {
//       dialog -> AddValue (new GSMP::SingleVolume (0, exp (1.6 * log (2)), 1, t, string ("Vol1")) );
//       dialog -> AddValue (new GSMP::SingleVolume (0, exp (1.6 * log (2)), 1, tt, string ("Vol2")), string ("Framed") );
//       dialog -> AddValue (new GSMP::SingleShortTime (20, 2000, 200, st, 44100, string ("Time")), string ("Framed") );
//       dialog -> AddValue (new AddOpt (dialog),  string ("Framed") );

//       dialog -> Show ();
//     }

//     virtual void Reset (GSMP::EnhParameterDialog* dialog) { dialog -> Reset (""); Init (dialog); }

//     bool b;
//     double t;
//     double tt;
//     GSML::sample_index st;

//     class AddOpt : public GSMP::CheckVal
//     {
//     public:
//       AddOpt (GSMP::EnhParameterDialog* dialog)
//       {
// 	label = "Additional Options";
// 	m_dialog = dialog;
// 	m_state = false;
//       }

//       virtual bool State () {return m_state;}

//       virtual bool Set (bool state)
//       {
// 	m_state = state;

// 	if (m_state)
// 	  {
// 	    m_dialog -> AddValue (new GSMP::SingleVolume (0, 1, exp (-1 * log (2)), v, string ("Vol x")), string ("Additional Options") );
// 	    m_dialog -> AddValue (new GSMP::SingleFrequency (2.0 * M_PI * 30.0 / 44100.0,
// 							     2.0 * M_PI * 15000.0 / 44100.0,
// 							     2.0 * M_PI * 300.0 / 44100.0,
// 							     v, 44100, string ("Freq x")), string ("Additional Options"));
// 	    m_dialog -> Show ();
// 	  }
// 	else
// 	  {
// 	    m_dialog -> Reset ("Additional Options");
// 	    m_dialog -> Show ();
// 	  }

// 	ValueChanged ();
// 	return state;
//       }

//       virtual bool Default ()
//       {
// 	return false;
//       }

//       bool m_state;

//       GSMP::EnhParameterDialog* m_dialog;

//       double v;
//     };

//   };
// };


class ToggleALSAPlugin : public GSMP::Menu::Activator
{
public:
  ToggleALSAPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view)
    : Activator (i_app, i_project, i_view) {}
  
  virtual void Execute ();
};

/* IO_Mixer_Node:ALSA is only a Factory for class GSML::ALSAMixerNode TODO: configure string */
#ifdef GSMP_WITH_ALSA
PLUGIN_FACTORY(IO_Mixer_Node,ALSA,GSML::HWMixerNode,GSML::ALSAMixerNode, (),());
#endif
PLUGIN_FACTORY(IO_Mixer_Node,OSS,GSML::HWMixerNode,GSML::OSSMixerNode, (),());

PLUGIN_FACTORY(Function,NewProjectPlugin,GSMP::Menu::Activator,NewProjectPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,ClearPlugin,GSMP::Menu::Activator,ClearProjectPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,CloseProjectPlugin,GSMP::Menu::Activator,CloseProjectPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,Quit,GSMP::Menu::Activator,QuitPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))

PLUGIN_FACTORY(Function,UndoPlugin,GSMP::Menu::Activator,UndoPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,RedoPlugin,GSMP::Menu::Activator,RedoPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,CutPlugin,GSMP::Menu::Activator,CutPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,CopyPlugin,GSMP::Menu::Activator,CopyPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,PastePlugin,GSMP::Menu::Activator,PastePlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))

PLUGIN_FACTORY(Function,SelAllPlugin,GSMP::Menu::Activator,SelAllPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,UnSelAllPlugin,GSMP::Menu::Activator,UnSelAllPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,InvertSelPlugin,GSMP::Menu::Activator,InvertSelPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))

PLUGIN_FACTORY(Function,AddTrackPlugin,GSMP::Menu::Activator,AddTrackPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,RemoveTrackPlugin,GSMP::Menu::Activator,RemoveTrackPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
//PLUGIN_FACTORY(Function,FX,GSMP::Menu::Activator,FXPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
//PLUGIN_FACTORY(Function,LoudnessFXPlugin,GSMP::Menu::Activator,LoudnessFXPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,CancelPlugin,GSMP::Menu::Activator,CancelPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))

PLUGIN_FACTORY(Function,StartIOPlugin,GSMP::Menu::Activator,StartIOPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,StopIOPlugin,GSMP::Menu::Activator,StopIOPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,SetPlaybackPlugin,GSMP::Menu::StatusActivator,SetPlaybackPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,SetCapturePlugin,GSMP::Menu::StatusActivator,SetCapturePlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,SetPlaySelPlugin,GSMP::Menu::StatusActivator,SetPlaySelPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,SetCaptSelPlugin,GSMP::Menu::StatusActivator,SetCaptSelPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,SetFXHearPlugin,GSMP::Menu::StatusActivator,SetFXHearPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,SetLoopPlugin,GSMP::Menu::StatusActivator,SetLoopPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,SetStopEndPlugin,GSMP::Menu::StatusActivator,SetStopEndPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,RemixPlugin,GSMP::Menu::Activator,RemixPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,MixerChannelConfigPlugin,GSMP::Menu::Activator,MixerChannelConfigPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))


PLUGIN_FACTORY(Function,ZoomInPlugin,GSMP::Menu::Activator,ZoomPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view, max (i_view -> GetViewImpl () -> Zoom () / 4, 1)) )
PLUGIN_FACTORY(Function,ZoomOutPlugin,GSMP::Menu::Activator,ZoomPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view, max (i_view -> GetViewImpl () -> Zoom () * 4, 1)) )
PLUGIN_FACTORY(Function,Zoom1Plugin,GSMP::Menu::Activator,ZoomPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view, 1))
PLUGIN_FACTORY(Function,Zoom100Plugin,GSMP::Menu::Activator,ZoomPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view, 100))
PLUGIN_FACTORY(Function,Zoom10000Plugin,GSMP::Menu::Activator,ZoomPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view, 10000))
PLUGIN_FACTORY(Function,ZoomAllPlugin,GSMP::Menu::Activator,ZoomAllPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,ZoomSelectionPlugin,GSMP::Menu::Activator,ZoomSelectionPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,DuplicateViewPlugin,GSMP::Menu::Activator,DuplicateViewPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,FollowCurPosPlugin,GSMP::Menu::StatusActivator,FollowCurPosPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,FollowIOPosPlugin,GSMP::Menu::StatusActivator,FollowIOPosPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
PLUGIN_FACTORY(Function,AutoDelViewsPlugin,GSMP::Menu::StatusActivator,AutoDelViewsPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))

// PLUGIN_FACTORY(Function,Check1Plugin,GSMP::Menu::Activator,C1Plugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
// PLUGIN_FACTORY(Function,Check2Plugin,GSMP::Menu::Activator,C2Plugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
//PLUGIN_FACTORY(Function,Check3Plugin,GSMP::Menu::StatusActivator,C3Plugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))
//PLUGIN_FACTORY(Function,EnhDialogPlugin,GSMP::Menu::Activator,EnhDialogPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))

PLUGIN_FACTORY(Function,ToggleALSAPlugin,GSMP::Menu::Activator,ToggleALSAPlugin,(GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view),(i_app, i_project, i_view))


// Scope with namespaces
using namespace GSMP;
using namespace GSML;

void ClearProjectPlugin :: Execute ()
{
  DataLayer* t_layer = m_project->PrepareNewLayer ();
  t_layer->Comment ("new file");
  m_project->progress_info () . PushInfo ("clearing file");
    
  t_layer->RemoveRegion (0, t_layer->Length () );
  for (DataLayer::iterator i = t_layer->Begin (); i != t_layer->End (); i++)
    (*i)->selection.Clear ();
    
  m_project->progress_info () . PopInfo ();
  m_project->FixateNewLayer ();
}

void CloseProjectPlugin :: Execute ()
{
  DEBUG__(GSMP, normal)
    cout << "User request to delete Project..." << endl;

  bool t_result = m_app -> EraseProject (m_project);

  DEBUG__(GSMP, normal)
    cout << ((t_result) ? string ("... done") : string ("... canceled")) << endl;
}

void QuitPlugin::Execute ()
{
  m_app -> Exit ();
}

void UndoPlugin::Execute ()
{
  Data::iterator i = m_project->CurrentLayer ();
    
  if (i == m_project->BeginLayer () )
    {
      cout << "Nothing to undo ... " << endl;
      return;
    }
    
  m_project->SetCurrentLayer (--i);
}
  
void RedoPlugin::Execute ()
{
  Data::iterator i = m_project->CurrentLayer ();
    
  if (i != m_project->EndLayer () )
    ++i;
    
  if (i == m_project->EndLayer () )
    {
      cout << "Nothing to redo ... " << endl;
      return;
    }
  
  m_project->SetCurrentLayer (i);
}
  
void CutPlugin::Execute ()
{
  DataLayer* t_layer = m_project->PrepareNewLayer ();
  
  Pattern* cnp_pattern = m_app->CurrentPattern ();
  cnp_pattern->CreateNEmptyTracks (t_layer->Tracks () );
  
  t_layer->Comment ("cut");
  m_project -> progress_info () . PushInfo ("cutting region");
    
  for (track_index tr=0; tr < t_layer->Tracks (); ++tr)
    {
      TrackLayer* t_track = (*t_layer) [tr];
	      
      if (t_track->selection.IsActive ())
	{
	  sample_index cutted_length = 0;
	  TrackLayer::Selection::const_iterator i;
	    
	  TrackLayer* copy_track;

	  i = t_track->selection.begin ();
	  while (i != t_track->selection.end () )
	    {
	      sample_index t_start = i->start;
	      sample_index t_length = i->end - i->start;
		
	      t_start -= cutted_length;
		
	      copy_track = t_track->CreateTrackLayer (t_start, t_length);
	      (*cnp_pattern) [tr]->InsertTrackLayer (cutted_length, *copy_track);
	      del_helper2 (copy_track);
		
	      t_track->RemoveRegion (t_start, t_length);
	      cutted_length += t_length;
	      i++;
	    }
	}
      t_track->selection.Clear ();
    }
    
  m_project -> progress_info () . PopInfo ();
  m_project -> FixateNewLayer ();
}

void CopyPlugin::Execute ()
{
  DataLayer* src_layer = m_project->CurrentLayerPtr ();
  
  Pattern* cnp_pattern = m_app->CurrentPattern ();
  cnp_pattern->CreateNEmptyTracks (src_layer->Tracks () );
  
  DataLayer::iterator src = src_layer->Begin ();
  DataLayer::iterator dest = cnp_pattern->Begin ();
  
  for (; src != src_layer->End (); ++src, ++dest)
    {
      sample_index copyed_length = 0;
      
      TrackLayer* copy_track;

      TrackLayer::Selection::const_iterator i = (*src)->selection.begin ();
      while (i != (*src)->selection.end () )
	{
	  sample_index t_start = i->start;
	  sample_index t_length = i->end - i->start;
	  
	  copy_track = (*src)->CreateTrackLayer (t_start, t_length);
	  (*dest)->InsertTrackLayer (copyed_length, *copy_track);
	  del_helper2 (copy_track);
	  
	  copyed_length += t_length;
	  ++i;
	}
    }
}


void PastePlugin::Execute ()
{
  track_index t_pasted_tracks = 0;
  
  sample_index paste_pos;
  if (!m_project->CurrentLayerPtr ()->markers.GetMarker ("cur_pos", &paste_pos) )
    {
      cout << "can't get current pos marker ..." << endl;
      return;
    }
  paste_pos = limit_max (paste_pos, m_project->CurrentLayerPtr ()->Length () );
  
  DataLayer* t_layer = m_project->PrepareNewLayer ();
  
  t_layer->Comment ("paste");
  m_project -> progress_info () . PushInfo ("pasting region");
  
  
  DataLayer* t_src = m_app->CurrentPattern ();
  
  for (track_index tr=0; tr < t_layer->Tracks (); ++tr)
    {
      TrackLayer::Selection& t_sel = (*t_layer) [tr] -> selection;
      // only paste if doFX
      if (t_sel.IsActive () )
	{
	  if (tr < t_src->Tracks () )
	    {
	      (*t_layer) [tr] -> InsertTrackLayer (paste_pos, * (*t_src) [tr] );
	      ++t_pasted_tracks;
	    }
	}
    }
  
  m_project -> progress_info () . PopInfo ();
  m_project -> FixateNewLayer ();
  
  stringstream text;
  text << "Pasted " << t_pasted_tracks << " of " << t_src->Tracks () << " tracks.";
  m_project -> message_info.PutMessage (text.str () );
}



void SelAllPlugin::Execute ()
{
  DataLayer* t_layer = m_project -> CurrentLayerPtr ();
  for (track_index tr = 0; tr < t_layer -> Tracks (); tr++)
    (*t_layer) [tr] -> selection.All ();
}

void UnSelAllPlugin::Execute ()
{
  DataLayer* t_layer = m_project -> CurrentLayerPtr ();
  for (track_index tr = 0; tr < t_layer -> Tracks (); tr++)
    (*t_layer) [tr] -> selection.Clear ();
}

void InvertSelPlugin::Execute ()
{
  DataLayer* t_layer = m_project -> CurrentLayerPtr ();
  for (track_index tr = 0; tr < t_layer -> Tracks (); tr++)
    (*t_layer) [tr] -> selection.Invert ();
}

void AddTrackPlugin::Execute ()
{
  sample_index t_length = m_project->Length ();
  
  DataLayer* t_layer = m_project->PrepareNewLayer ();
  t_layer->Comment ("track added");
  m_project -> progress_info () . PushInfo ("adding track");
    
  t_layer->CreateTrack (t_layer->Tracks () );
  
  TrackLayer::Entry t_entry;
  t_entry.length = t_length;
  
  // TODO: copy selection from the previous track ??
  
  (*t_layer) [t_layer->Tracks () - 1]->InsertRawEntry (0, t_entry);
  
  m_project -> progress_info () .PopInfo ();
  m_project -> FixateNewLayer ();
};
  
void RemoveTrackPlugin::Execute ()
{
  if (m_project->Tracks () == 0)
    {
      cout << "no track to remove ..." << endl;
      return;
    }
    
  DataLayer* t_layer = m_project->PrepareNewLayer ();
  t_layer->Comment ("track removed");
  m_project -> progress_info () .PushInfo ("removing track");
    
  t_layer->EraseTrack (t_layer->End () - 1);
    
  m_project -> progress_info () .PopInfo ();
  m_project -> FixateNewLayer ();
}


/*
  void FXPlugin::Execute ()
  {
  m_project -> DoFX (new FakeFX);
  }

  void LoudnessFXPlugin::Execute ()
  {
  m_project -> DoFX (new LoudnessFX);
  }
*/


void CancelPlugin::Execute ()
{
  m_project -> Cancel ();
}


void NewProjectPlugin::Execute ()
{
  // TODO : Exception handling
  Project* t_project = m_app -> CreateProject ();
  m_app -> CreateMainWindow (t_project);
}

void StartIOPlugin::Execute ()
{
  if (! m_project -> GetMixer () -> IsCapture () )
    {
      cout << "StopIOPlugin: no capture -> playback only ..." << endl;
      m_project -> DataSource () . SetDataLayer (m_project -> CurrentLayerPtr (), 0);
    }
  else
    {
      cout << "StopIOPlugin: preparing capture stuff" << endl;
      
      DataLayer* t_layer = m_project -> PrepareNewLayer ();
      m_project -> progress_info ().PushInfo ("capturing");
      t_layer -> Comment ("capture");
      m_project -> DataSource () . SetDataLayer (m_project -> CurrentLayerPtr (), t_layer);
    }

  m_project -> MarkerFilter () . SetDataLayer (m_project -> CurrentLayerPtr () );
  m_project -> SelectionFilter () . SetDataLayer (m_project -> CurrentLayerPtr () );
  m_project -> FXFilter () . SetDataLayer (m_project -> CurrentLayerPtr () );
  m_project -> MarkerFilter () . SetPlaybackStart ("cur_pos");
  m_project -> MarkerFilter () . SetCaptureStart ("cur_pos");

  sample_index t_latency =  m_project -> GetMixer () -> PlaybackPeriodSize () + m_app -> GetMixerNode () -> PlaybackPeriodSize ();

  View* t_view = *(m_project -> BeginViews ());

  Menu::SimpleItemFinder t_stop_item (*(m_app -> GetManager ()), string ("IO>Stop IO")); 
  t_stop_item . Get () -> SetState (t_view, true);
  Menu::SimpleItemFinder t_start_item (*(m_app -> GetManager ()), string ("IO>Start IO")); 
  t_start_item . Get () -> BlockAll ();
  Menu::CheckedItemFinder t_pl_item (*(m_app -> GetManager ()), string ("IO>Do Playback")); 
  t_pl_item . Get () -> SetState (t_view, false);
  Menu::CheckedItemFinder t_ca_item (*(m_app -> GetManager ()), string ("IO>Do Capture")); 
  t_ca_item . Get () -> SetState (t_view, false);

  m_project -> GetMixer () -> StartIO (m_project -> GetToplevelSource (), m_project -> m_pl_mixer_params, m_project -> m_ca_mixer_params);
  m_project -> StartPlaybackTracking (t_latency, string ("IO>Stop IO"));
}

void StopIOPlugin::Execute ()
{

  if (m_project -> GetMixer () -> IsIORunning () )
    {
      m_project -> GetMixer () -> StopIO ();
      
      while (m_project -> GetMixer () -> IsIORunning () )
	{
	  DEBUG__ (GSMP, low_level)
	    cout << "waiting for mixer to stop" << endl;
	  usleep (250000);
	}
    }

  DEBUG__ (GSMP, low_level)
    cout << "mixer seems to have stopped" << endl;

  m_project -> StopPlaybackTracking ();

  // fixate data-layer after capture
  if (m_project -> GetMixer () -> IsCapture () )
    {
      cout << "StopIOPlugin: fixating capture stuff" << endl;
      m_project -> progress_info ().PopInfo ();
      m_project -> FixateNewLayer ();
      m_project -> DataSource () . SetDataLayer (m_project -> CurrentLayerPtr (), 0);
    }
  else
    cout << "StopIOPlugin: no capture was running" << endl;

  View* t_view = *(m_project -> BeginViews ());

  Menu::SimpleItemFinder t_stop_item (*(m_app -> GetManager ()), string ("IO>Stop IO")); 
  t_stop_item . Get () -> SetState (t_view, false);
  Menu::SimpleItemFinder t_start_item (*(m_app -> GetManager ()), string ("IO>Start IO")); 
  t_start_item . Get () -> UnBlockAll ();
  Menu::CheckedItemFinder t_pl_item (*(m_app -> GetManager ()), string ("IO>Do Playback")); 
  t_pl_item . Get () -> SetState (t_view, true);
  Menu::CheckedItemFinder t_ca_item (*(m_app -> GetManager ()), string ("IO>Do Capture")); 
  t_ca_item . Get () -> SetState (t_view, true);
}


bool SetCapturePlugin::Execute (bool state)
{
  if (! (state || m_project -> GetMixer () -> IsPlayback ()) )
    return true;

  m_project -> GetMixer () -> SetCapture (state);
  return state;
}

bool SetPlaybackPlugin::Execute (bool state)
{
  if (! (state || m_project -> GetMixer () -> IsCapture ()) )
    return true;

  m_project -> GetMixer () -> SetPlayback (state);
  return state;
}

bool SetPlaySelPlugin::Execute (bool state)
{
  m_project -> SelectionFilter () . SetSelectionForPlayback (state);
  return state;
}

bool SetCaptSelPlugin::Execute (bool state)
{
  m_project -> SelectionFilter () . SetSelectionForCapture (state);
  return state;
}

bool SetFXHearPlugin::Execute (bool state)
{
  m_project -> MultiplexerFilter () . SetWay (state);
  return state;
}

bool SetLoopPlugin::Execute (bool state)
{
  m_project -> LoopFilter () . SetLooping (state);
  return state;
}

bool SetStopEndPlugin::Execute (bool state)
{
  m_project -> LoopFilter () . SetStopAtEnd (state);
  return state;
}


void ZoomPlugin::Execute ()
{
  ViewImpl* t_view = m_view -> GetViewImpl ();
  int t_zoom = t_view  -> Zoom ();
  sample_index t_start = t_view->Start ();
  int t_width = t_view->Width ();
  int t_length = t_view->Length ();
  
  sample_index t_cur_pos;
  
  DataLayer* t_layer = m_project -> CurrentLayerPtr ();
  
  if (t_layer->markers.GetMarker ("cur_pos", &t_cur_pos) )
    {
      // zoom the cur_pos marker into the screen's centre if it was on the screen
      if (t_cur_pos >= t_start && t_cur_pos < t_start + t_length)
	{
	  t_view -> SetZoom (m_zoom);
	  t_view -> SetStart (limit_min (t_cur_pos - t_width * m_zoom / 2, 0) );
	  return;
	}
    }
  
  t_view -> SetZoom (m_zoom);
}


void ZoomAllPlugin::Execute ()
{
  m_view -> GetViewImpl () -> SetZoomAll ();
}

void ZoomSelectionPlugin::Execute ()
{
  sample_index start;
  sample_index end;
  bool found_selection = false;  

  DataLayer* t_layer = m_project -> CurrentLayerPtr ();

  for (track_index tr = 0; tr < t_layer->Tracks (); tr++)
    {
      TrackLayer* t_track = (*t_layer) [tr];

      sample_index t_start;
      sample_index t_end;

      if (t_track -> selection.MinStart (&t_start) && t_track -> selection.MaxEnd (&t_end))
	{
	  start = (found_selection) ? min (t_start, start) : t_start;
	  end = (found_selection) ? max (t_end, end) : t_end;
	  found_selection = true;
	}
    }

  if (found_selection)
    {
      sample_index zoom = (end - start) / (m_view -> GetViewImpl () -> Width ());
      m_view -> GetViewImpl () -> SetZoom (zoom);
      m_view -> GetViewImpl () -> SetStart (start);
    }
  else
    m_project -> message_info.PutMessage (string ("no selection."));
}

void DuplicateViewPlugin::Execute ()
{
  // TODO : Exception handling
  m_app -> CreateMainWindow (m_view);
}


FollowMarkerPlugin::FollowMarkerPlugin (GSMP::App* i_app, GSMP::Project* i_project, GSMP::View* i_view, string i_marker)
  : StatusActivator (i_app, i_project, i_view)
{
  m_marker = i_marker;
}

bool FollowMarkerPlugin::Execute (bool state)
{
  if (state)
    {
      slot_markers_changed = m_project -> CurrentLayerPtr () ->  markers . SignalChanged.connect (slot (this, &FollowMarkerPlugin::SlotMarkersChanged));
      m_project -> SignalCurrentLayerChanged () . connect (slot (this, &FollowMarkerPlugin::SlotLayerChanged));

      sample_index t_position;
      if (m_project -> CurrentLayerPtr () -> markers . GetMarker (m_marker, &t_position))
	Update (t_position);
 
      MakeParasite ();
    }
  else
    DeParasite ();

  return state;
}

void FollowMarkerPlugin::SlotMarkersChanged (const string& marker, sample_index old_pos, sample_index new_pos)
{
  if (marker == m_marker)
    Update (new_pos);
}
 
void FollowMarkerPlugin::SlotLayerChanged (GSML::DataLayer* new_layer)
{
  slot_markers_changed . disconnect ();
  slot_markers_changed = new_layer ->  markers . SignalChanged.connect (slot (this, &FollowMarkerPlugin::SlotMarkersChanged));

  sample_index t_position;
  if (new_layer -> markers . GetMarker (m_marker, &t_position))
    Update (t_position);
}

void FollowCurPosPlugin :: Update (sample_index new_index)
{
  ViewImpl* t_impl = m_view -> GetViewImpl ();
  sample_index t_start = t_impl -> Start ();
  sample_index t_length = t_impl -> Length ();

  if (new_index < t_start || (t_start + t_length - 20) < new_index)
    t_impl -> SetStart (new_index - (t_length / 2));
}

void FollowIOPosPlugin :: Update (sample_index new_index)
{
  ViewImpl* t_impl = m_view -> GetViewImpl ();
  sample_index t_start = t_impl -> Start ();
  sample_index t_length = t_impl -> Length ();
  
  int t_zoom = 64;
  
  if (t_impl -> Zoom () < t_zoom)
    t_impl -> SetZoom (t_zoom);

  if (new_index < t_start || (t_start + t_length) < new_index)
    t_impl -> SetStart (new_index);
}

bool AutoDelViewsPlugin::Execute (bool state)
{
  if (state)
    {
      if (!IsParasite ())
	{
	  for (App::main_window_iterator i = m_app -> BeginMainWindows (); i != m_app -> EndMainWindows (); i++)
	    (*i) -> SignalDeleted . connect (slot (this, &AutoDelViewsPlugin::SlotWindowDeleted));
	  for (App::simple_window_iterator i = m_app -> BeginSimpleWindows (); i != m_app -> EndSimpleWindows (); i++)
	    (*i) -> SignalDeleted . connect (slot (this, &AutoDelViewsPlugin::SlotWindowDeleted));
	  m_app -> SignalWindowCreated . connect (slot (this, &AutoDelViewsPlugin::SlotWindowCreated));
	}

      SlotWindowDeleted ();
      MakeParasite ();
    }
  else
    DeParasite ();

  return state;
}

void AutoDelViewsPlugin::SlotWindowDeleted ()
{
  cout << "AutoDelViewsPlugin::SlotWindowDeleted" << endl;
  for (App::project_iterator i = m_app -> BeginProjects (); i != m_app -> EndProjects (); i++)
    {
      GSMP::Project* t_project = *i;
      cout << "Project is " << t_project << endl;
      while ( (t_project -> OrphanedViews ()) > 0 && ( (t_project -> OrphanedViews ()) + (t_project -> VisibleViews ()) ) > 1)
	t_project -> EraseView (t_project -> BeginOrphanedViews ());
    }
}

void AutoDelViewsPlugin::SlotWindowCreated (GSMP::SimpleWindow* n_win)
{
  cout << "AutoDelViewsPlugin::SlotWindowCreated" << endl;
  n_win -> SignalDeleted . connect (slot (this, &AutoDelViewsPlugin::SlotWindowDeleted));
}

// void C1Plugin::Execute ()
// {
//   Menu::SimpleItemFinder mf (*(m_app -> GetManager ()), string ("View>Sensitive Below"));
//   mf . Get () -> SetState ( *(m_project -> BeginViews ()), false);
// }

// void C2Plugin::Execute ()
// {
//   Menu::SimpleItemFinder mf (*(m_app -> GetManager ()), string ("View>Sensitive Above"));
//   mf . Get () -> SetState (m_view,false);
// }

// bool C3Plugin::Execute (bool status)
// {
//   bool t_ret = (((counter++) % 4) > 1); 
//   cout << "They want " << status << " I say " << t_ret << endl;
//   MakeParasite ();
//   return t_ret;
// }



// void EnhDialogPlugin::Execute ()
// {
//   Action* t_action = new Action ();
//   SigC::Handle <GSMP::EnhParameterDialog, SigC::Scopes::Extend> t_dialog = m_project -> FXConfigDialog (t_action, string ("Test EnhParameterDialog"));
//   t_dialog -> Show ();
// }

void ToggleALSAPlugin::Execute ()
{
  HWMixerNode* t_node = m_app->GetMixerNode ();
  if (!t_node->IsIORunning () )
    {
      string name = t_node->Interface ();
      cout << name << endl;
	
      if (name == "hw:1")
	name = "hw:0";
      else
	name = "hw:1";
	
      if (!t_node->SetInterface (name) )
	cout << "can't toggle interface" << endl;
    }
  // TODO: else app message io running
};
