/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The Application class.
 */

#include <App.hh>
#include "generated/config.hh"
#include <sigc++/bind.h>

using namespace GSMP;
using namespace GSMX_utils::Plugin;
using namespace GSML;

#ifdef GCC_SUCKS
void gcc_sucks_hack::Exit ()
{
  my_app -> Exit ();
}
#endif

App::Exception::Exception (const string& error)
  : GSMP::Exception (error) {}


App::ProjectNotFound::ProjectNotFound ()
  : Exception (string ("Pointer reference to Project not in App Container")) {}

App::App (int argc, char *argv[], const string& i_plugin_file, const string& i_toolkit_file, Manager* i_app_manager)
  : m_fmc ("App")
{
  m_fmc . Check4Corruption ();

  m_manager = i_app_manager;
  m_manager -> InitializeMenuData (this);

  m_fmc . Check4Corruption ();

  m_plugin_file = i_plugin_file;
  unnamed_project_counter = 0;

  DEBUG__(GSMP,normal)
    cout << "Creating Main Application" << endl
	 << "Loading preferences " << endl; // not realy yet ... TODO !

  m_preferences . file_index_config . filename_base = "GSMP_pcm_raw_";
  m_preferences . file_index_config . filename_ext = ".tmp";
  m_preferences . file_index_config . working_dir = "./tmp.GSMP/";
  m_preferences . default_view_type = "normal";

  m_preferences . default_sample_rate = 44100;
  m_preferences . default_tracks = 2;
  m_preferences . default_max_datalayers = 15;

#ifdef GSMP_WITH_ALSA
  m_preferences . default_mixer_node = "ALSA";
#else
  m_preferences . default_mixer_node = "OSS";
#endif

  for (int i=0; i < argc; i++)
    if (strcmp (argv [i], "--oss") == 0)
      {
	DEBUG__(GSMP,normal)
	  cout << "User requested OSS plugin" << endl;
	m_preferences . default_mixer_node = "OSS";
      }

  m_preferences . recommended_main_menu_l . push_back (Preferences::MenuFilter ("Project", "Project") );
  m_preferences . recommended_main_menu_l . push_back (Preferences::MenuFilter ("Edit", "Edit") );
  m_preferences . recommended_main_menu_l . push_back (Preferences::MenuFilter ("Filters", "Filters") );
  m_preferences . recommended_main_menu_l . push_back (Preferences::MenuFilter ("IO", "IO") );
  m_preferences . recommended_main_menu_l . push_back (Preferences::MenuFilter ("Dialogs", "Dialogs") );
  m_preferences . recommended_main_menu_l . push_back (Preferences::MenuFilter ("View", "View") );
  m_preferences . recommended_main_menu_r . push_back (Preferences::MenuFilter ("Help", "Help") );

  m_preferences . recommended_view_menu . push_back (Preferences::MenuFilter ("View", "View") );
  m_preferences . recommended_view_menu . push_back (Preferences::MenuFilter ("Edit", "Edit") );

  string t_prefix = GSMP_PATH_PLUGIN + string ("/");

  try
    {
      delete RegisterFile (t_prefix + i_toolkit_file);

      m_kit = Build <GUI::Kit, int, char**> (string ("GUI"), string ("Kit"), argc, argv);
    }

  catch (GSMX_utils::Plugin::Exception e)
    {
      cout << endl << "Fatal error while setting up GUI Kit" << endl
	   << e << endl
	   << "Exiting." << endl;
      exit (-1);
    }

  m_fmc . Check4Corruption ();
}


void App::Init (GUI::InitScreen* init_screen)
{

  m_fmc . Check4Corruption ();

  string t_prefix = GSMP_PATH_PLUGIN + string ("/");

  try
    {  // HACK - Plugins will be loaded automaticaly in near future...
      init_screen -> Update (string ("Loading ") + t_prefix + m_plugin_file, 0.0);
      delete RegisterFile (t_prefix + m_plugin_file);
      init_screen -> Update (string ("Loading ") + t_prefix + string ("GUI-gtk.so"), 0.1);
      delete RegisterFile (t_prefix + string ("GUI-gtk.so") );
      init_screen -> Update (string ("Loading ") + t_prefix + string ("Dialogs-gtk.so"), 0.2);
      delete RegisterFile (t_prefix + string ("Dialogs-gtk.so") );
      init_screen -> Update (string ("Loading ") + t_prefix + string ("VolumeFX.so"), 0.3);
      delete RegisterFile (t_prefix + string ("VolumeFX.so") );
      init_screen -> Update (string ("Loading ") + t_prefix + string ("DeveloperFX.so"), 0.4);
      delete RegisterFile (t_prefix + string ("DeveloperFX.so") );
      init_screen -> Update (string ("Loading ") + t_prefix + string ("DynamicsFX.so"), 0.5);
      delete RegisterFile (t_prefix + string ("DynamicsFX.so") );
      init_screen -> Update (string ("Loading ") + t_prefix + string ("FrequencyFX.so"), 0.6);
      delete RegisterFile (t_prefix + string ("FrequencyFX.so") );

      init_screen -> Update (string ("Creating menus"), 0.7);
      content* functions = Items (string ("Function"));
      for (content::const_iterator i = functions -> begin (); i != functions -> end (); ++ i)
	{
	  const Info& info = PlugInfo (string ("Function"), (*i) );
	  m_manager -> Add (info . location, info . category, info . name);
	}
      delete functions;

    }

  /* I know it is very ugly to brute-force terminate the program
     here, and we do it here only in worst cases */

  catch (GSMX_utils::Plugin::Exception e)
    {
      cout << endl << "Fatal error while loading base DLs:" << endl
	   << e << endl
	   << "Exiting." << endl;
      exit (-1);
    }

    gui_extension = m_kit -> ID ();

    // TODO: Exception Handling down below....

    DEBUG__(GSMP,normal)
      cout << "GUI id is \"" << gui_extension << "\"" << endl
	   << "Creating DataFileIndex class" << endl;

    gui_extension = string ("_") + gui_extension;
    

    init_screen -> Update (string ("Initializing File index at ") + m_preferences . file_index_config . working_dir, 0.8);
    m_file_index = new DataFileIndex (m_preferences . file_index_config);
    if (! m_file_index -> Configure (true) )
      {
	cout << endl << "Fatal error while configuring FileIndex" << endl
	     << "Exiting." << endl;
	exit (-1);
      }
  
    m_fmc . Check4Corruption ();

    m_current_pattern = new Pattern ("internal cut'n paste layer",
				     m_file_index, &pcm_type,
				     m_preferences . default_sample_rate);
    

    m_fmc . Check4Corruption ();

    DEBUG__(GSMP,normal)
      cout << "Initializing MixerNode \"" << m_preferences . default_mixer_node << "\" (Not realy now..)" << endl;
    
    
    init_screen -> Update (string ("Initializing mixer node: ") + m_preferences . default_mixer_node, 0.9);
    m_mixer_node = Build <GSML::HWMixerNode> ("IO_Mixer_Node", m_preferences . default_mixer_node);
    if (!m_mixer_node -> Init () )
      cout << "ERROR: Configureing the primary sound device!" << endl;
      
    m_fmc . Check4Corruption ();

    CreateProject ();
    CreateMainWindow ();
};

App::~App ()
{
  DEBUG__(GSMP,normal)
    cout << "Signaling App termination" << endl;
  
  // delete all projects
  for (project_iterator p = m_projects . begin (); p != m_projects . end (); ++ p)
    {
      cout << "deleting next project ... ;-)" << endl;
      delete (*p);
    }
  m_projects . resize (0);
    
  SignalTerminated ();
  
  delete m_current_pattern;
  delete m_file_index;
}

void App::Run ()
{
  m_kit -> Run (this);
}

bool App::Exit ()
{
  DEBUG__(GSMP, normal)
    cout << "Request for app termination" << endl;
  
  // request for all projects
  for (project_iterator p = m_projects . begin (); p != m_projects . end (); ++ p)
    {
      cout << "requesting next project ..." << endl;
      if (! (*p) -> SignalEraseRequest () ) {
	DEBUG__(GSMP, normal)
	  cout << "App termination cancelled by project" << endl;
	return false;
      }
    }
  
  if (! SignalTerminateRequest () )
    {
      DEBUG__(GSMP, normal)
	cout << "App termination cancelled globally" << endl;
      return false;
    }
  
  m_kit -> Quit ();
  return true;
}

Project* App::CreateProject ()
{
  ++ unnamed_project_counter;
  return CreateProject (m_preferences . default_sample_rate, m_preferences . default_tracks,
			m_preferences . default_max_datalayers,
			"unnamed-" + itos (unnamed_project_counter) );
}

Project* App::CreateProject (sample_rate i_samplerate, int i_track_no, int i_max_datalayers, const string& i_name)
{
  DEBUG__(GSMP,normal)
    cout << "Creating new Project" << endl;

  // Caller is supposed to do Exception handing...
  Project* tmp = new Project (this, i_samplerate, i_track_no, i_max_datalayers, i_name);
  m_projects . push_back (tmp);
  
  SignalProjectCreated (tmp);
  return tmp;
}

bool App::EraseProject (Project* del_prj)
{
  // find del_prj
  project_iterator p = m_projects . begin ();
  while (p != m_projects . end () && *p != del_prj)
    ++ p;
  
  if (p == m_projects . end () )
    throw ProjectNotFound ();
  
  // ugly hack to check if the prj got deleted (old iterator might be invaild after erase ...)
  int t_size = m_projects . size ();
  EraseProject (p);
  
  return m_projects . size () != t_size;
}

App::project_iterator App::EraseProject (const project_iterator& del_prj)
{
  if (! (*del_prj) -> SignalEraseRequest () )
  {
    DEBUG__(GSMP,normal)
      cout << "Project Erase cancelled" << endl;
    return del_prj;
  }
 
  Project* t_project = (*del_prj);
  project_iterator r_iterator =  m_projects . erase (del_prj); 

  t_project -> SignalErased ();
  delete t_project;
  
  return r_iterator;
}

App::project_iterator App::BeginProjects ()
{
  return m_projects . begin ();
}

App::project_iterator App::EndProjects ()
{
  return m_projects . end ();
}

Pattern* App::CurrentPattern ()
{
  return m_current_pattern;
}

string App::GuiComponentName (string component_name)
{
  return component_name + gui_extension;
}

MainWindow* App :: CreateMainWindow (const string &view_type)
{
  return CreateMainWindow (m_projects . front (), view_type);
}

MainWindow* App::CreateMainWindow (Project* project, const string &view_type)
{
  View* t_view;
  if (project -> BeginOrphanedViews () != project -> EndOrphanedViews ())
    {
      DEBUG__(GSMP,detail)
	cout << "Creating window with orphaned view" << endl;

      t_view = *(project -> BeginOrphanedViews ()); // TODO: search for type !!
    }
  else if (project -> BeginVisibleViews () != project -> EndVisibleViews ())
    {
      DEBUG__(GSMP,detail)
	cout << "Creating window with copy of visible view" << endl;

      t_view = *(project -> BeginVisibleViews ());  // TODO: search for type !!
    }
  else
    {
      DEBUG__(GSMP,detail)
	cout << "Creating window with new view" << endl;

      t_view = project -> CreateView (view_type);
    }

  return CreateMainWindow (t_view); 
}

MainWindow* App::CreateMainWindow (SimpleWindow* copy_from)
{
  return CreateMainWindow (copy_from -> GetView ());
}


MainWindow* App::CreateMainWindow (View* instantiate_from)
{
  MainWindow* r_win;

  View* t_view = (instantiate_from -> IsVisible ()) ? instantiate_from -> GetProject () -> CopyView (instantiate_from) : instantiate_from;

  try
    {
      r_win = Build <MainWindow, App*, View*> (string ("GUI"), string ("MainWindow"), this, t_view);
    }

  catch (GSMX_utils::Plugin::Exception e)
    {
      cout << endl << "Fatal PluginSystem error while instanciating MainWindow" << endl
	   << e << endl
	   << "Exiting." << endl;
      exit (-1);
    }

  SignalWindowCreated (r_win);

  manage (r_win);

  m_main_windows . push_back (r_win);

  using SigC::bind;

  r_win ->  SignalDeleted . connect (bind (slot (this, &App::SlotMainWindowDeleted), r_win));

  return r_win;
}

App::main_window_iterator App::BeginMainWindows ()
{
  return m_main_windows . begin ();
}

App::main_window_iterator App::EndMainWindows ()
{
  return m_main_windows . end ();
}

SimpleWindow* App::CreateSimpleWindow (const string &view_type)
{
  return CreateMainWindow (view_type);
}

SimpleWindow* App::CreateSimpleWindow (Project* project, const string &view_type)
{
  return CreateMainWindow(project, view_type);
}

SimpleWindow* App::CreateSimpleWindow (SimpleWindow* copy_from)
{
  return CreateMainWindow (copy_from);
}

SimpleWindow* App::CreateSimpleWindow (View* instantiate_from)  // If View has parent win, it will be copied
{
  return CreateMainWindow(instantiate_from);
}

App::simple_window_iterator App::BeginSimpleWindows ()
{
  return m_simple_windows . begin ();
}

App::simple_window_iterator App::EndSimpleWindows ()
{
  return m_simple_windows . end ();
}

GSMX_utils::Plugin::content* App::GetViewImplTypes ()
{
  content* t_retval;
  try 
    {
      t_retval = Items (string ("ViewImpl") );
    }
  catch (GSMX_utils::Plugin::Exception e)
    {
      DEBUG__(GSMP,normal)
	cout << "Warning: Unable to get List of View Implementations !" << endl;
      t_retval = new content;
    }
  return t_retval;
}

GSML::DataFileIndex* App::GetDataFileIndex ()
{
  return m_file_index;
}

GSML::HWMixerNode* App::GetMixerNode ()
{
  return m_mixer_node;
}

void App::SlotMainWindowDeleted (MainWindow* the_window)
{
  for (main_window_iterator i = m_main_windows . begin (); i != m_main_windows . end (); i++)
    if ((*i) == the_window)
      {
	cout << "Found Window .. " << endl;

	m_main_windows . erase (i);
	break;
      }

  if (m_main_windows . size () + m_simple_windows . size () == 0)
    {
      DEBUG__(GSMP, normal)
	cout << "Last Window killed, terminating" << endl;
      Exit (); // Todo: check if canceled
    }
}
