/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* This code is mostly adapted from CoolPlayer's CPI_Player_CoDec_MPEG.c:
 * CoolPlayer - Blazing fast audio player.
 *   Copyright (C) 2000-2001 Niek Albers
 *   Copyright (C) 2001 Robert Leslie
 *
 *   Also released under the terms of the GPL v.2
 *
 * Because Robert Leslie pointed me to it to get an reference implementaion
 *   - THANKS!.
 *
 * Feel free to use it as mid-layer C++ API!
 *
 */

/* Short Description:
 *   Midlevel and C++ API for the libMAD !!! (mp3 library)
 */

#include "MADHelper.hh"

#if EXT_FIO_MP3 == 1

#include <sys/types.h>
#include <fcntl.h>

#define MP3_SAMPLES_PER_FRAME 1152
#define XING_MAGIC     (('X' << 24) | ('i' << 16) | ('n' << 8) | 'g')

enum {
  XING_FRAMES = 0x00000001L,
  XING_BYTES  = 0x00000002L,
  XING_TOC    = 0x00000004L,
  XING_SCALE  = 0x00000008L
};

using namespace std;

MADHelper::MADHelper ()
{
  m_fd = 0;
  m_frame_cache = 0;
}

MADHelper::~MADHelper ()
{
  if (m_fd > 0)
    close (m_fd);
  
  if (m_frame_cache != 0) {
    m_frame_cache -> Unuse ();
    if (m_frame_cache -> used == 0)
      delete m_frame_cache;
  }
}

bool MADHelper::Open (const string& fname, MADHelper* cache_partner = 0)
{
  cout << "MADHelper::Open: " << m_fname << endl;
  
  m_fname = fname;
  
  if (m_frame_cache != 0) {
    m_frame_cache -> Unuse ();
    if (m_frame_cache -> used == 0)
      delete m_frame_cache;
  }
  
  if (cache_partner != 0)
    m_frame_cache = cache_partner -> m_frame_cache;
  else
    m_frame_cache = new FrameCache;
  
  m_frame_cache -> Use ();
  
  m_fd = open (m_fname . c_str (), O_RDONLY);
  if (m_fd <= 0)
    return false;
  
  mad_stream_init (&m_stream);
  mad_frame_init (&m_frame);
  mad_synth_init (&m_synth);
  
  if (!ScanHeader () ) {
    // TODO: nice generic way to free mad_*_init info
    return false;
  }
  
  // UN*X style hacks ;-)
  if ( (m_size = lseek (m_fd, 0, SEEK_END) ) < 0)
    cout << "BUG: seek error ???" << endl;
  if (lseek (m_fd, 0, SEEK_SET) < 0)
    cout << "BUG: seek error ???" << endl;
  
  m_synth . pcm.length = 0;
  m_samplecount = 0;
  m_timer = mad_timer_zero;
  
  // very inaccurate way to get the frame length :-(
  cout << "try to get pcm frame count" << endl;
  if (m_xing . flags & XING_FRAMES) {
    /* Rene: get via XING ? */
    m_length = m_frame . header . duration;
    mad_timer_multiply (&m_length, m_xing.frames);
  }
  else {
    /* estimate playing time from file size */
    
    mad_timer_set (&m_length, 0, 1, m_frame . header . bitrate / 8);

    mad_timer_multiply (&m_length, m_size);
  }
  
  if (lseek (m_fd, 0, SEEK_SET) < 0)
    cout << "BUG: seek error ???" << endl;
  
  m_rate   = 0;
  m_frames = -1;
  m_buflen = 0;
  
  // Rene's new info stuff
  m_samplerate =  m_frame . header . samplerate;
  if (m_frame . header . mode == MAD_MODE_SINGLE_CHANNEL)
    m_tracks = 1;
  else
    m_tracks = 2;
  
  m_byte_of_mp3frame = 0;
  m_sample_of_mp3frame = 0;
  
  // UGLY UGLY HACK HACK !!!!! :
  m_pcm_frames = (m_length . seconds + 1) * m_samplerate;
  
  cout << "the info we gathered / randomed together: frames: " << m_pcm_frames
       << ", rate: " << m_samplerate
       << ", tracks: " << m_tracks << endl;
  
  cout << "MADHelper::Open return true" << endl;
  return true;
}

bool MADHelper::Close ()
{
  cout << "MADHelper::Close" << endl;
  if (m_fd != 0)
    {
      cout << "  file open - closing" << endl;
      bool closed = (close (m_fd) >= 0);
      m_fd = 0;
      
      if (m_frame_cache != 0)
	{
	  m_frame_cache -> Unuse ();
	  
	  if (m_frame_cache -> used == 0)
	    delete m_frame_cache;
	  
	  m_frame_cache = 0;
	}
      
      Cleanup ();
      
      return closed;
    }
  else
    return false;
}

long MADHelper::Length ()
{
  return m_pcm_frames;
}

int MADHelper::Tracks ()
{
  return m_tracks;
}

int MADHelper::Rate ()
{
  return m_samplerate;
}

bool MADHelper::Read (int length, mad_fixed_t* buffer)
{
  //cout << "MADHelper::Read";
  
  if (m_fd <= 0)
    return false;

  // loop to decode mp3-frames until we have the requested length
  bool t_eof = false;
  while (length > 0 && !t_eof)
    {
      int avail = m_synth . pcm . length - m_samplecount;
      
      cout << "read_vail: " << avail << endl;
      
      if (avail > length)
	avail = length;
      
      // s.th. buffered ??
      if (avail > 0) {
	mad_fixed_t const *ch1, *ch2;
	
	ch1 = m_synth . pcm . samples [0] + m_samplecount;
	ch2 = m_synth . pcm . samples [1] + m_samplecount;
	
	if (! true /* m_info . m_bStereo */ )
	  ch2 = 0;
	else if (m_synth . pcm . channels == 1)
	  ch2 = ch1;
	
	PackPCM (buffer, avail, ch1, ch2);
	
	buffer += avail * 2; // stereo
	length -= avail;
	m_samplecount += avail;
      }  
      else { // decode next frame
	t_eof = ! DecodeNext (true);
	
	mad_synth_frame (&m_synth, &m_frame);
	mad_timer_add (&m_timer, m_frame . header . duration);
	
	m_samplecount = 0;
      } // end else decode next frame
    }
  
  // the requested length is not available -> zero buffer
  if (length > 0) {
    cout << "Read: filling with zero" << endl;
    for (int i = 0; i < length * 2; ++ i) // stereo see above
      buffer [i] = 0;
  }

  return true;
}

bool MADHelper::Seek (long pos)
{
  // Ren: modified to decode at least two frames ...
  cout << "MADHelper::Seek" << endl;
  
  if (m_fd <= 0)
    return false;
  
  int t_seek_frame = pos / MP3_SAMPLES_PER_FRAME;
  int t_jump_frame = t_seek_frame - t_seek_frame % 10;
  if (t_jump_frame > t_seek_frame - 2)
    t_jump_frame = t_seek_frame - 2;
  
  long t_jump_pos = 0;
  
  // byte pos cached ??
  {
    SigC::Threads::MLock a (m_frame_cache -> lock);
    
    /*
b    for (int i = 0; i < m_frame_cache -> frames . size (); ++ i)
      cout << "  frame: " << i << "; pos: " << m_frame_cache -> frames [i] << endl;
    */
    
    int t_cache_key = t_jump_frame / 10;
    if (m_frame_cache -> frames . size () < t_cache_key)
      {
	t_cache_key = m_frame_cache -> frames . size ();
	if (t_cache_key > 0)
	  -- t_cache_key;
      }
    
    if (t_cache_key > 0)
      t_jump_pos = m_frame_cache -> frames [t_cache_key];
    
    t_jump_frame = t_cache_key * 10;
  }
  
  cout << "seek: jump2: frame: " << t_jump_frame
       << ", pos: " << t_jump_pos << endl;
  
  // move file-pointer to jump pos
  
  if (lseek (m_fd, t_jump_pos, SEEK_SET) < 0)
    cout << "BUG: seek error ???" << endl;
  
  m_frames = t_jump_frame;
  m_buflen = 0;
  m_sample_of_mp3frame = t_jump_frame * MP3_SAMPLES_PER_FRAME;
  m_byte_of_mp3frame = t_jump_pos;
  
  // I hope this resets some internal stuff an DecodeNext has to read in the next turn ??
  mad_stream_buffer (&m_stream, m_buffer, 0);
  
  mad_frame_mute (&m_frame);
  mad_synth_mute (&m_synth);
  
  // ok - after jumping to best matching or last cached frame seek to the right one
  while (m_frames < t_seek_frame - 2) {
    cout << "seek: decode next header. is: " << m_frames
	 << ", seek: " << t_seek_frame << endl;
    DecodeNext (false); // false -> only decode the headers
  }
  
  mad_synth_mute (&m_synth);
  
  while (m_frames < t_seek_frame) {
    cout << "seek: decode next frame (really). is: " << m_frames
	 << ", seek: " << t_seek_frame << endl;
    DecodeNext (true); // true -> decode frame, too
    mad_synth_frame (&m_synth, &m_frame);
  }
  
  // adjust m_samplecount (outputted samples from synthed samples)
  m_samplecount = pos - m_sample_of_mp3frame;
  
  cout << "  seek: m_samplecount: " << m_samplecount << endl;

  if (m_samplecount >= MP3_SAMPLES_PER_FRAME)
    cout << "  seek: BUG: samplecount to large!!!" << endl;
  
  return true;
}

void MADHelper::Cleanup ()
{
  cout << "MADHelper::Cleanup" << endl;
  
  mad_synth_finish (&m_synth);
  mad_frame_finish (&m_frame);
  mad_stream_finish (&m_stream);
}

bool MADHelper::ParseXing (struct mad_bitptr ptr, unsigned int bitlen)
{
  cout << "MADHelper::ParseXing" << endl;
  
  if (bitlen < 64 || mad_bit_read (&ptr, 32) != XING_MAGIC)
    goto fail;

  m_xing . flags = mad_bit_read (&ptr, 32);
  bitlen -= 64;

  if (m_xing . flags & XING_FRAMES) {
    if (bitlen < 32)
      goto fail;

    m_xing . frames = mad_bit_read (&ptr, 32);
    bitlen -= 32;
  }

  if (m_xing .flags & XING_BYTES) {
    if (bitlen < 32)
      goto fail;

    m_xing . bytes = mad_bit_read (&ptr, 32);
    bitlen -= 32;
  }

  if (m_xing . flags & XING_TOC) {
    int i;

    if (bitlen < 800)
      goto fail;

    for (i = 0; i < 100; ++ i)
      m_xing . toc[i] = (unsigned char) mad_bit_read (&ptr, 8);

    bitlen -= 800;
  }

  if (m_xing . flags & XING_SCALE) {
    if (bitlen < 32)
      goto fail;

    m_xing . scale = mad_bit_read (&ptr, 32);
    bitlen -= 32;
  }
  
  cout << "  returning success" << endl;
  return true;

 fail:
  m_xing . flags = 0;
  cout << "  returning failure" << endl;
  return false;
}

bool MADHelper::ScanHeader ()
{
  struct mad_stream t_stream;
  struct mad_frame t_frame;
  unsigned char t_buffer [8192];
  unsigned int buflen = 0;
  int count = 0;
  bool result = true;
  
  cout << "MADHelper::ScanHeader" << endl;
  
  mad_stream_init (&t_stream);
  mad_frame_init (&t_frame);
  
  m_xing . flags = 0;

  while (1) {
    if (buflen < sizeof (t_buffer) ) {
      int bytes;
      
      if ( (bytes = read (m_fd, t_buffer + buflen, sizeof (t_buffer) - buflen) ) < 0) {
	result = false;
	break;
      }
      
      buflen += bytes;
    }
    
    mad_stream_buffer (&t_stream, t_buffer, buflen);
    
    while (1) {
      if (mad_frame_decode (&t_frame, &t_stream) == -1) {
	if (!MAD_RECOVERABLE (t_stream . error) )
	  break;
	
	continue;
      }
      
      if (count++ || (ParseXing (t_stream . anc_ptr, t_stream . anc_bitlen) ) )
	break;
    }
    
    if (count || t_stream . error != MAD_ERROR_BUFLEN)
      break;
    
    memmove (t_buffer, t_stream . next_frame,
	     buflen = &t_buffer [buflen] - t_stream . next_frame);
  }
  
  if (count) {
    m_frame . header = t_frame . header;
  }
  else
    result = false;
  
  mad_frame_finish (&t_frame);
  mad_stream_finish (&t_stream);
  
  cout << "  returning: " << result << endl;
  return result;
}

bool MADHelper::DecodeNext (bool frame_too)
{
  // bytes of compressed stream left ??
  if (m_byte_of_mp3frame >= m_size) {
    cout << "DNF: end of mp3 stream reached!" << endl;
    
    m_pcm_frames = m_sample_of_mp3frame + m_samplecount;
    
    return false;
  }
  
  // save length of current frame (if one got already decoded)
  int t_frame_bytes;
  if (m_frames >= 0)
    t_frame_bytes = m_stream . next_frame - m_stream . this_frame;
  else {
    cout << "first frame DEBUG: " << endl
	 << "this_f: " << m_stream . this_frame
	 << ", next_f: " << m_stream . next_frame
	 << ", buffer_ptr: " << m_buffer << endl;
    t_frame_bytes = 0;
  }
  
  bool t_eof = false;
  bool t_done = false;
  while (!t_done && !t_eof)
    {
      int error;
      if (frame_too) {
	error = mad_frame_decode (&m_frame, &m_stream);
      }
      else {
	mad_header t_header;
	error = mad_header_decode (&t_header, &m_stream);
      }
      if (error != -1)
	t_done = true;
      else // read more data
	{
	  // buffer data is not enough - recycle last buffered and read more
	  int bytes = 0;
	  
	  if (MAD_RECOVERABLE (m_stream . error) ) {
	    cout << "DNF: we triggered a recoverable MAD error: " << m_stream . error << endl;
	    continue;
	  }
      
	  if (m_buflen > sizeof (m_buffer) ) {
	    cout << "!!!!!!! m_buflen incorrect !!!!!!! " << m_buflen << endl;
	    m_buflen = sizeof (m_buflen);
	  }
      
	  // data for next frame left
	  if (m_stream . next_frame != 0) {
	    int t_reuse_length = m_buffer + m_buflen - m_stream . next_frame;
	
	    if (m_stream . next_frame + t_reuse_length > m_buffer + sizeof(m_buffer) )
	      cout << "DNF: !!!! memcopy out of range !!!! ommited !!!!" << endl;
	    else
	      memmove (m_buffer, m_stream . next_frame, t_reuse_length);
	
	    m_buflen = t_reuse_length;
	  }
	  else
	    cout << "DNF: info: next_frame == 0!!" << endl;
      
	  // how much to fill ??
	  bytes = sizeof (m_buffer) - m_buflen;
	  if (m_buflen + bytes > sizeof (m_buffer) )
	    cout << "!!!! read bytes too large ... !!!!" << endl;
      
	  if ( (bytes = read (m_fd, m_buffer + m_buflen, bytes) ) < 0)
	    {
	      cout << "DNF:  POSIX read error - file too short ??" << endl;
	      return false;
	    }
      
	  if (bytes == 0) {
	    cout << "DNF: eof case during POSIX read" << endl;
	    t_eof = true;
	  }
	  m_buflen += bytes;
	  if (m_buflen > sizeof(m_buffer) ) {
	    cout << "DNF: !!!! buflen too large after read ??? - corrected !!!!" << endl;
	    m_buflen = sizeof(m_buffer);
	  }
      
	  mad_stream_buffer (&m_stream, m_buffer, m_buflen);
	} // end if mad error
    } // end while not done
  
  // update states
  if (m_frames >= 0) {
    m_sample_of_mp3frame += MP3_SAMPLES_PER_FRAME; // fix pcm_frames in one mp3 frame - I hope ...
    m_byte_of_mp3frame += t_frame_bytes;
  }
  ++ m_frames;
  
  cout << m_frames << ": byte_pos: " << m_byte_of_mp3frame
       << ", sample_pos: " << m_sample_of_mp3frame << endl;
  
  // add to pos cache ???
  if ( (m_frames) % 10 == 0)
    {
      SigC::Threads::MLock a (m_frame_cache -> lock);
      
      //cout << "cached frame infos: " << m_frame_cache -> frames . size () << endl;
      
      int cache_key = m_frames / 10;
      if (m_frame_cache -> frames . size () == cache_key)
	{
	  cout << "adding: info for: " << cache_key
	       << ", frame: " << m_frames
	       << ", byte_pos: " << m_byte_of_mp3frame
	       << ", sample_pos: " << m_sample_of_mp3frame
	       << " to cache." << endl;
	  m_frame_cache -> frames . push_back (m_byte_of_mp3frame);
	}
    }
  
  return true;
}

void MADHelper::PackPCM (mad_fixed_t* pcm, unsigned int nsamples,
			 mad_fixed_t const* ch1, mad_fixed_t const* ch2)
{
  mad_fixed_t s0, s1;
  
  // interleave only
  
  if (ch2 != 0) {  /* stereo */
    while (nsamples--) {
      s0 = *ch1; ++ ch1;
      s1 = *ch2; ++ ch2;
      
      pcm [0 + 0] = s0;
      pcm [0 + 1] = s1;
      
      pcm += 2;
    }
  }
  else {  /* mono */
    while (nsamples--) {
      s0 = *ch1; ++ ch1;
      
      pcm [0] = s0;
      
      pcm += 1;
      
    }
  }
}

#ifdef TO_BE_USED

/*
 * NAME:                parse_xing()
 * DESCRIPTION: read a Xing VBR tag
 */
static
int parse_xing(struct xing *xing, struct mad_bitptr ptr, unsigned int bitlen)
{
  if (bitlen < 64 || mad_bit_read(&ptr, 32) != XING_MAGIC)
    goto fail;

  xing->flags = mad_bit_read(&ptr, 32);
  bitlen -= 64;

  if (xing->flags & XING_FRAMES) {
    if (bitlen < 32)
      goto fail;

    xing->frames = mad_bit_read(&ptr, 32);
    bitlen -= 32;
  }

  if (xing->flags & XING_BYTES) {
    if (bitlen < 32)
      goto fail;

    xing->bytes = mad_bit_read(&ptr, 32);
    bitlen -= 32;
  }

  if (xing->flags & XING_TOC) {
    int i;

    if (bitlen < 800)
      goto fail;

    for (i = 0; i < 100; ++i)
      xing->toc[i] = (unsigned char) mad_bit_read(&ptr, 8);

    bitlen -= 800;
  }

  if (xing->flags & XING_SCALE) {
    if (bitlen < 32)
      goto fail;

    xing->scale = mad_bit_read(&ptr, 32);
    bitlen -= 32;
  }

  return 0;

 fail:
  xing->flags = 0;
  return -1;
}

/*
 * NAME:                scan_header()
 * DESCRIPTION: read the initial frame(s) to get stream statistics
 */
static
int scan_header(HANDLE file, struct mad_header *header, struct xing *xing)
{
  struct mad_stream t_stream;
  struct mad_frame t_frame;
  unsigned char t_buffer[8192];
  unsigned int buflen = 0;
  int count = 0, result = 0;
  
  mad_stream_init (&t_stream);
  mad_frame_init (&t_frame);
  
  m_xing -> flags = 0;

  while (1) {
    if (buflen < sizeof (buffer) ) {
      DWORD bytes;
      
      if (ReadFile(file, buffer + buflen, sizeof(buffer) - buflen,
		   &bytes, 0) == FALSE || bytes == 0) {
	result = -1;
	break;
      }
      
      buflen += bytes;
    }
    
    mad_stream_buffer(&stream, buffer, buflen);
    
    while (1) {
      if (mad_frame_decode(&frame, &stream) == -1) {
                                if (!MAD_RECOVERABLE(stream.error))
				  break;
				
                                continue;
      }
      
      if (count++ ||
	  (xing && Parse_Xing(xing, stream.anc_ptr,
			      stream.anc_bitlen) == 0))
	break;
    }
    
    if (count || stream.error != MAD_ERROR_BUFLEN)
      break;
    
    memmove(buffer, stream.next_frame,
	    buflen = &buffer[buflen] - stream.next_frame);
  }
  
  if (count) {
    if (header)
      *header = frame.header;
  }
  else
    result = -1;
  
  mad_frame_finish(&frame);
  mad_stream_finish(&stream);
  
  return result;
}

/*
 * NAME:                round()
 * DESCRIPTION: round and scale sample
 */
static __inline
signed int round(mad_fixed_t sample, struct dither *dither)
{
        enum {
                MIN = -MAD_F_ONE,
                MAX =  MAD_F_ONE - 1
        };

        /* round */
        sample += (1L << (MAD_F_FRACBITS - SAMPLE_DEPTH));

        /* clip */
        if (sample > MAX)
                sample = MAX;
        else if (sample < MIN)
                sample = MIN;

        /* quantize */
        return sample >> (MAD_F_FRACBITS + 1 - SAMPLE_DEPTH);
}

/*
 * NAME:                prng()
 * DESCRIPTION: 32-bit pseudo-random number generator
 */
static __inline
unsigned long prng(unsigned long state)
{
  return (state * 0x0019660dL + 0x3c6ef35fL) & 0xffffffffL;
}

/*
 * NAME:                dither()
 * DESCRIPTION: dither and scale sample
 */
static __inline
signed int dither(mad_fixed_t sample, struct dither *dither)
{
        unsigned int scalebits;
        mad_fixed_t output, mask, random;

        enum {
                MIN = -MAD_F_ONE,
                MAX =  MAD_F_ONE - 1
        };

        /* noise shape */
        sample += dither->error[0] - dither->error[1] + dither->error[2];

        dither->error[2] = dither->error[1];
        dither->error[1] = dither->error[0] / 2;

        /* bias */
        output = sample + (1L << (MAD_F_FRACBITS + 1 - SAMPLE_DEPTH - 1));

        scalebits = MAD_F_FRACBITS + 1 - SAMPLE_DEPTH;
        mask = (1L << scalebits) - 1;

        /* dither */
        random  = prng(dither->random);
        output += (random & mask) - (dither->random & mask);

        dither->random = random;

        /* clip */
        if (output > MAX) {
                output = MAX;

                if (sample > MAX)
                        sample = MAX;
        }
        else if (output < MIN) {
                output = MIN;

                if (sample < MIN)
                        sample = MIN;
        }

        /* quantize */
        output &= ~mask;

        /* error feedback */
        dither->error[0] = sample - output;

        /* scale */
        return output >> scalebits;
}

/*
 * NAME:                pack_pcm()
 * DESCRIPTION: scale and dither MAD output
 */
static
void pack_pcm(unsigned char **pcm, unsigned int nsamples,
                          mad_fixed_t const *ch1, mad_fixed_t const *ch2)
{
        register signed int s0, s1;
        static struct dither d0, d1;

        if (ch2) {  /* stereo */
                while (nsamples--) {
                        s0 = scale(*ch1++, &d0);
                        s1 = scale(*ch2++, &d1);
# if SAMPLE_DEPTH == 16
                        (*pcm)[0 + 0] = s0 >> 0;
                        (*pcm)[0 + 1] = s0 >> 8;
                        (*pcm)[2 + 0] = s1 >> 0;
                        (*pcm)[2 + 1] = s1 >> 8;

                        *pcm += 2 * 2;
# elif SAMPLE_DEPTH == 8
                        (*pcm)[0] = s0 ^ 0x80;
                        (*pcm)[1] = s1 ^ 0x80;

                        *pcm += 2;
# else
#  error "bad SAMPLE_DEPTH"
# endif
                }
        }
        else {  /* mono */
                while (nsamples--) {
                        s0 = scale(*ch1++, &d0);

# if SAMPLE_DEPTH == 16
                        (*pcm)[0] = s0 >> 0;
                        (*pcm)[1] = s0 >> 8;

                        *pcm += 2;
# elif SAMPLE_DEPTH == 8
                        *(*pcm)++ = s0 ^ 0x80;
# endif
                }
        }
}

/*
 * NAME:                cleanup()
 * DESCRIPTION: close file and free memory
 */
static
void cleanup(CPs_CoDec_MPEG *context)
{
        if (context->file != INVALID_HANDLE_VALUE) {
                CloseHandle(context->file);
                context->file = INVALID_HANDLE_VALUE;

                mad_synth_finish(&context->synth);
                mad_frame_finish(&context->frame);
                mad_stream_finish(&context->stream);
        }
}



void CP_InitialiseCodec_MPEG(CPs_CoDecModule* pCoDec)
{
        CPs_CoDec_MPEG *pContext;

        // Setup functions
        pCoDec->Uninitialise = CPP_OMMP3_Uninitialise;
        pCoDec->OpenFile = CPP_OMMP3_OpenFile;
        pCoDec->CloseFile = CPP_OMMP3_CloseFile;
        pCoDec->Seek = CPP_OMMP3_Seek;
        pCoDec->GetFileInfo = CPP_OMMP3_GetFileInfo;
        
        pCoDec->SetEQ = CPP_OMMP3_SetEQ;
        pCoDec->SetOptions = CPP_OMMP3_SetOptions;
        pCoDec->GetPCMBlock = CPP_OMMP3_GetPCMBlock;
        pCoDec->GetCurrentPos_secs = CPP_OMMP3_GetCurrentPos_secs;
        
        // Setup private data
        pCoDec->m_pModuleCookie = malloc(sizeof(CPs_CoDec_MPEG));
        pContext = (CPs_CoDec_MPEG*)pCoDec->m_pModuleCookie;
        pContext->file = INVALID_HANDLE_VALUE;
        pContext->size = 0;

        pContext->options.m_iPretendOption = 42;

        pContext->xing.flags = 0;

        pContext->samplecount = 0;

        pContext->timer  = mad_timer_zero;
        pContext->length = mad_timer_zero;
        pContext->rate   = 0;
        pContext->frames = 0;

        pContext->info.m_iFileLength_Secs = 0;
        pContext->info.m_iBitRate_Kbs     = 0;
        pContext->info.m_iFreq_Hz         = 0;
        pContext->info.m_bStereo          = TRUE;
        pContext->info.m_b16bit           = (SAMPLE_DEPTH == 16);

        pContext->buflen = 0;

        CPFA_InitialiseFileAssociations(pCoDec);
        CPFA_AddFileAssociation(pCoDec, "MP3");
        CPFA_AddFileAssociation(pCoDec, "MP2");
        CPFA_AddFileAssociation(pCoDec, "MP1");
}


/*
 * NAME:                codec->Destroy()
 * DESCRIPTION: clean-up and delete a codec object
 */
void CPP_OMMP3_Uninitialise(CPs_CoDecModule* pModule)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;
        CP_CHECKOBJECT(context);

        cleanup(context);

        free(context);

        CPFA_EmptyFileAssociations(pModule);
}

/*
 * NAME:                codec->OpenFile()
 * DESCRIPTION: open a new MPEG audio stream
 */
BOOL CPP_OMMP3_OpenFile(CPs_CoDecModule* pModule, char const *path)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;
        CP_CHECKOBJECT(context);

        cleanup(context);

        CP_TRACE1("CPI_CoDec__OpenFile: \"%s\"", path);

        context->file = CreateFile(path, GENERIC_READ,
                                                           FILE_SHARE_READ | FILE_SHARE_WRITE, 0,
                                                           OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, 0);
        if (context->file == INVALID_HANDLE_VALUE) {
                CP_TRACE0("CPI_CoDec__OpenFile: failed");
                return FALSE;
        }

        mad_stream_init(&context->stream);
        mad_frame_init(&context->frame);
        mad_synth_init(&context->synth);

        if (scan_header(context->file,
                                        &context->frame.header, &context->xing) == -1) {
                CP_TRACE0("CPI_CoDec__OpenFile: failed to read file header");
                cleanup(context);
                return FALSE;
        }

        SetFilePointer(context->file, 0, 0, FILE_BEGIN);

        context->size             = GetFileSize(context->file, 0);
        context->synth.pcm.length = 0;
        context->samplecount      = 0;
        context->timer            = mad_timer_zero;

        if (context->xing.flags & XING_FRAMES) {
                context->length = context->frame.header.duration;
                mad_timer_multiply(&context->length, context->xing.frames);
        }
        else {
                /* estimate playing time from file size */

                mad_timer_set(&context->length, 0,
                                          1, context->frame.header.bitrate / 8);
                mad_timer_multiply(&context->length, context->size);
        }

        context->rate   = 0;
        context->frames = 0;

        context->info.m_iFileLength_Secs =
                mad_timer_count(context->length, MAD_UNITS_SECONDS);
        context->info.m_iBitRate_Kbs     = context->frame.header.bitrate / 1000;
        context->info.m_iFreq_Hz         = context->frame.header.samplerate;
        context->info.m_bStereo          =
                context->frame.header.mode == MAD_MODE_SINGLE_CHANNEL ? FALSE : TRUE;
        context->info.m_b16bit           = (SAMPLE_DEPTH == 16);

        context->buflen = 0;

        return TRUE;
}

/*
 * NAME:                codec->CloseFile()
 * DESCRIPTION: Closes the file
 */
void CPP_OMMP3_CloseFile(CPs_CoDecModule* pModule)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;
        CP_CHECKOBJECT(context);
        CP_TRACE0("Close MPEG file");

        cleanup(context);
}

/*
 * NAME:                codec->Seek()
 * DESCRIPTION: seek to a specific location
 */
void CPP_OMMP3_Seek(CPs_CoDecModule* pModule, int const numer, int const denom)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;
        double fraction;
        unsigned long position;

        CP_CHECKOBJECT(context);

        if (context->file == INVALID_HANDLE_VALUE) {
                CP_TRACE0("CPI_CoDec__Seek: no file is open");
                return;
        }

        if (numer < 0 || denom <= 0) {
                CP_TRACE0("CPI_CoDec__Seek: numer < 0 || denom <= 0");
                return;
        }

        fraction = (double) numer / denom;
        position = (unsigned long)
                (mad_timer_count(context->length, MAD_UNITS_MILLISECONDS) * fraction);

        mad_timer_set(&context->timer, position / 1000, position % 1000, 1000);

        if (context->xing.flags & XING_TOC) {
                int percent, p1, p2;

                percent = (int) (fraction * 100);
                p1 = (percent < 100) ? context->xing.toc[percent    ] : 0x100;
                p2 = (percent <  99) ? context->xing.toc[percent + 1] : 0x100;

                fraction = (p1 + (p2 - p1) * (fraction * 100 - percent)) / 0x100;
        }

        SetFilePointer(context->file, (LONG) (context->size * fraction), 0,
                                   FILE_BEGIN);
        if (ReadFile(context->file, context->buffer, sizeof(context->buffer),
                                 &context->buflen, 0) == FALSE)
                context->buflen = 0;

        mad_stream_buffer(&context->stream, context->buffer, context->buflen);

        mad_frame_mute(&context->frame);
        mad_synth_mute(&context->synth);

        if (numer) {
                int skip;

                skip = 2;
                do {
                        if (mad_frame_decode(&context->frame, &context->stream) == 0) {
                                mad_timer_add(&context->timer, context->frame.header.duration);
                                if (--skip == 0)
                                        mad_synth_frame(&context->synth, &context->frame);
                        }
                        else if (!MAD_RECOVERABLE(context->stream.error))
                                break;
                }
                while (skip);
        }

        context->synth.pcm.length = 0;
        context->samplecount      = 0;
}

/*
 * NAME:                codec->SetEQ()
 * DESCRIPTION: change EQ settings
 */
void CPP_OMMP3_SetEQ(CPs_CoDecModule* pModule, CPs_EQSettings const *eq)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;

        CP_CHECKOBJECT(context);

        /* ... */
}

/*
 * NAME:                codec->SetOptions()
 * DESCRIPTION: change codec options
 */
void CPP_OMMP3_SetOptions(CPs_CoDecModule* pModule, CPs_CoDecOptions const *options)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;

        CP_CHECKOBJECT(context);

        context->options = *options;
}

/*
 * NAME:                codec->GetPCMBlock()
 * DESCRIPTION: decode and return the next PCM block
 */
BOOL CPP_OMMP3_GetPCMBlock(CPs_CoDecModule* pModule, void *block, DWORD *size)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;
        unsigned char *samples = block;
        unsigned int nsamples;

        CP_CHECKOBJECT(context);
        CP_ASSERT(context->file != INVALID_HANDLE_VALUE);

        nsamples = (*size / (SAMPLE_DEPTH / 8)) >>
                (context->info.m_bStereo ? 1 : 0);
        *size = 0;

        while (nsamples) {
                unsigned int count, bitrate;

                count = context->synth.pcm.length - context->samplecount;
                if (count > nsamples)
                        count = nsamples;

                if (count) {
                        mad_fixed_t const *ch1, *ch2;

                        ch1 = context->synth.pcm.samples[0] + context->samplecount;
                        ch2 = context->synth.pcm.samples[1] + context->samplecount;

                        if (context->info.m_bStereo == FALSE)
                                ch2 = 0;
                        else if (context->synth.pcm.channels == 1)
                                ch2 = ch1;

                        pack_pcm(&samples, count, ch1, ch2);

                        context->samplecount += count;
                        nsamples             -= count;

                        if (nsamples == 0)
                                break;
                }

                while (mad_frame_decode(&context->frame, &context->stream) == -1) {
                        DWORD bytes;

                        if (MAD_RECOVERABLE(context->stream.error))
                                continue;

                        if (context->stream.next_frame) {
                                memmove(context->buffer, context->stream.next_frame,
                                                context->buflen = context->buffer +
                                                context->buflen - context->stream.next_frame);
                        }

                        if (ReadFile(context->file, context->buffer +
                                                 context->buflen, sizeof(context->buffer) -
                                                 context->buflen, &bytes, 0) == FALSE || bytes == 0)
                                return FALSE;

                        mad_stream_buffer(&context->stream,
                                                          context->buffer, context->buflen += bytes);
                }

                bitrate = context->frame.header.bitrate / 1000;

                context->rate += bitrate;
                context->frames++;

                context->info.m_iBitRate_Kbs = bitrate;

                mad_synth_frame(&context->synth, &context->frame);

                context->samplecount = 0;

                mad_timer_add(&context->timer, context->frame.header.duration);
        }

        *size = samples - (unsigned char *) block;

        return TRUE;
}

/*
 * NAME:                codec->GetFileInfo()
 * DESCRIPTION: return statistics about the current stream
 */
void CPP_OMMP3_GetFileInfo(CPs_CoDecModule* pModule, CPs_FileInfo *info)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;

        CP_CHECKOBJECT(context);

        if (!(context->xing.flags & XING_FRAMES) && context->frames) {
                /* update length estimate */

                mad_timer_set(&context->length, 0,
                                          1, (context->rate / context->frames) * (1000 / 8));
                mad_timer_multiply(&context->length, context->size);

                if (mad_timer_compare(context->timer, context->length) > 0) {
                        context->length = context->timer;
                        context->size   = GetFileSize(context->file, 0);
                }

                context->info.m_iFileLength_Secs =
                        mad_timer_count(context->length, MAD_UNITS_SECONDS);
        }

        *info = context->info;
}

/*
 * NAME:                codec->GetCurrentPos_secs()
 * DESCRIPTION: return the current playing time position
 */
int CPP_OMMP3_GetCurrentPos_secs(CPs_CoDecModule* pModule)
{
        CPs_CoDec_MPEG *context = (CPs_CoDec_MPEG *)pModule->m_pModuleCookie;

        CP_CHECKOBJECT(context);

        return mad_timer_count(context->timer, MAD_UNITS_SECONDS);
}

#endif

#endif // EXT_FIO_MP3 == 1
