/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   Helper Classes to configure single values with constant ranges
 */

#include "ValuesGeneric.hh"

using namespace GSMP;
using namespace GSML;

SingleCounter::SingleCounter (int min_v, int max_v, int def_v, int& result, string name)
  : res (result)
{
  m_max = max_v;
  m_def = def_v;
  res = m_def;
  label = name;
}

int SingleCounter::Min ()
{
  return m_min;
}

int SingleCounter::Max ()
{
  return m_max;
}

int SingleCounter::Default ()
{
  return m_def;
}

int SingleCounter::Set (int val)
{
  res = min (max (val, m_min), m_max);
  ValueChanged ();
  return res;
}

SingleVolume::SingleVolume (double min_v, double max_v, double def_v, double& result, string name)
  : res (result)
{
  m_min = min_v;
  m_max = max_v;
  def = def_v;
  res = def;
  label = name;
}


double SingleVolume::Get ()
{
  return res;
}

double SingleVolume::Min ()
{
  return m_min;
}

double SingleVolume::Max ()
{
  return m_max;
}

double SingleVolume::Default ()
{
  return def;
}

double SingleVolume::Set (double val)
{
  res = min (max (val, m_min), m_max);
  ValueChanged ();
  return res;
}


SingleCheck::SingleCheck (bool def_state, bool& result, string name)
  : res (result)
{
  label = name;
  res = def_state;
  def = def_state;
}


bool SingleCheck::State ()
{
  return res;
}

bool SingleCheck::Set (bool state)
{
  res = state;
  ValueChanged ();
  return state;
}

bool SingleCheck::Default ()
{
  return def;
}





SingleShortTime::SingleShortTime (sample_index min_t, sample_index max_t, sample_index def_t, sample_index& result,
				  int samples_per_second, string name, sample_index stepping = 1, bool linear = true)
  : res (result)
{
  m_min = min_t;
  m_max = max_t;
  def = def_t;
  res = def;
  label = name;
  sps = samples_per_second;
  step = stepping;
  lin = linear;
}


sample_index SingleShortTime::Get ()
{
  return res;
}

sample_index SingleShortTime::Min ()
{
  return m_min;
}

sample_index SingleShortTime::Max ()
{
  return m_max;
}

sample_index SingleShortTime::Default ()
{
  return def;
}

sample_index SingleShortTime::Set (sample_index val)
{
  res = min (max (val, m_min), m_max);
  // Todo: snap to step..
  ValueChanged ();
  return res;
}

sample_index SingleShortTime::MinStepping ()
{
  return step;
}

Scaleable::Behavior SingleShortTime::Scale ()
{
  return (lin) ? Scaleable::LINEAR : Scaleable::EXP;
}


int SingleShortTime::SamplesPerSecond ()
{
  return sps;
}





SingleFrequency::SingleFrequency (double min_f, double max_f, double def_f, double& result, int samples_per_second, string name)
  : res (result)
{
  m_min = min_f;
  m_max = max_f;
  def = def_f;
  res = def;
  label = name;
  sps = samples_per_second;
}


double SingleFrequency::Get ()
{
  return res;
}

double SingleFrequency::Min ()
{
  return m_min;
}

double SingleFrequency::Max ()
{
  return m_max;
}

double SingleFrequency::Default ()
{
  return def;
}

double SingleFrequency::Set (double val)
{
  res = min (max (val, m_min), m_max);
  ValueChanged ();
  return res;
}

Scaleable::Behavior SingleFrequency::Scale ()
{
  return Scaleable::SQUARE;
}


int SingleFrequency::SamplesPerSecond ()
{
  return sps;
}
