/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   View interface.
 */

#include "View.hh"

using namespace GSMP;
using namespace GSMX_utils::Plugin;

ViewImpl::ViewImpl (View* i_view)
{
  m_view = i_view;
  DEBUG__(GSMP,low_level) cout << "ViewImpl constructed "<< endl;
}

string ViewImpl::Name () const
{
  return m_view -> ViewImplType ();
} 

Project* View::GetProject () 
{
  return m_project;
}

const Project* View::GetProject () const
{
  return m_project;
}




bool View:: IsVisible ()
{
  return (m_win);
}

SimpleWindow* View:: MyWindow ()
{
  return m_win;
}


void View::SwitchToViewImpl (const string& view_impl_type)
{
  if (view_impl_type == m_current_impl_type)
    return;

  DEBUG__(GSMP,detail)
    cout << "switching view type to" << view_impl_type << endl;

  m_fmc . Check4Corruption ();

  ViewImpl* t_new_view = Build <ViewImpl, View*> (string ("ViewImpl"), view_impl_type, this);
  m_current_impl_type = view_impl_type;
  ViewImpl* t_old_view = m_current_view; // signal View change before deleteing old ViewImpl...
  m_current_view = t_new_view;
  SignalViewImplExchanged ();
  delete t_old_view;
}


View :: View (Project* i_project)
  : m_fmc ("View")
{
  DEBUG__(GSMP,detail)
    cout << "Instantiating new View" << endl;

  m_fmc . Check4Corruption ();

  m_project = i_project;
  m_app = m_project -> GetApp ();
  m_current_view = 0;
  m_win = 0;
  m_current_impl_type = "";

  m_app -> m_manager -> InitializeMenuData (this);

  m_fmc . Check4Corruption ();

  if (!m_current_view)
    {
      DEBUG__(GSMP,low_level)
	cout << "We still have no view implementation, so we construct default one." << endl;
      SwitchToViewImpl (m_app -> GetPreferences () . default_view_type);
    }

  m_fmc . Check4Corruption ();
}


View :: View (const View& copy_from)
  : m_fmc ("View")
{
  DEBUG__(GSMP,detail)
    cout << "Copy View" << endl;

  m_fmc . Check4Corruption ();

  m_project = copy_from.m_project;
  m_app = m_project -> GetApp ();
  m_app -> m_manager -> InitializeMenuDataCopy (this);

  m_fmc . Check4Corruption ();

  m_current_view = copy_from.m_current_view -> Clone (this);

  m_fmc . Check4Corruption ();

  m_current_impl_type = copy_from.m_current_impl_type;
  m_win = 0;
  //m_win_reparent_hack = false;
}


View::~View ()
{
  DEBUG__(GSMP,detail)
    cout << "Destructing View " << this << endl;

  if (m_current_view)
    {
      delete m_current_view;
      m_current_view = 0;
    }
  else
    cout << "Warning: ViewImpl == 0";
}

void View::SetWindow (SimpleWindow* new_window)
{
  DEBUG__(GSMP,low_level)
    cout << "View got new Window" << endl;

  m_win = new_window;

  if (m_win)
    m_project -> SetViewVisible (this);
  else
    m_project -> SetViewOphaned (this);

  SignalWindowExchanged ();
}

