/* Test application to test the data integrity of the ExtFileIOStream
 * objects.
 * (C) 2001 Ren Rebe
 */

#include "Basics.hh"
#include "ExtFileIO_impl.hh"

#include "Threads.hh"

#include "sstream"
#include "deque"

#define TESTSTREAM MP3Stream
#define TESTFNAME "/home/everybody/music/Abdul, Paula/spellbound/track10.cdda.mp3"

//#define TESTSTREAM OggVorbisStream
//#define TESTFNAME "/home/everybody/music/Corrs, The/forgiven not forgotten/track08.cdda.ogg"

#define MAX_MEM (128 * 1024 * 1024)
#define READ_CHUNK (1024 * 8)

using namespace std;
using namespace GSML;
using namespace GSMP;

// global structs and complete file dump

track_index m_tracks;
sample_index m_length;

pcm_24_8bit::value_type* m_data_cache;
sample_index m_data_cache_length;

/**************************************
 * global cached-data compare methods *
 **************************************/

bool RawCompareWithCache (sample_index pos, int length,
			  pcm_24_8bit::value_type* data)
{
  bool t_ok = true;
  
  for (int i = 0; t_ok && i < length; ++ i) {
    if (! (m_data_cache [pos * m_tracks + i] == data [i]) )
      t_ok = false;
  }
  
  return t_ok;
}

signed int RawFindPatternInCache (sample_index expected_pos, int length,
				  pcm_24_8bit::value_type* data)
{
  int test_region = 52323;
  sample_index t_test_end = limit_max (expected_pos + test_region, m_length);
  sample_index t_test_pos = limit_min (t_test_end - 2 * test_region, 0);
  bool t_pattern_found = false;
  
  cout << "  testing for pattern match: " << t_test_pos << " - " << t_test_end << endl;
  
  while (!t_pattern_found && t_test_pos < t_test_end)
    {
      //cout << ".";
      if (RawCompareWithCache (t_test_pos, length, data) )
	t_pattern_found = true;
      else
	++ t_test_pos;
    } // end while pattern search
  
  if (t_pattern_found)
    return t_test_pos - expected_pos;
  else
    return 0;
}

bool InterleavedCompareWithCache (sample_index pos, int length, int track,
				  pcm_24_8bit::value_type* data)
{
  bool t_ok = true;
  
  for (int i = 0; t_ok && i < length; ++ i) {
    if (! (m_data_cache [(pos + i) * m_tracks + track] == data [i]) )
      t_ok = false;
  }
  
  return t_ok;
}

signed int InterleavedFindPatternInCache (sample_index expected_pos, int length, int track,
					  pcm_24_8bit::value_type* data)
{
  int test_region = 52323;
  sample_index t_test_end = limit_max (expected_pos + test_region, m_length);
  sample_index t_test_pos = limit_min (t_test_end - 2 * test_region, 0);
  bool t_pattern_found = false;
  
  cout << "  testing for pattern match: " << t_test_pos << " - " << t_test_end << endl;
  
  while (!t_pattern_found && t_test_pos < t_test_end)
    {
      //cout << ".";
      if (InterleavedCompareWithCache (t_test_pos, length, track, data) )
	t_pattern_found = true;
      else
	++ t_test_pos;
    } // end while pattern search
  
  if (t_pattern_found)
    return t_test_pos - expected_pos;
  else
    return 0;
}

/***********************
 * Test interface base *
 ***********************/

class Test
{
public:
  
  Test () {
    test_name = "unnamed"; errors = 0; error_desc . str ("");
  }
  
  // returns wether the test got passed successfully or not
  bool Run (StreamIO* stream) {
    cout << endl
	 << "... Preparing ..." << endl;
    
    cout << "-->> " << test_name << endl;
    bool t_ret = RunImpl (stream);
    cout << "<<-- " << test_name;
    
    if (t_ret)
      cout << " ... OK.";
    else
      cout << " ... FAILD !!!";
    
    cout << " - " << errors << " errors." << endl;
    
    cout << "RESULT: " << error_desc . str () << endl;
    
    return t_ret;
  }
  
  int errors;
  stringstream error_desc;
  
protected:
  
  virtual bool RunImpl (StreamIO* stream) = 0;
  
  string test_name;
};

/************
 * "Test" 0 *
 ************/

class Test0 : public Test
{
public:
  Test0 () {
    test_name = "\"Test\"0: (StreamIO: read file into memmory)";
  }
protected:
  virtual bool RunImpl (StreamIO* stream);
  
};

bool Test0::RunImpl (StreamIO* stream)
{
  cout << "1st read the whole file into memory ..." << endl;
  
  m_tracks = stream -> Tracks ();
  m_length = stream -> Length ();
  
  m_data_cache_length = limit_max (m_length, MAX_MEM / sizeof (pcm_24_8bit::value_type) / m_tracks);
  
  cout << "  tracks: " << m_tracks << ", length: " << m_length
       << ", cached length: " << m_data_cache_length << endl;
  
  m_data_cache = new pcm_24_8bit::value_type [m_data_cache_length * m_tracks];
  
  sample_index t_read = 0;
  while (t_read < m_data_cache_length)
    {
      cout << ".";
      sample_index length = limit_max (READ_CHUNK, m_data_cache_length - t_read);
      stream -> Read (length, & (m_data_cache [t_read * m_tracks] ) );
      t_read += length;
    }
  cout << "  read: " << t_read << endl;
  return true;
}

/**********
 * Test 1 *
 **********/

class Test1 : public Test
{
public:
  Test1 () {
    test_name = "Test1: (StreamIO: reopen - many times)";
  }
protected:
  virtual bool RunImpl (StreamIO* stream);
  
};

bool Test1::RunImpl (StreamIO* stream)
{
  TESTSTREAM* t_stream = new TESTSTREAM ();
  
  int tests = 100;
  bool test_error = false;
  
  while (tests > 0 && ! test_error)
    {
      cout << ".";
      if (! t_stream -> Open (TESTFNAME) ) {
	cout << "  Error opening stream!" << endl;
	test_error = true;
	continue;
      }
       if (! t_stream -> Close () ) {
	cout << "  Error closing stream!" << endl;
	test_error = true;
	continue;
      }
      -- tests;
    }
  
  delete t_stream;
  
  errors = test_error;
  error_desc << "errors: " << test_error;
  
  return errors == 0;
}

/**********
 * Test 2 *
 **********/

class Test2 : public Test
{
public:
  Test2 () {
    test_name = "Test2: (StreamIO: data_after_seek)";
  }
protected:
  virtual bool RunImpl (StreamIO* stream);
};

bool Test2::RunImpl (StreamIO* stream)
{
  int test_seeks = 500;
  int test_seek_errors = 0;
  int test_read_errors = 0;
  int test_data_errors = 0;
  int test_data_jitters = 0;
  
  sample_index t_last_seek = 0;
  
  while (test_seeks > 0)
    {
      pcm_24_8bit::value_type t_seek_data [READ_CHUNK * m_tracks];
      
      double t_pos = (double) random () / RAND_MAX * m_data_cache_length;
      long pos = (long) limit_max (t_pos, m_data_cache_length - READ_CHUNK);
      
      //cout << "  seek to pos: " << pos << endl;
      if (stream -> Seek (pos) == pos) {
	if (stream -> Read (READ_CHUNK, t_seek_data) == READ_CHUNK)
	  {
	    bool t_error = false;
	    
	    if (! RawCompareWithCache (pos, READ_CHUNK * m_tracks, t_seek_data) )
	      {
		cout << "  data error !! last_seek: " << t_last_seek << ", seek: " << pos << endl;
		++ test_data_errors;
		
		// try to find the pattern in the region arround ...
		int t_delta = RawFindPatternInCache (pos, READ_CHUNK * m_tracks, t_seek_data);
		if (t_delta != 0) {
		  cout << "  FOUND PATTERN: at delta: " << t_delta << endl;
		  ++ test_data_jitters;
		}
		else
		  cout << "  Pattern not found in the near surrounding ..." << endl;
		
	      } // end if not Data OK
	  }
	else {
	  cout << "  read error ?!" << endl;
	  ++ test_read_errors;
	  }
      }
      else {
	cout << "  seek error ?!" << endl;
	++ test_seek_errors;
      }
      
      t_last_seek = pos;
      -- test_seeks;
    }
  
  errors = test_seek_errors + test_read_errors + test_data_errors;
  error_desc << "seek_errors: " << test_seek_errors
	     << ", read_errors: " << test_read_errors
	     << ", data_errors: " << test_data_errors
	     << " (jitter: " << test_data_jitters << ")";
  
  return errors == 0;
}

/***********
 * Test 10 *
 ***********/

class Test10 : public Test
{
public:
  Test10 () {
    test_name = "Test10: (RandomAccessIO: normal chain data correctness)";
  }
protected:
  virtual bool RunImpl (StreamIO* stream);
};

bool Test10::RunImpl (StreamIO* stream)
{
  StreamIO* t_stream = stream -> Clone (); // because the stream will be deleted by the top-layer dtors ...  
  
  FileCache* t_cache = new FileCache (t_stream);
  FileStreamer* t_streamer = new FileStreamer (t_cache);
  
  RandomAccessIO* t_obj = t_streamer;
  
  int test_seeks = 500;
  int test_read_errors = 0;
  int test_data_errors = 0;
  int test_data_jitters = 0;
  
  sample_index t_last_seek = 0;
  
  for (int t_i = 0; t_i < test_seeks; ++ t_i)
    {
      pcm_24_8bit::value_type t_seek_data [READ_CHUNK];
      
      double t_pos = (double) random () / RAND_MAX * m_data_cache_length;
      long pos = (long) limit_max (t_pos, m_data_cache_length - READ_CHUNK);
      
      //cout << "  read at pos: " << pos << endl;
      if (t_obj -> Read (pos, READ_CHUNK, 0, t_seek_data) == READ_CHUNK)
	{
	  bool t_error = false;
	  
	  if (! InterleavedCompareWithCache (pos, READ_CHUNK, 0, t_seek_data) )
	    {
	      cout << "  data error !! last_read: " << t_last_seek << ", read: " << pos << endl;
	      ++ test_data_errors;
	      
	      // try to find the pattern in the region arround ...
	      int t_delta = InterleavedFindPatternInCache (pos, READ_CHUNK * m_tracks, 0, t_seek_data);
	      if (t_delta != 0) {
		cout << "  FOUND PATTERN: at delta: " << t_delta << endl;
		++ test_data_jitters;
	      }
	      else
		cout << "  Pattern not found in the near surrounding ..." << endl;
	    } // end if not Data OK
	}
      else {
	cout << "  read error ?!" << endl;
	++ test_read_errors;
      }
      
      t_last_seek = pos;
    }
  
  delete t_obj;
  
  errors = test_read_errors + test_data_errors;
  error_desc  << "read_errors: " << test_read_errors << "/" << test_seeks
	      << ", data_errors: " << test_data_errors << "/" << test_seeks
	      << " (jitter: " << test_data_jitters << ")";
  
  return errors == 0;
}

/***********
 * Test 11 *
 ***********/

class Test11 : public Test
{
public:
  Test11 () {
    test_name = "Test11: (RandomAccessIO: threaded access)";
  }
protected:
  virtual bool RunImpl (StreamIO* stream);
  
  class Child : public GSMX_utils::Threads::Job
  {
  public:
    Child (RandomAccessIO* i_io) {m_io = i_io; Ready (false);}
    
    void Start () {Continue ();};
    
    int test_seeks;
    int test_read_errors;
    int test_data_errors;
    int test_data_jitters;
    
  protected:
    virtual void Loop ();
    
    RandomAccessIO* m_io;
  };
};

void Test11::Child::Loop ()
{
  cout << "Loop" << endl;
  
  test_seeks = 500;
  test_read_errors = 0;
  test_data_errors = 0;
  test_data_jitters = 0;
  
  sample_index t_last_seek = 0;
  
  for (int t_i = 0; t_i < test_seeks; ++ t_i)
    {
      pcm_24_8bit::value_type t_seek_data [READ_CHUNK];
      
      double t_pos = (double) random () / RAND_MAX * m_data_cache_length;
      long pos = (long) limit_max (t_pos, m_data_cache_length - READ_CHUNK);
      
      //cout << "  read at pos: " << pos << endl;
      if (m_io -> Read (pos, READ_CHUNK, 0, t_seek_data) == READ_CHUNK)
	{
	  bool t_error = false;
	  
	  if (! InterleavedCompareWithCache (pos, READ_CHUNK, 0, t_seek_data) )
	    {
	      cout << "  data error !! last_read: " << t_last_seek << ", read: " << pos << endl;
	      ++ test_data_errors;
	      
	      // try to find the pattern in the region arround ...
	      int t_delta = InterleavedFindPatternInCache (pos, READ_CHUNK * m_tracks, 0, t_seek_data);
	      if (t_delta != 0) {
		cout << "  FOUND PATTERN: at delta: " << t_delta << endl;
		++ test_data_jitters;
	      }
	      else
		cout << "  Pattern not found in the near surrounding ..." << endl;
	    } // end if not Data OK
	}
      else {
	cout << "  read error ?!" << endl;
	++ test_read_errors;
      }
      
      t_last_seek = pos;
    }
  
  cout << "RESULTS: read_errors: " << test_read_errors << "/" << test_seeks
       << ", data_errors: " << test_data_errors << "/" << test_seeks
       << " (jitter: " << test_data_jitters << ")"
       << endl;
  
  Wait ();
  cout << "END Loop" << endl;
}

bool Test11::RunImpl (StreamIO* stream)
{
  StreamIO* t_stream = stream -> Clone (); // because the stream will be deleted by the top-layer dtors ...
  
  FileCache* t_cache = new FileCache (t_stream);
  FileStreamer* t_streamer = new FileStreamer (t_cache);
  
  RandomAccessIO* t_obj = t_streamer; // only stuff after the FileStreamer IS garanteed to be thread-safe!!!
  
  int t_child_count = 8;
  deque <Child*> t_childs;
  
  int test_seeks = 0;
  int test_read_errors = 0;
  int test_data_errors = 0;
  int test_data_jitters = 0;
  
  // create childs
  for (int i = 0; i < t_child_count; ++ i) {
    cout << "  creating child: " << i << endl;
    t_childs . push_back (new Child (t_obj) );
  }
  
  // start childs
  for (int i = 0; i < t_child_count; ++ i) {
    cout << "  starting child: " << i << endl;
    t_childs [i] -> Start ();
  }
  
  sleep (5);
  
  // wait for childs to finish
  
  bool t_running = true;
  while (t_running)
    {
      cout << "waiting for childs ..." << endl;
      sleep (1);
      
      // check childs
      t_running = false;
      for (int i = 0; i < t_child_count; ++ i) {
	cout << i << ", ";
	if (t_childs [i] -> IsRunning () ) {
	  cout << "  child " << i << " is running" << endl;
	  t_running = true;
	}
      }
    }
  
  cout << "child seem to be finished ..." << endl;
  
  // delete childs
  while (t_childs . size () > 0) {
    cout << "  deleting child: " << endl;
    
    test_seeks += t_childs . back () -> test_seeks;
    test_read_errors += t_childs . back () -> test_read_errors;
    test_data_errors += t_childs . back () -> test_data_errors;
    test_data_jitters += t_childs . back () -> test_data_jitters;
    
    delete t_childs . back ();
    t_childs . pop_back ();
  }
  
  errors = test_read_errors + test_data_errors;
  cout << "errors: " << errors << endl;
  error_desc  << "read_errors: " << test_read_errors << "/" << test_seeks
	      << ", data_errors: " << test_data_errors << "/" << test_seeks
	      << " (jitter: " << test_data_jitters << ")"
	      << " in " << t_child_count << " threads";
  
  return errors == 0;
}

int main ()
{
  cout << "ExtIOStreamTest" << endl;
  
  TESTSTREAM* t_stream = new TESTSTREAM ();
  
  if (! t_stream -> Open (TESTFNAME) ) {
    cout << "Stream can't open the file ... returning." << endl;
    delete t_stream;
    return false;
  }

  // must be first (reads file into cache ...)
  Test0 test0;
  test0 . Run (t_stream);
  
  cout << "\"Test0\" found: " << m_tracks << " tracks; cached length: " << m_data_cache_length << endl;
  
  // StreamIO tests
  Test1 test1;
  test1 . Run (t_stream);
  
  Test2 test2;
  test2 . Run (t_stream);
  
  // RandomAccessIO tests
  Test10 test10;
  test10 . Run (t_stream);

  Test11 test11;
  test11 . Run (t_stream);

  if (! t_stream -> Close () )
    cout  << "Stream can't close the file ????" << endl;
  
  delete t_stream;
  
  delete m_data_cache; // created by test0
}
