#include "menutest.hh"
#include "Factory.hh"

using namespace GSMP;
using namespace GSMP::Menu;


class Test : public MenuManager
{
public:
  void Run ();

  App* app;
  Project* projects[2];
  View* views[4];

  Test ()
  {
    app = new App (this);
    projects[0] = new Project (this, app, 0);
    projects[1] = new Project (this, app, 1);
    views [0] = new View (this, projects[0], 0);
    views [1] = new View (this, projects[0], 1);
    views [2] = new View (this, projects[1], 2);
    views [3] = new View (this, projects[1], 3);
  }

  ~Test ()
  {
    int i;
    for (i = 0; i<4; i++)
      delete views [i];
    for (i = 0; i<2; i++)
      delete projects [i];
    delete app;
  }
}; 

void Test::Run ()
{
  MenuCollection* collection = 0;
  string s;
  do
    {
      s = "";
      cout << "Enter activator string : ";
      cin >> s;
      try 
	{
	  if (s != "quit")
	    {

	      if (s == "debug")
		DebugMenuManager ();

	      else if (s == "collect")
		{
		  deque <Filter> i_filters;
		  i_filters.push_back (Filter (string ("File"), string ("HalloFile")));
		  i_filters.push_back (Filter (string ("stuff>useless"), string ("Some Stuff")));
		      
		  if (collection)
		    {
		      delete collection;
		      collection = 0;
		    }
		    
		  collection = CreateMenuCollection (i_filters, true);
		}
	      
	      else if (s == "show")		
		{
		  if (collection)
		    collection -> Debug ();
		  else
		    cout << "No collection allocated yet..." << endl;
		}

	      else if (s == "SI_state")
		{
		  string loc;
		  cin >> loc;
		  SimpleItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      for (int i = 0; i < 4; i ++)
			cout << "View [" << i << "] " << sif.Get () -> IsEnabled (views [i]) << endl;
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "SI_activate")
		{
		  string loc;
		  cin >> loc;
		  SimpleItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      for (int i = 0; i < 4; i ++)
			sif.Get () -> Activate (views[i]); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "SI_enable1")
		{
		  string loc;
		  cin >> loc;
		  SimpleItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> SetState (views[1], true); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "SI_disable1")
		{
		  string loc;
		  cin >> loc;
		  SimpleItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> SetState (views[1], false); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "SI_block")
		{
		  string loc;
		  cin >> loc;
		  SimpleItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> BlockAll (); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}


	      else if (s == "SI_unblock")
		{
		  string loc;
		  cin >> loc;
		  SimpleItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> UnBlockAll (); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "CI_state")
		{
		  string loc;
		  cin >> loc;
		  CheckedItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      for (int i = 0; i < 4; i ++)
			cout << "View [" << i << "] " << sif.Get () -> IsEnabled (views [i])
			     << ((sif.Get () -> IsChecked (views [i])) ? string (" x") : string (" -")) << endl;

		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "CI_activate")
		{
		  string loc;
		  cin >> loc;
		  CheckedItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      for (int i = 0; i < 4; i ++)
			sif.Get () -> Activate (views[i]); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "CI_enable1")
		{
		  string loc;
		  cin >> loc;
		  CheckedItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> SetState (views[1], true); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "CI_disable1")
		{
		  string loc;
		  cin >> loc;
		  CheckedItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> SetState (views[1], false); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "CI_block")
		{
		  string loc;
		  cin >> loc;
		  CheckedItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> BlockAll (); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}


	      else if (s == "CI_unblock")
		{
		  string loc;
		  cin >> loc;
		  CheckedItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      sif.Get () -> UnBlockAll (); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}


	      else if (s == "SE_sel")
		{
		  string loc;
		  cin >> loc;
		  SelectionItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      for (int i = 0; i < 4; i ++)
			cout << "View [" << i << "] " << sif.Get () -> IsSelected (views [i]) << endl;
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}

	      else if (s == "SE_select")
		{
		  string loc;
		  cin >> loc;
		  SelectionItemFinder sif (*this, loc);
		  if (sif.Matched ())
		    {
		      for (int i = 0; i < 4; i ++)
			sif.Get () -> Select (views[i]); 
		    }
		  else
		    cout << "No LeaveItem @ " << loc << endl;
		}


	      else  
		Add (s, string ("Foo"), string("Boo"));

	    }
	}

      catch (GSMP::Exception e)
	{
	  cout << e << endl;
	}

      catch (GSMX_utils::BaseException e)
	{
	  cout << e << endl;
	}
    } while (s != "quit");
}

void main ()
{
  Test test;
  try
    {
      GSMX_utils::Plugin::RegisterFile (string("menutestPlugins.so"));
    }
  catch (GSMX_utils::BaseException e)
    {
      cout << e << endl;
    }
  test.Run ();
}
