/* Exceptions.hh  (C) by V. Ziegler 2001
 
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLUGIN_EXCEPTIONS_HH
#define PLUGIN_EXCEPTIONS_HH

#include <string>
#include <iostream.h>

namespace GSMX_utils
{
  using namespace std;

#ifndef GSMX_BASE_EXCEPTION
#define GSMX_BASE_EXCEPTION

  /* this is the base for all Exceptions in GSMX Utils, GSML and GSMP */

  class BaseException
  {
  public:
    string info;

    BaseException () {};
    BaseException (const string& init_info) : info (init_info) {}; 
  };

  /* easy debuging output for "panic exception handlers" */

  inline ostream& operator << (ostream& s, const BaseException& e)
  {
    return s << e.info;
  }

#endif // GSMX_BASE_EXCEPTION

  namespace Plugin
  {
    /* GSMx Plugin exception */

    class Exception : public BaseException 
    {
    public:
      Exception (const string& errmsg) : BaseException (string ("Plugin-system : ") + errmsg) {};
    };


    /* Error while opening a Dl */

    class ExceptionOpenFailed : public Exception
    {
    public:
      ExceptionOpenFailed (char* dl_message) : Exception (string (dl_message)) {};
    };


    /* Dl contains no GSMx plugin system interface */

    class ExceptionNoPluginDl : public Exception
    {
    public:
      ExceptionNoPluginDl (string filename) :  Exception (filename + string (" is not an plugin dl")) {}; 
    };


    /* Error while resolving Symbol */

    class ExceptionDlError : public Exception
    {
    public:
      ExceptionDlError (char* dl_message) : Exception (string (dl_message)) {};
    };


    /* A Plugin was "reregistered" */

    class ExceptionNameConflict  : public Exception
    {
    public:
      ExceptionNameConflict (string category, string name) : Exception (string (category) + string ("->") +
									string (name) + string (" already registered")) {};
    };

    /* unknown category id*/

    class ExceptionUnknownCategory  : public Exception
    {
    public:
      ExceptionUnknownCategory (string category) : Exception (string ("no such category: ") + string (category) ) {};
    };

    /* unknown item id*/

    class ExceptionUnknownName  : public Exception
    {
    public:
      ExceptionUnknownName (string category, string name) : Exception (string ("no such plugin: ") + string (category) +
								       string ("->") + string (name) ) {};
    };



  }; // end namespace Plugin
}; // end namespase GSMX_utils

#endif // PLUGIN_EXCEPTIONS_HH
