/* Factory.hh  (C) by V. Ziegler 2001
 
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLUGIN_FACTORY_HH
#define PLUGIN_FACTORY_HH

#include <string>
#include <deque>
#include "Exceptions.hh"

namespace GSMX_utils
{
  namespace Plugin
  {
    class Info;

    typedef deque <string> content;
    typedef deque <const Info*> pluginset;

    class Info 
    {
      // Author, Date, Help, Information, Locations etc etc etc..... 
    public:
      string category;
      string name;
      string author;
      string date;
      string location;
      void* user_attached_data;
    };

    /* plugin system API */

    pluginset* RegisterFile (const string& filename);

    content* Categorys ();
    content* Items (const string& category);
    const Info& PlugInfo (const string& category, const string& name);

    void Alias (const string& category, const string& name, const string& alias);
    const Info& ResolveAlias (const string& alias);
    void UnAlias (const string& alias);

    template <class BASE>
    BASE* Build (const string& category, const string& name);

    template <class BASE, class P1>
    BASE* Build (const string& category, const string& name, P1 p1);

    template <class BASE, class P1, class P2>
    BASE* Build (const string& category, const string& name, P1 p1, P2 p2);

    template <class BASE, class P1, class P2, class P3>
    BASE* Build (const string& category, const string& name, P1 p1, P2 p2, P3 p3);

  }; // end namespace Plugin
}; // end namespase GSMX_utils

#include "Templates/Factory.cc"

#endif // PLUGIN_FACTORY_HH
