/* Command.hh      (C) by V. Ziegler 2001
 
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THREAD_COMMAND_HH
#define THREAD_COMMAND_HH

namespace GSMX_utils
{
  namespace Threads
  {
  
    class ExceptUnbound;
    class ExceptBound;

    /* Base for all commands */
    
    class Command__
    {
    protected:

      __inline__ void DropCmd (Private :: Command* cmd)
      {
	target -> PushCmd (cmd);
      }

      Job* target;
    };

    /****************************************************************
     *                    Command 0                                 *
     ****************************************************************/

    class Command0 : public Command__
    { 
    public:
      void operator () ();

      template <class C>
      void Connect (C* j, void (C::*callback) ());

      Command0 ();
    protected:
      class Recaller;
      void (Job::*call) (); friend class Recaller;
    };

     /****************************************************************
     *                    Command 1                                 *
     ****************************************************************/

    template <class P1>
    class Recaller1;

    template <class P1>
    class Command1 : public Command__
    { 
    public:
      void operator () (P1 p1);

      template <class C>
      void Connect (C* j, void (C::*callback) (P1 p1));

      Command1 ();
    protected:
      void (Job::*call) (P1); friend class Recaller1 <P1>;
    };



    /****************************************************************
     *                    Command 2                                 *
     ****************************************************************/

    template <class P1, class P2>
    class Recaller2;

    template <class P1, class P2>
    class Command2 : public Command__
    { 
    public:
      void operator () (P1 p1, P2 p2);

      template <class C>
      void Connect (C* j, void (C::*callback) (P1 p1, P2 p2));

      Command2 ();
    protected:
      void (Job::*call) (P1, P2); friend class Recaller2 <P1, P2>;
    };

    /****************************************************************
     *                    Command 3                                 *
     ****************************************************************/

    template <class P1, class P2, class P3>
    class Recaller3;

    template <class P1, class P2, class P3>
    class Command3 : public Command__
    { 
    public:
      void operator () (P1 p1, P2 p2, P3 p3);

      template <class C>
      void Connect (C* j, void (C::*callback) (P1 p1, P2 p2, P3 p3));

      Command3 ();
    protected:
      void (Job::*call) (P1, P2, P3); friend class Recaller3 <P1, P2, P3>;
    };

  }; // end namespace Threads
}; // end namespace GSMX_utils

#include "Templates/Command.cc"

#endif // THREAD_COMMAND_HH
