/* Function.hh      (C) by V. Ziegler 2001
 
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THREAD_FUNCTION_HH
#define THREAD_FUNCTION_HH

namespace GSMX_utils
{
  namespace Threads
  {
    namespace Private { class CrossExec; };

    class ExceptUnbound // : public some_base_exception
    {
    };

    class ExceptBound // : public some_base_exception
    {
    };


    /* Function Base Class */
    class Funct__
    {
    protected:
      void Bind (Job* j);
      void Hold ();
      void Release ();
      Funct__ ();
      Job* target;
      Private::CrossExec* synchronisation;
    };


    /****************************************************************
     *                    Function 0                                *
     ****************************************************************/

    /* Prototype */
    template <class RET>
    class Function0__ : protected Funct__
    {
    public:
      template <class C>
      void Connect (C* j, RET (C::*callback) ());

      Function0__ ();
    protected:
      RET (Job::*call) ();
    };



    /* deriviate with return value */
    template <class RET>
    class Function0 : public Function0__ <RET>
    {
    public:
      RET operator () ();
    };

    /* deriviate without return value */
    template <>
    class Function0<void> : public Function0__ <void>
    {
    public:
      void operator () ();
    };


    /****************************************************************
     *                    Function 1                                *
     ****************************************************************/

    /* Prototype */
    template <class RET, class P1>
    class Function1__ : protected Funct__
    {
    public:
      template <class C>
      void Connect (C* j, RET (C::*callback) (P1 p1));

      Function1__ ();
    protected:
      RET (Job::*call) (P1);
    };



    /* deriviate with return value */
    template <class RET, class P1>
    class Function1 : public Function1__ <RET, P1>
    {
    public:
      RET operator () (P1 p1);
    };

    /* deriviate without return value */
    template <class P1>
    class Function1<void, P1> : public Function1__ <void, P1>
    {
    public:
      void operator () (P1 p1);
    };

    /****************************************************************
     *                    Function 2                                *
     ****************************************************************/

    /* Prototype */
    template <class RET, class P1, class P2>
    class Function2__ : protected Funct__
    {
    public:
      template <class C>
      void Connect (C* j, RET (C::*callback) (P1 p1, P2 p2));

      Function2__ ();
    protected:
      RET (Job::*call) (P1, P2);
    };



    /* deriviate with return value */
    template <class RET, class P1, class P2>
    class Function2 : public Function2__ <RET, P1, P2>
    {
    public:
      RET operator () (P1 p1, P2 p2);
    };

    /* deriviate without return value */
    template <class P1, class P2>
    class Function2<void, P1, P2> : public Function2__ <void, P1, P2>
    {
    public:
      void operator () (P1 p1, P2 p2);
    };

    /****************************************************************
     *                    Function 3                                *
     ****************************************************************/

    /* Prototype */
    template <class RET, class P1, class P2, class P3>
    class Function3__ : protected Funct__
    {
    public:
      template <class C>
      void Connect (C* j, RET (C::*callback) (P1 p1, P2 p2, P3 p3));

      Function3__ ();
    protected:
      RET (Job::*call) (P1, P2, P3);
    };



    /* deriviate with return value */
    template <class RET, class P1, class P2, class P3>
    class Function3 : public Function3__ <RET, P1, P2, P3>
    {
    public:
      RET operator () (P1 p1, P2 p2, P3 p3);
    };

    /* deriviate without return value */
    template <class P1, class P2, class P3>
    class Function3<void, P1, P2, P3> : public Function3__ <void, P1, P2, P3>
    {
    public:
      void operator () (P1 p1, P2 p2, P3 p3);
    };


  }; // end namespace Threads
}; // end namespace GSMX_utils

#include "Templates/Function.cc"

#endif // THREAD_FUNCTION_HH
