/* Job.hh      (C) by V. Ziegler 2001
 
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THREAD_JOB_HH
#define THREAD_JOB_HH

// Those sigc++/thread guys are somewhat strange... Don't wanna have dummy arrays for pthread structs !
#define SIGC_THREAD_IMPL

#include <sigc++/thread.h>
#include <queue>

#ifndef THREAD_MAX_JOB_CMDS
#define THREAD_MAX_JOB_CMDS 64
#endif

namespace GSMX_utils
{
  namespace Threads
  {
    using namespace SigC::Threads;
    using namespace std;

    namespace Private { class Command; class CMDWait;};

    class ExceptTerminated {};

    class Job : protected Thread
    {
      friend class Funct__;
      friend class Command__;

    public:
      bool IsRunning ();
 
      Job ();
      virtual ~Job ();
 
 
    protected:
      friend class Private :: CMDWait;

      /* Master side */
      void SyncWait ();                             // Command Job to stop after current Loop () returned,
                                                    // and all pending commands have been executed.
                                                    // suspends caller until Jop has stopped.

      void AsyncWait ();                            // Command Job to stop after current Loop () returned,
                                                    // and all pending commands have been executed.
                                                    // does not suspend caller.

      void Ready (bool run_now);                    // to be called by implementation's constructor
 
      /* Master & Slave side */
      void Continue ();                             // Continue cyclic execution of Loop ()
      
      /* Slave side */
      void Wait ();                                 // Stop cyclic execution of Loop () // MUST NOT be called from Master Side !!
      bool IsRequest ();                            // Are there pending commands/function requests ?
      virtual void Loop () = 0;
      
    private:
      void Wait__ ();
      virtual void* main (void* data);
      void PushCmd (Private::Command* cmd);
      volatile bool waiting;
      volatile bool exit;
      volatile bool wait_after_loop;
      queue <Private::Command*> commands;           // queue for pending commands
      Mutex waiting__;                              // locks "waiting"
      Mutex on_request;                             // locked when slave is handling a Request
      Mutex commands__;                             // locks the command queue
      Mutex dtor_ctor;
      Mutex dtor_ctor2;
      //Mutex exited;                                 // handshake for dtor
      
      Condition new_request;
    };


  }; // end namespace Threads
}; // end namespace GSMX_utils

#endif // THREAD_JOB_HH
