/* Command.cc      (C) by V. Ziegler 2001
 
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THREAD_COMMAND_HH
#  error "Never include this File in your code directly"
#  error "This file is automatically included by Command.hh"
#endif

#include "Commands.hh"

namespace GSMX_utils
{
  namespace Threads
  {
    /****************************************************************
     *                    Command 0                                 *
     ****************************************************************/    
    
    template <class C> 
    void  Command0 :: Connect (C* j, void (C::*callback) ())
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (void (Job::*) ()) callback;
      target = j;
    }

    class Command0 :: Recaller : public Private::Command
    {
    public:
      virtual void Execute ();
      Recaller (Command0* creator)
      {
	references = creator;
      }
    protected:
      Command0* references;
    };

    /* --== See Threads.cc for non-template-function code ==-- */


    /****************************************************************
     *                    Command 1                                 *
     ****************************************************************/    
    
    template <class P1> template <class C> 
    void  Command1 <P1>::Connect (C* j, void (C::*callback) (P1 p1))
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (void (Job::*) (P1)) callback;
      target = j;
    }


    template <class P1>
    Command1 <P1>::Command1 ()
    {
      call = 0;
      target = 0;
    }

    template <class P1>
    class Recaller1 : public Private::Command
    {
    public:
      virtual void Execute ();
      Recaller1 (P1 p1, Command1 <P1>* creator)
      {
	a1 = p1;
	references = creator;
      }
    protected:
      P1 a1;
      Command1 <P1>* references;
    };

    template <class P1>
    void Recaller1 <P1> :: Execute ()
    {
      ((references -> target) ->* (references -> call)) (a1);
    };


    template <class P1>
    void Command1 <P1>::operator () (P1 p1)
    {
      if (!call)
	{   
	  ExceptUnbound e;
	  throw e;
	}
      DropCmd (new Recaller1 <P1> (p1, this));
    }


    /****************************************************************
     *                    Command 2                                 *
     ****************************************************************/    
    
    template <class P1, class P2> template <class C> 
    void  Command2 <P1, P2>::Connect (C* j, void (C::*callback) (P1 p1, P2 p2))
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (void (Job::*) (P1, P2)) callback;
      target = j;
    }


    template <class P1, class P2>
    Command2 <P1, P2>::Command2 ()
    {
      call = 0;
      target = 0;
    }

    template <class P1, class P2>
    class Recaller2 : public Private::Command
    {
    public:
      virtual void Execute ();
      Recaller2 (P1 p1, P2 p2, Command2 <P1, P2>* creator)
      {
	a1 = p1;
	a2 = p2;
	references = creator;
      }
    protected:
      P1 a1;
      P2 a2;
      Command2 <P1, P2>* references;
    };

    template <class P1, class P2>
    void Recaller2 <P1, P2> :: Execute ()
    {
      ((references -> target) ->* (references -> call)) (a1, a2);
    };


    template <class P1, class P2>
    void Command2 <P1, P2>::operator () (P1 p1, P2 p2)
    {
      if (!call)
	{   
	  ExceptUnbound e;
	  throw e;
	}
      DropCmd (new Recaller2 <P1, P2> (p1, p2, this));
    }


    /****************************************************************
     *                    Command 3                                 *
     ****************************************************************/    
    
    template <class P1, class P2, class P3> template <class C> 
    void  Command3 <P1, P2, P3>::Connect (C* j, void (C::*callback) (P1 p1, P2 p2, P3 p3))
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (void (Job::*) (P1, P2, P3)) callback;
      target = j;
    }


    template <class P1, class P2, class P3>
    Command3 <P1, P2, P3>::Command3 ()
    {
      call = 0;
      target = 0;
    }

    template <class P1, class P2, class P3>
    class Recaller3 : public Private::Command
    {
    public:
      virtual void Execute ();
      Recaller3 (P1 p1, P2 p2, P3 p3, Command3 <P1, P2, P3>* creator)
      {
	a1 = p1;
	a2 = p2;
	a3 = p3;
	references = creator;
      }
    protected:
      P1 a1;
      P2 a2;
      P3 a3;
      Command3 <P1, P2, P3>* references;
    };

    template <class P1, class P2, class P3>
    void Recaller3 <P1, P2, P3> :: Execute ()
    {
      ((references -> target) ->* (references -> call)) (a1, a2, a3);
    };


    template <class P1, class P2, class P3>
    void Command3 <P1, P2, P3>::operator () (P1 p1, P2 p2, P3 p3)
    {
      if (!call)
	{   
	  ExceptUnbound e;
	  throw e;
	}
      DropCmd (new Recaller3 <P1, P2, P3> (p1, p2, p3, this));
    }

  }; // end namespace Threads
}; // end namespace GSMX_utils
