/* Function.cc      (C) by V. Ziegler 2001
 
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THREAD_FUNCTION_HH
#  error "Never include this File in your code directly"
#  error "This file is automatically included by Function.hh"
#endif

namespace GSMX_utils
{
  namespace Threads
  {

    /****************************************************************
     *                    Function 0                                *
     ****************************************************************/ 
    template <class RET>
    RET Function0<RET>::operator () ()
    {
      if (!call)
	{
	  ExceptUnbound e;
	  throw e;
	}

      Hold ();
      RET retval = (target->*call) ();
      Release ();

      return retval;
    }

    //   template <>
    //   void Function0<void>::operator () ()
    //   {
    //     if (!call)
    //       {
    // 	ExceptUnbound e;
    // 	throw e;
    //       }

    //     Hold ();
    //     (target->*call) ();
    //     Release ();
    //   }

    template <class RET> template <class C> 
    void  Function0__<RET>::Connect (C* j, RET (C::*callback) ())
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (RET (Job::*) ()) callback;
      Bind (j);
    }


    template <class RET>
    Function0__<RET>::Function0__ ()
    {
      call = 0;
    }



    /****************************************************************
     *                    Function 1                                *
     ****************************************************************/ 
    template <class RET, class P1>
    RET Function1<RET, P1>::operator () (P1 p1)
    {
      if (!call)
	{
	  ExceptUnbound e;
	  throw e;
	}

      Hold ();
      RET retval = (target->*call) (p1);
      Release ();

      return retval;
    }

    template <class P1>
    void Function1<void, P1>::operator () (P1 p1)
    {
      if (!call)
	{
	  ExceptUnbound e;
	  throw e;
	}

      Hold ();
      (target->*call) (p1);
      Release ();
    }

    template <class RET, class P1> template <class C> 
    void  Function1__<RET, P1>::Connect (C* j, RET (C::*callback) (P1 p1))
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (RET (Job::*) (P1)) callback;
      Bind (j);
    }


    template <class RET, class P1>
    Function1__<RET, P1>::Function1__ ()
    {
      call = 0;
    }

    /****************************************************************
     *                    Function 2                                *
     ****************************************************************/ 
    template <class RET, class P1, class P2>
    RET Function2<RET, P1, P2>::operator () (P1 p1, P2 p2)
    {
      if (!call)
	{
	  ExceptUnbound e;
	  throw e;
	}

      Hold ();
      RET retval = (target->*call) (p1, p2);
      Release ();

      return retval;
    }

    template <class P1, class P2>
    void Function2<void, P1, P2>::operator () (P1 p1, P2 p2)
    {
      if (!call)
	{
	  ExceptUnbound e;
	  throw e;
	}

      Hold ();
      (target->*call) (p1, p2);
      Release ();
    }

    template <class RET, class P1, class P2> template <class C> 
    void  Function2__<RET, P1, P2>::Connect (C* j, RET (C::*callback) (P1 p1, P2 p2))
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (RET (Job::*) (P1, P2)) callback;
      Bind (j);
    }


    template <class RET, class P1, class P2>
    Function2__<RET, P1, P2>::Function2__ ()
    {
      call = 0;
    }

    /****************************************************************
     *                    Function 3                                *
     ****************************************************************/ 
    template <class RET, class P1, class P2, class P3>
    RET Function3<RET, P1, P2, P3>::operator () (P1 p1, P2 p2, P3 p3)
    {
      if (!call)
	{
	  ExceptUnbound e;
	  throw e;
	}

      Hold ();
      RET retval = (target->*call) (p1, p2, p3);
      Release ();

      return retval;
    }

    template <class P1, class P2, class P3>
    void Function3<void, P1, P2, P3>::operator () (P1 p1, P2 p2, P3 p3)
    {
      if (!call)
	{
	  ExceptUnbound e;
	  throw e;
	}

      Hold ();
      (target->*call) (p1, p2, p3);
      Release ();
    }

    template <class RET, class P1, class P2, class P3> template <class C> 
    void  Function3__<RET, P1, P2, P3>::Connect (C* j, RET (C::*callback) (P1 p1, P2 p2, P3 p3))
    {
      if (call)
	{   
	  ExceptBound e;
	  throw e;
	}

      call = (RET (Job::*) (P1, P2, P3)) callback;
      Bind (j);
    }


    template <class RET, class P1, class P2, class P3>
    Function3__<RET, P1, P2, P3>::Function3__ ()
    {
      call = 0;
    }

  }; // end namespace Threads
}; // end namespace GSMX_utils
