#include "Threads.hh"

#include <unistd.h>
#include <iostream.h>
#include "Commands.hh"

using namespace std;
using namespace SigC;
using namespace SigC::Threads;
using namespace GSMX_utils::Threads;
using namespace GSMX_utils::Threads::Private;

Command0 :: Command0 ()
{
  call = 0;
  target = 0;
}


void Command0 :: Recaller :: Execute ()
{
  ((references -> target) ->* (references -> call)) ();
};


void Command0 :: operator () ()
{
  if (!call)
    {   
      ExceptUnbound e;
      throw e;
    }
  DropCmd (new Command0 :: Recaller (this));
}


bool Job::IsRunning () 
{ 
  MLock lazy (waiting__);
  return !waiting;
}


void Job::Ready (bool run_now) 
{
  waiting = !run_now;
  on_request.unlock (); // signal "Constructors finished"
  dtor_ctor2.lock (); // handshake with slave thread
  // usleep (1000); // hacked fix for ugly Job BUG
}

void Job::SyncWait ()
{
  Private::CrossExec* synchronisation = new Private::CrossExec ();
  PushCmd (synchronisation);
  synchronisation -> Wait ();
  Wait ();
  synchronisation -> Release ();
}

void Job::AsyncWait ()
{
  PushCmd (new Private::CMDWait (this));
}

bool Job::IsRequest ()
{
  MLock lazy (commands__);
  return (!commands.empty ());
}


void Job::Wait__ ()
{
  waiting__.lock ();
  waiting = true;
  new_request.wait (waiting__);
  waiting = false;
  waiting__.unlock ();
}

void Job::Wait ()
{
  wait_after_loop = true;
}

void Job::Continue ()
{
  waiting__.lock ();
  wait_after_loop = false;
  if (waiting)             // This makes shure Continue () works when called outside second thread; 
    new_request.signal ();
  waiting__.unlock ();
}

void* Job::main (void* data)
{
  on_request.lock ();
  dtor_ctor2.lock ();
  dtor_ctor.unlock ();   // signal "Thread is running"
  on_request.lock ();    // Wait for handshake "Constructors finished"
  on_request.unlock ();
  dtor_ctor2.unlock ();  // shake hands back
  if (waiting) Wait__ ();


  // main loop
  while (!exit)
    {
      Private::Command* command = 0;
      
      commands__.lock ();
      if (!commands.empty ())
	{
	  command = commands.front ();
	  commands.pop ();
	}	
      commands__.unlock ();

      if (command)
	{
	  command -> Execute ();
	  delete command;
	}      
      else
	{
	  if (wait_after_loop) 
	    Wait__ ();
	  else
	    Loop ();
	}    
    }
  //exited.unlock ();
  cout << "THREAD-DEBUG: main finished ..." << endl;
}


void Job::PushCmd (Private::Command* cmd)
{
  int pending_commands;

  if (exit)
    {
      ExceptTerminated e;
      throw e;
    }

  do
    {
      MLock lazy (commands__);
      pending_commands = commands.size ();
      if (pending_commands < THREAD_MAX_JOB_CMDS)
	commands.push (cmd);
      else // command queue is flooded ! 
	usleep (10000);
    } while (pending_commands >= THREAD_MAX_JOB_CMDS);
  
  new_request.signal ();
}

Job::Job ()
{
  /* TODO: exceptions */
  wait_after_loop = false;
  waiting = false;
  exit = false;
  dtor_ctor.lock ();
  this -> start (NULL);
  dtor_ctor.lock ();   // Wait for Handshake "Thread is running"
}

Job::~Job ()
{
  //exited.lock ();
  exit = true;
  Continue ();
  //exited.lock ();  // wait fr handshake "Thread exited"
  pthread_join (thread_, NULL);
  cout << "THREAD-DEBUG: DTor finished ..." << endl;
}


CrossExec::CrossExec ()
{
  start.lock ();
  finished.lock ();
}

void CrossExec::Execute ()
{
  start.unlock ();
  finished.lock ();
}

  

Funct__::Funct__ () 
{ 
  // Does nothing, bot Funct__ needs a protected ctor ! 
}

void Funct__::Bind (Job* j)
{
  target = j;
}

void Funct__::Hold ()
{
  synchronisation = new Private::CrossExec ();
  target -> PushCmd (synchronisation);
  synchronisation -> Wait ();
}

void Funct__::Release ()
{
  synchronisation -> Release ();
}

